# 1 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F" 1
!--------------------------------------------------------------------------------------------------!
!   CP2K: A general program to perform molecular dynamics simulations                              !
!   Copyright 2000-2024 CP2K developers group <https://cp2k.org>                                   !
!                                                                                                  !
!   SPDX-License-Identifier: GPL-2.0-or-later                                                      !
!--------------------------------------------------------------------------------------------------!
MODULE callgraph
   USE kinds, ONLY: int_4, int_8
   USE timings_base_type, ONLY: call_stat_type
#include "../base/base_uses.f90"

   IMPLICIT NONE
   PRIVATE

# 1 "/builddir/build/BUILD/cp2k-2024.1/src/common/hash_map.fypp" 1
# 7 "/builddir/build/BUILD/cp2k-2024.1/src/common/hash_map.fypp"
# 428 "/builddir/build/BUILD/cp2k-2024.1/src/common/hash_map.fypp"
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F" 2

! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \brief A hash map (also known as hashtable or dictionary).
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!>        Internally the hash map uses an array to holds its data.
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!>        If this array reaches a load-factor of 75%, a new array with twice the
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!>        size will be allocated and the items are then copied over.
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!>        This ensures that the dictionary will perform operations in O(1).
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \par History
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!>      12.2012 first version [Ole Schuett]
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!>      08.2019 refactored for Fypp [Ole Schuett]
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \author Ole Schuett
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! ***************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      PUBLIC :: callgraph_init
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      PUBLIC :: callgraph_items
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      PUBLIC :: callgraph_haskey
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      PUBLIC :: callgraph_set
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      PUBLIC :: callgraph_get
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      PUBLIC :: callgraph_size
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      PUBLIC :: callgraph_destroy
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      PUBLIC :: callgraph_type
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      PUBLIC :: callgraph_item_type
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!this is an internal type
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!Calculating hashes might be expensive, therefore they are stored
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!for use during change_capacity().
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      TYPE private_item_type
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         PRIVATE
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         INTEGER(kind=int_4), DIMENSION(2)                                :: key  = 0_int_4
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(call_stat_type), POINTER                              :: value  => NULL()
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         INTEGER(KIND=int_8)                         :: hash = 0_int_8
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(private_item_type), POINTER            :: next => Null()
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      END TYPE private_item_type
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!this is an internal type
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      TYPE private_item_p_type
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         PRIVATE
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(private_item_type), POINTER           :: p => Null()
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      END TYPE private_item_p_type
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! this is the public type, which holds a hash map instance
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      TYPE callgraph_type
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         PRIVATE
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(private_item_p_type), DIMENSION(:), POINTER      :: buckets => Null()
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         INTEGER                                               :: size = -1
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      END TYPE callgraph_type
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! this is a public type, its returned by  callgraph_items()
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      TYPE callgraph_item_type
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         INTEGER(kind=int_4), DIMENSION(2)        :: key  = 0_int_4
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(call_stat_type), POINTER      :: value  => NULL()
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      END TYPE callgraph_item_type
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      CONTAINS
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \brief Allocates the internal data-structures of the given hash map.
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param hash_map ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param initial_capacity The initial size of the internal array (default=11).
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \author Ole Schuett
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      SUBROUTINE callgraph_init(hash_map, initial_capacity)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(callgraph_type), INTENT(inout)                 :: hash_map
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         INTEGER, INTENT(in), OPTIONAL                             :: initial_capacity
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         INTEGER :: initial_capacity_
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         IF (PRESENT(initial_capacity)) THEN
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            initial_capacity_ = initial_capacity
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         ELSE
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            initial_capacity_ = 11
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         END IF
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         IF (initial_capacity_ < 1) &
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            CPABORT("initial_capacity < 1")
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         IF (ASSOCIATED(hash_map%buckets)) &
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            CPABORT("hash map is already initialized.")
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         ALLOCATE (hash_map%buckets(initial_capacity_))
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         hash_map%size = 0
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      END SUBROUTINE callgraph_init
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \brief Test if the given hash map has been initialized.
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param hash_map ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \return ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \author Ole Schuett
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      FUNCTION callgraph_isready(hash_map) RESULT(res)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(callgraph_type), INTENT(inout)             :: hash_map
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         LOGICAL                                               :: res
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         res = ASSOCIATED(hash_map%buckets)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      END FUNCTION callgraph_isready
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \brief Deallocated the internal data-structures if the given hash map.
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!>        Caution: If the stored keys or values are pointers, their targets will
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!>                 not get deallocated by this routine.
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param hash_map ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \author Ole Schuett
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      SUBROUTINE callgraph_destroy(hash_map)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(callgraph_type), INTENT(inout)  :: hash_map
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(private_item_type), POINTER           :: item, prev_item
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         INTEGER :: i
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         CPASSERT(ASSOCIATED(hash_map%buckets))
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         DO i = 1, size(hash_map%buckets)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            item => hash_map%buckets(i)%p
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            DO WHILE (ASSOCIATED(item))
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
               prev_item => item
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
               item => item%next
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
               DEALLOCATE (prev_item)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            END DO
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         END DO
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         DEALLOCATE (hash_map%buckets)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         hash_map%size = -1
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      END SUBROUTINE callgraph_destroy
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \brief Stores, and possibly overwrites, a given value under a given key.
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param hash_map ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param key ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param value ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \author Ole Schuett
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      SUBROUTINE callgraph_set(hash_map, key, value)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(callgraph_type), INTENT(inout)             :: hash_map
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         INTEGER(kind=int_4), DIMENSION(2), INTENT(in)                            :: key
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(call_stat_type), POINTER, INTENT(in)                            :: value
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         INTEGER(KIND=int_8)                                   :: hash
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         CPASSERT(ASSOCIATED(hash_map%buckets))
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         hash = callgraph_hash_function(key)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         CALL callgraph_set_hashed(hash_map, key, value, hash)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      END SUBROUTINE callgraph_set
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \brief Common code used internally by callgraph_set() and callgraph_change_capacity().
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param hash_map ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param key ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param value ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param hash ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \author Ole Schuett
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      RECURSIVE SUBROUTINE callgraph_set_hashed(hash_map, key, value, hash)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(callgraph_type), INTENT(inout)               :: hash_map
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         INTEGER(kind=int_4), DIMENSION(2), intent(in)                              :: key
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(call_stat_type), POINTER, intent(in)                              :: value
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         INTEGER(KIND=int_8), intent(in)                         :: hash
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(private_item_type), POINTER                        :: item, new_item
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         INTEGER(KIND=int_8)                                     :: idx
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         idx = MOD(hash, INT(size(hash_map%buckets), KIND=int_8)) + 1
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         ! if already in hash map just update its value
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         item => hash_map%buckets(idx)%p
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         DO WHILE (ASSOCIATED(item))
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            IF (item%hash == hash) THEN
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
               IF (callgraph_keys_equal(item%key, key)) THEN
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
                  item%value =>value
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
                  RETURN
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
               END IF
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            END IF
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            item => item%next
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         END DO
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         ! check load-factor
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         IF (4*hash_map%size > 3*size(hash_map%buckets)) THEN ! load-factor > 75%
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            call callgraph_change_capacity(hash_map, 2*size(hash_map%buckets)) !double capacity
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            idx = MOD(hash, INT(size(hash_map%buckets), KIND=int_8)) + 1
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         END IF
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         ! create a new item
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         allocate (new_item)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         new_item%hash = hash
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         new_item%key =key
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         new_item%value =>value
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         new_item%next => hash_map%buckets(idx)%p
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         hash_map%buckets(idx)%p => new_item
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         hash_map%size = hash_map%size + 1
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      END SUBROUTINE callgraph_set_hashed
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \brief Internal routine for changing the hash map's capacity.
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param hash_map ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param new_capacity ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \author Ole Schuett
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      RECURSIVE SUBROUTINE callgraph_change_capacity(hash_map, new_capacity)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(callgraph_type), INTENT(inout)         :: hash_map
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         INTEGER, INTENT(in)                               :: new_capacity
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         INTEGER                                           :: i, old_size, new_cap
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(private_item_type), POINTER                  :: item, prev_item
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(private_item_p_type), DIMENSION(:), POINTER  :: old_buckets
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         new_cap = new_capacity
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         ! pre checks
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         IF (new_cap > HUGE(i)) THEN
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            IF (size(hash_map%buckets) == HUGE(i)) RETURN ! reached maximum - stay there.
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            new_cap = HUGE(i) ! grow as far as possible
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         END IF
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         CPASSERT(new_cap >= 1)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         CPASSERT(4*hash_map%size < 3*new_cap)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         old_size = hash_map%size
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         old_buckets => hash_map%buckets
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         ALLOCATE (hash_map%buckets(new_capacity))
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         hash_map%size = 0
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         DO i = 1, size(old_buckets)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            item => old_buckets(i)%p
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            DO WHILE (ASSOCIATED(item))
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
               CALL callgraph_set_hashed(hash_map, item%key, item%value, item%hash)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
               prev_item => item
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
               item => item%next
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
               DEALLOCATE (prev_item)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            END DO
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         END DO
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         DEALLOCATE (old_buckets)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         CPASSERT(old_size == hash_map%size)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      END SUBROUTINE callgraph_change_capacity
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \brief Gets a value for a given key from the hash map.
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!>        If the key is not found the default_value will be returned.
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!>        If the key is not found and default_value was not provided the program stops.
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param hash_map ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param key ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param default_value ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \return ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \author Ole Schuett
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      FUNCTION callgraph_get(hash_map, key, default_value) RESULT(value)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(callgraph_type), INTENT(in)              :: hash_map
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         INTEGER(kind=int_4), DIMENSION(2), INTENT(in)                            :: key
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(call_stat_type), POINTER, INTENT(in), OPTIONAL                :: default_value
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(call_stat_type), POINTER                                      :: value
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(private_item_type), POINTER                    :: item
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         INTEGER(KIND=int_8)                                 :: hash, idx
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         CPASSERT(ASSOCIATED(hash_map%buckets))
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         hash = callgraph_hash_function(key)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         idx = MOD(hash, INT(size(hash_map%buckets), KIND=int_8)) + 1
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         item => hash_map%buckets(idx)%p
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         DO WHILE (ASSOCIATED(item))
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            IF (item%hash == hash) THEN
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
               IF (callgraph_keys_equal(item%key, key)) THEN
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
                  value =>item%value
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
                  RETURN
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
               END IF
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            END IF
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            item => item%next
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         END DO
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         IF (PRESENT(default_value)) THEN
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            value =>default_value
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            RETURN
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         END IF
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         CPABORT("Key not found.")
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      END FUNCTION callgraph_get
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \brief Remove the value for a given key from the hash map.
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!>        If the key is not found the program stops.
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param hash_map ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param key ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \author Ole Schuett
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      SUBROUTINE callgraph_del(hash_map, key)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(callgraph_type), INTENT(inout)               :: hash_map
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         INTEGER(kind=int_4), DIMENSION(2), INTENT(in)                                :: key
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(private_item_type), POINTER                        :: item, prev_item
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         INTEGER(KIND=int_8)                                     :: hash, idx
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         CPASSERT(ASSOCIATED(hash_map%buckets))
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         hash = callgraph_hash_function(key)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         idx = MOD(hash, INT(size(hash_map%buckets), KIND=int_8)) + 1
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         item => hash_map%buckets(idx)%p
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         prev_item => Null()
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         DO WHILE (ASSOCIATED(item))
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            IF (item%hash == hash) THEN
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
               IF (callgraph_keys_equal(item%key, key)) THEN
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
                  IF (ASSOCIATED(prev_item)) THEN
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
                     prev_item%next => item%next
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
                  ELSE
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
                     hash_map%buckets(idx)%p => item%next
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
                  END IF
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
                  DEALLOCATE (item)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
                  hash_map%size = hash_map%size - 1
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
                  RETURN
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
               END IF
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            END IF
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            prev_item => item
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            item => item%next
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         END DO
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         CPABORT("Key not found.")
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      END SUBROUTINE callgraph_del
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \brief Returns the number of key/value-items currently stored in the hash map.
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param hash_map ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \return ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \author Ole Schuett
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      FUNCTION callgraph_size(hash_map) RESULT(size)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(callgraph_type), INTENT(IN)  :: hash_map
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         INTEGER                                 :: size
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         CPASSERT(ASSOCIATED(hash_map%buckets))
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         size = hash_map%size
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      END FUNCTION callgraph_size
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \brief Checks whether a given key is currently stored in the hash_map.
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param hash_map ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param key ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \return ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \author Ole Schuett
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      FUNCTION callgraph_haskey(hash_map, key) RESULT(res)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(callgraph_type), INTENT(IN)                :: hash_map
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         INTEGER(kind=int_4), DIMENSION(2), INTENT(IN)                              :: key
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         LOGICAL                                               :: res
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(private_item_type), POINTER                      :: item
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         INTEGER(KIND=int_8)                                   :: hash, idx
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         CPASSERT(ASSOCIATED(hash_map%buckets))
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         res = .FALSE.
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         IF (hash_map%size == 0) RETURN
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         hash = callgraph_hash_function(key)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         idx = MOD(hash, INT(size(hash_map%buckets), KIND=int_8)) + 1
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         item => hash_map%buckets(idx)%p
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         DO WHILE (ASSOCIATED(item))
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            IF (item%hash == hash) THEN
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
               IF (callgraph_keys_equal(item%key, key)) THEN
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
                  res = .TRUE.
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
                  return
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
               END IF
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            END IF
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            item => item%next
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         END DO
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      END FUNCTION callgraph_haskey
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \brief Returns a pointer to an array of all key/value-items stored in the hash map.
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!>        Caution: The caller is responsible for deallocating targeted array after usage.
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param hash_map ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \return ...
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \author Ole Schuett
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      FUNCTION callgraph_items(hash_map) RESULT(items)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(callgraph_type), INTENT(IN)                 :: hash_map
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(callgraph_item_type), DIMENSION(:), POINTER :: items
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(private_item_type), POINTER  :: item
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         INTEGER :: i, j
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         CPASSERT(ASSOCIATED(hash_map%buckets))
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         ALLOCATE (items(hash_map%size))
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         j = 1
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         DO i = 1, size(hash_map%buckets)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            item => hash_map%buckets(i)%p
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            DO WHILE (ASSOCIATED(item))
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
               items(j)%key =item%key
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
               items(j)%value =>item%value
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
               j = j + 1
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
               item => item%next
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            END DO
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         END DO
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         CPASSERT(j == hash_map%size + 1)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      END FUNCTION callgraph_items
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \brief Copies all key/values-items from one hash map to another.
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!>        Afterwards hash_map will contain all items from the from_hash_map and
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!>        additionally all its previous items, which were not overwritten.
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!>        The two hash maps have to be of the same type.
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param hash_map destination of items
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \param from_hash_map source of items - will not be change
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
!> \author Ole Schuett
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
! **************************************************************************************************
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      SUBROUTINE callgraph_update(hash_map, from_hash_map)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(callgraph_type), INTENT(inout)              :: hash_map
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(callgraph_type), INTENT(in)                 :: from_hash_map
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         TYPE(callgraph_item_type), DIMENSION(:), POINTER :: from_items
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         INTEGER :: i
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         CPASSERT(ASSOCIATED(hash_map%buckets))
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         CPASSERT(ASSOCIATED(from_hash_map%buckets))
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         from_items => callgraph_items(from_hash_map)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         DO i = 1, size(from_items)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
            CALL callgraph_set(hash_map, from_items(i)%key, from_items(i)%value)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         END DO
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
         DEALLOCATE (from_items)
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"
      END SUBROUTINE callgraph_update
# 17 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

# 23 "/builddir/build/BUILD/cp2k-2024.1/src/common/callgraph.F"

! **************************************************************************************************
!> \brief ...
!> \param key ...
!> \return ...
! **************************************************************************************************
   PURE FUNCTION callgraph_hash_function(key) RESULT(hash)
      INTEGER(kind=int_4), DIMENSION(2), INTENT(in)      :: key
      INTEGER(KIND=int_8)                                :: hash

      INTEGER(kind=int_8)                                :: k1, k2
      k1 = key(1) ! cast to int_8
      k2 = key(2)
      hash = IOR(k1, ISHFT(k2, 32))
   END FUNCTION callgraph_hash_function

! **************************************************************************************************
!> \brief ...
!> \param key ...
!> \return ...
! **************************************************************************************************
   PURE FUNCTION callgraph_keys_equal(key1, key2) RESULT(res)
      INTEGER(kind=int_4), DIMENSION(2), INTENT(in)      :: key1, key2
      LOGICAL                                            :: res

      res = ALL(key1 == key2)
   END FUNCTION callgraph_keys_equal

END MODULE callgraph
