# 1 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F" 1
!--------------------------------------------------------------------------------------------------!
!   CP2K: A general program to perform molecular dynamics simulations                              !
!   Copyright 2000-2024 CP2K developers group <https://cp2k.org>                                   !
!                                                                                                  !
!   SPDX-License-Identifier: GPL-2.0-or-later                                                      !
!--------------------------------------------------------------------------------------------------!

! **************************************************************************************************
!> \brief Routine for sorting an array
!> \note
!>      CP2K:
!>      Please use the interface defined in util.F for calling sort().
!>
!>      DBCSR:
!>      Please use the interface defined in dbcsr_toollib.F for calling sort().
!> \par History
!>      12.2012 first version [ole]
!> \author Ole Schuett
! **************************************************************************************************
MODULE cp_array_sort

   USE kinds, ONLY: sp, dp, int_4, int_8

#include "../base/base_uses.f90"

# 1 "/builddir/build/BUILD/cp2k-2024.1/src/common/array_sort.fypp" 1
# 7 "/builddir/build/BUILD/cp2k-2024.1/src/common/array_sort.fypp"
# 132 "/builddir/build/BUILD/cp2k-2024.1/src/common/array_sort.fypp"
# 27 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F" 2

   IMPLICIT NONE
   PRIVATE

   PUBLIC :: cp_1d_s_sort, cp_1d_r_sort, cp_1d_i4_sort, cp_1d_i8_sort

CONTAINS


# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
! **************************************************************************************************
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \brief Sorts an array inplace using a combination of merge- and bubble-sort.
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!>        It also returns the indices, which the elements had before the sort.
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param arr the array to sort
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param n length of array
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param indices returns elements-indices before the sort
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \par History
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!>      12.2012 created [ole]
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \author Ole Schuett
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
! **************************************************************************************************
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
      SUBROUTINE cp_1d_s_sort(arr, n, indices)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER, INTENT(IN)                      :: n
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         REAL(kind=sp), DIMENSION(1:n), INTENT(INOUT)  :: arr
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         integer, DIMENSION(1:n), INTENT(INOUT)   :: indices
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER                                  :: i
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         REAL(kind=sp), ALLOCATABLE                    :: tmp_arr(:)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER, ALLOCATABLE                     :: tmp_idx(:)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         IF (n > 1) THEN
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! scratch space used during the merge step
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ALLOCATE (tmp_arr((n + 1)/2), tmp_idx((n + 1)/2))
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            indices = (/(i, i=1, n)/)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            CALL cp_1d_s_sort_low(arr(1:n), indices, tmp_arr, tmp_idx)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            DEALLOCATE (tmp_arr, tmp_idx)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ELSE IF (n > 0) THEN ! can be a frequent case in cp2k
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            indices(1) = 1
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         END IF
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
      END SUBROUTINE cp_1d_s_sort
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
! **************************************************************************************************
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \brief The actual sort routing. Only cp_1d_s_sort and itself should call this.
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param arr the array to sort
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param indices elements-indices before the sort
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param tmp_arr scratch space
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param tmp_idx scratch space
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \par History
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!>      12.2012 created [ole]
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \author Ole Schuett
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
! **************************************************************************************************
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
      RECURSIVE SUBROUTINE cp_1d_s_sort_low(arr, indices, tmp_arr, tmp_idx)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         REAL(kind=sp), DIMENSION(:), INTENT(INOUT)                 :: arr
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER, DIMENSION(size(arr)), INTENT(INOUT)          :: indices
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         REAL(kind=sp), DIMENSION((size(arr) + 1)/2), INTENT(INOUT)   :: tmp_arr
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER, DIMENSION((size(arr) + 1)/2), INTENT(INOUT)    :: tmp_idx
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         REAL(kind=sp)                                              :: a
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER                                               :: t, m, i, j, k
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         LOGICAL                                               :: swapped
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ! a,t:  used during swaping of elements in arr and indices
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         swapped = .TRUE.
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ! If only a few elements are left we switch to bubble-sort for efficiency.
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         IF (size(arr) <= 7) THEN ! 7 seems to be a good choice for the moment
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            DO j = size(arr) - 1, 1, -1
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               swapped = .FALSE.
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               DO i = 1, j
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  IF (cp_1d_s_less_than(arr(i + 1), arr(i))) THEN
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     ! swap arr(i) with arr(i+1)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     a = arr(i)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     arr(i) = arr(i + 1)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     arr(i + 1) = a
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     ! swap indices(i) with indices(i+1)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     t = indices(i)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     indices(i) = indices(i + 1)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     indices(i + 1) = t
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     swapped = .TRUE.
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  END IF
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               END DO
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               IF (.NOT. swapped) EXIT
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            END DO
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            RETURN
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         END IF
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ! split list in half and recursively sort both sublists
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         m = (size(arr) + 1)/2 ! index where we're going to divide the list in two
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         CALL cp_1d_s_sort_low(arr(1:m), indices(1:m), tmp_arr, tmp_idx)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         CALL cp_1d_s_sort_low(arr(m + 1:), indices(m + 1:), tmp_arr, tmp_idx)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ! Check for a special case: Can we just concate the two sorted sublists?
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ! This leads to O(n) scaling if the input is already sorted.
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         IF (cp_1d_s_less_than(arr(m + 1), arr(m))) THEN
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! ...no - let's merge the two sorted sublists arr(:m) and arr(m+1:)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! Merge will be performed directly in arr. Need backup of first sublist.
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            tmp_arr(1:m) = arr(1:m)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            tmp_idx(1:m) = indices(1:m)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            i = 1; ! number of elemens consumed from 1st sublist
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            j = 1; ! number of elemens consumed from 2nd sublist
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            k = 1; ! number of elemens already merged
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            DO WHILE (i <= m .and. j <= size(arr) - m)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               IF (cp_1d_s_less_than(arr(m + j), tmp_arr(i))) THEN
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  arr(k) = arr(m + j)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  indices(k) = indices(m + j)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  j = j + 1
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               ELSE
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  arr(k) = tmp_arr(i)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  indices(k) = tmp_idx(i)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  i = i + 1
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               END IF
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               k = k + 1
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            END DO
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! One of the two sublist is now empty.
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! Copy possibly remaining tail of 1st sublist
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            DO WHILE (i <= m)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               arr(k) = tmp_arr(i)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               indices(k) = tmp_idx(i)
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               i = i + 1
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               k = k + 1
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            END DO
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! The possibly remaining tail of 2nd sublist is already at the right spot.
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         END IF
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
      END SUBROUTINE cp_1d_s_sort_low
# 35 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 37 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

   PURE FUNCTION cp_1d_s_less_than(a, b) RESULT(res)
      REAL(kind=sp), INTENT(IN) :: a, b
      LOGICAL                   :: res
      res = a < b
   END FUNCTION cp_1d_s_less_than


# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
! **************************************************************************************************
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \brief Sorts an array inplace using a combination of merge- and bubble-sort.
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!>        It also returns the indices, which the elements had before the sort.
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param arr the array to sort
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param n length of array
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param indices returns elements-indices before the sort
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \par History
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!>      12.2012 created [ole]
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \author Ole Schuett
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
! **************************************************************************************************
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
      SUBROUTINE cp_1d_r_sort(arr, n, indices)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER, INTENT(IN)                      :: n
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         REAL(kind=dp), DIMENSION(1:n), INTENT(INOUT)  :: arr
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         integer, DIMENSION(1:n), INTENT(INOUT)   :: indices
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER                                  :: i
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         REAL(kind=dp), ALLOCATABLE                    :: tmp_arr(:)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER, ALLOCATABLE                     :: tmp_idx(:)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         IF (n > 1) THEN
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! scratch space used during the merge step
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ALLOCATE (tmp_arr((n + 1)/2), tmp_idx((n + 1)/2))
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            indices = (/(i, i=1, n)/)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            CALL cp_1d_r_sort_low(arr(1:n), indices, tmp_arr, tmp_idx)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            DEALLOCATE (tmp_arr, tmp_idx)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ELSE IF (n > 0) THEN ! can be a frequent case in cp2k
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            indices(1) = 1
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         END IF
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
      END SUBROUTINE cp_1d_r_sort
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
! **************************************************************************************************
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \brief The actual sort routing. Only cp_1d_r_sort and itself should call this.
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param arr the array to sort
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param indices elements-indices before the sort
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param tmp_arr scratch space
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param tmp_idx scratch space
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \par History
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!>      12.2012 created [ole]
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \author Ole Schuett
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
! **************************************************************************************************
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
      RECURSIVE SUBROUTINE cp_1d_r_sort_low(arr, indices, tmp_arr, tmp_idx)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         REAL(kind=dp), DIMENSION(:), INTENT(INOUT)                 :: arr
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER, DIMENSION(size(arr)), INTENT(INOUT)          :: indices
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         REAL(kind=dp), DIMENSION((size(arr) + 1)/2), INTENT(INOUT)   :: tmp_arr
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER, DIMENSION((size(arr) + 1)/2), INTENT(INOUT)    :: tmp_idx
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         REAL(kind=dp)                                              :: a
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER                                               :: t, m, i, j, k
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         LOGICAL                                               :: swapped
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ! a,t:  used during swaping of elements in arr and indices
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         swapped = .TRUE.
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ! If only a few elements are left we switch to bubble-sort for efficiency.
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         IF (size(arr) <= 7) THEN ! 7 seems to be a good choice for the moment
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            DO j = size(arr) - 1, 1, -1
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               swapped = .FALSE.
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               DO i = 1, j
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  IF (cp_1d_r_less_than(arr(i + 1), arr(i))) THEN
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     ! swap arr(i) with arr(i+1)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     a = arr(i)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     arr(i) = arr(i + 1)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     arr(i + 1) = a
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     ! swap indices(i) with indices(i+1)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     t = indices(i)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     indices(i) = indices(i + 1)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     indices(i + 1) = t
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     swapped = .TRUE.
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  END IF
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               END DO
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               IF (.NOT. swapped) EXIT
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            END DO
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            RETURN
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         END IF
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ! split list in half and recursively sort both sublists
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         m = (size(arr) + 1)/2 ! index where we're going to divide the list in two
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         CALL cp_1d_r_sort_low(arr(1:m), indices(1:m), tmp_arr, tmp_idx)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         CALL cp_1d_r_sort_low(arr(m + 1:), indices(m + 1:), tmp_arr, tmp_idx)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ! Check for a special case: Can we just concate the two sorted sublists?
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ! This leads to O(n) scaling if the input is already sorted.
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         IF (cp_1d_r_less_than(arr(m + 1), arr(m))) THEN
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! ...no - let's merge the two sorted sublists arr(:m) and arr(m+1:)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! Merge will be performed directly in arr. Need backup of first sublist.
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            tmp_arr(1:m) = arr(1:m)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            tmp_idx(1:m) = indices(1:m)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            i = 1; ! number of elemens consumed from 1st sublist
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            j = 1; ! number of elemens consumed from 2nd sublist
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            k = 1; ! number of elemens already merged
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            DO WHILE (i <= m .and. j <= size(arr) - m)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               IF (cp_1d_r_less_than(arr(m + j), tmp_arr(i))) THEN
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  arr(k) = arr(m + j)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  indices(k) = indices(m + j)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  j = j + 1
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               ELSE
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  arr(k) = tmp_arr(i)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  indices(k) = tmp_idx(i)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  i = i + 1
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               END IF
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               k = k + 1
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            END DO
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! One of the two sublist is now empty.
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! Copy possibly remaining tail of 1st sublist
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            DO WHILE (i <= m)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               arr(k) = tmp_arr(i)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               indices(k) = tmp_idx(i)
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               i = i + 1
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               k = k + 1
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            END DO
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! The possibly remaining tail of 2nd sublist is already at the right spot.
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         END IF
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
      END SUBROUTINE cp_1d_r_sort_low
# 44 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 46 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

   PURE FUNCTION cp_1d_r_less_than(a, b) RESULT(res)
      REAL(kind=dp), INTENT(IN) :: a, b
      LOGICAL                   :: res
      res = a < b
   END FUNCTION cp_1d_r_less_than


# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
! **************************************************************************************************
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \brief Sorts an array inplace using a combination of merge- and bubble-sort.
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!>        It also returns the indices, which the elements had before the sort.
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param arr the array to sort
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param n length of array
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param indices returns elements-indices before the sort
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \par History
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!>      12.2012 created [ole]
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \author Ole Schuett
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
! **************************************************************************************************
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
      SUBROUTINE cp_1d_i4_sort(arr, n, indices)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER, INTENT(IN)                      :: n
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER(kind=int_4), DIMENSION(1:n), INTENT(INOUT)  :: arr
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         integer, DIMENSION(1:n), INTENT(INOUT)   :: indices
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER                                  :: i
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER(kind=int_4), ALLOCATABLE                    :: tmp_arr(:)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER, ALLOCATABLE                     :: tmp_idx(:)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         IF (n > 1) THEN
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! scratch space used during the merge step
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ALLOCATE (tmp_arr((n + 1)/2), tmp_idx((n + 1)/2))
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            indices = (/(i, i=1, n)/)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            CALL cp_1d_i4_sort_low(arr(1:n), indices, tmp_arr, tmp_idx)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            DEALLOCATE (tmp_arr, tmp_idx)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ELSE IF (n > 0) THEN ! can be a frequent case in cp2k
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            indices(1) = 1
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         END IF
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
      END SUBROUTINE cp_1d_i4_sort
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
! **************************************************************************************************
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \brief The actual sort routing. Only cp_1d_i4_sort and itself should call this.
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param arr the array to sort
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param indices elements-indices before the sort
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param tmp_arr scratch space
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param tmp_idx scratch space
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \par History
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!>      12.2012 created [ole]
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \author Ole Schuett
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
! **************************************************************************************************
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
      RECURSIVE SUBROUTINE cp_1d_i4_sort_low(arr, indices, tmp_arr, tmp_idx)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER(kind=int_4), DIMENSION(:), INTENT(INOUT)                 :: arr
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER, DIMENSION(size(arr)), INTENT(INOUT)          :: indices
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER(kind=int_4), DIMENSION((size(arr) + 1)/2), INTENT(INOUT)   :: tmp_arr
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER, DIMENSION((size(arr) + 1)/2), INTENT(INOUT)    :: tmp_idx
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER(kind=int_4)                                              :: a
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER                                               :: t, m, i, j, k
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         LOGICAL                                               :: swapped
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ! a,t:  used during swaping of elements in arr and indices
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         swapped = .TRUE.
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ! If only a few elements are left we switch to bubble-sort for efficiency.
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         IF (size(arr) <= 7) THEN ! 7 seems to be a good choice for the moment
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            DO j = size(arr) - 1, 1, -1
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               swapped = .FALSE.
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               DO i = 1, j
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  IF (cp_1d_i4_less_than(arr(i + 1), arr(i))) THEN
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     ! swap arr(i) with arr(i+1)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     a = arr(i)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     arr(i) = arr(i + 1)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     arr(i + 1) = a
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     ! swap indices(i) with indices(i+1)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     t = indices(i)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     indices(i) = indices(i + 1)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     indices(i + 1) = t
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     swapped = .TRUE.
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  END IF
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               END DO
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               IF (.NOT. swapped) EXIT
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            END DO
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            RETURN
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         END IF
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ! split list in half and recursively sort both sublists
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         m = (size(arr) + 1)/2 ! index where we're going to divide the list in two
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         CALL cp_1d_i4_sort_low(arr(1:m), indices(1:m), tmp_arr, tmp_idx)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         CALL cp_1d_i4_sort_low(arr(m + 1:), indices(m + 1:), tmp_arr, tmp_idx)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ! Check for a special case: Can we just concate the two sorted sublists?
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ! This leads to O(n) scaling if the input is already sorted.
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         IF (cp_1d_i4_less_than(arr(m + 1), arr(m))) THEN
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! ...no - let's merge the two sorted sublists arr(:m) and arr(m+1:)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! Merge will be performed directly in arr. Need backup of first sublist.
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            tmp_arr(1:m) = arr(1:m)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            tmp_idx(1:m) = indices(1:m)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            i = 1; ! number of elemens consumed from 1st sublist
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            j = 1; ! number of elemens consumed from 2nd sublist
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            k = 1; ! number of elemens already merged
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            DO WHILE (i <= m .and. j <= size(arr) - m)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               IF (cp_1d_i4_less_than(arr(m + j), tmp_arr(i))) THEN
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  arr(k) = arr(m + j)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  indices(k) = indices(m + j)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  j = j + 1
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               ELSE
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  arr(k) = tmp_arr(i)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  indices(k) = tmp_idx(i)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  i = i + 1
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               END IF
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               k = k + 1
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            END DO
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! One of the two sublist is now empty.
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! Copy possibly remaining tail of 1st sublist
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            DO WHILE (i <= m)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               arr(k) = tmp_arr(i)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               indices(k) = tmp_idx(i)
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               i = i + 1
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               k = k + 1
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            END DO
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! The possibly remaining tail of 2nd sublist is already at the right spot.
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         END IF
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
      END SUBROUTINE cp_1d_i4_sort_low
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 55 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

   PURE FUNCTION cp_1d_i4_less_than(a, b) RESULT(res)
      INTEGER(kind=int_4), INTENT(IN) :: a, b
      LOGICAL                         :: res
      res = a < b
   END FUNCTION cp_1d_i4_less_than


# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
! **************************************************************************************************
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \brief Sorts an array inplace using a combination of merge- and bubble-sort.
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!>        It also returns the indices, which the elements had before the sort.
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param arr the array to sort
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param n length of array
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param indices returns elements-indices before the sort
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \par History
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!>      12.2012 created [ole]
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \author Ole Schuett
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
! **************************************************************************************************
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
      SUBROUTINE cp_1d_i8_sort(arr, n, indices)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER, INTENT(IN)                      :: n
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER(kind=int_8), DIMENSION(1:n), INTENT(INOUT)  :: arr
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         integer, DIMENSION(1:n), INTENT(INOUT)   :: indices
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER                                  :: i
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER(kind=int_8), ALLOCATABLE                    :: tmp_arr(:)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER, ALLOCATABLE                     :: tmp_idx(:)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         IF (n > 1) THEN
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! scratch space used during the merge step
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ALLOCATE (tmp_arr((n + 1)/2), tmp_idx((n + 1)/2))
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            indices = (/(i, i=1, n)/)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            CALL cp_1d_i8_sort_low(arr(1:n), indices, tmp_arr, tmp_idx)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            DEALLOCATE (tmp_arr, tmp_idx)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ELSE IF (n > 0) THEN ! can be a frequent case in cp2k
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            indices(1) = 1
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         END IF
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
      END SUBROUTINE cp_1d_i8_sort
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
! **************************************************************************************************
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \brief The actual sort routing. Only cp_1d_i8_sort and itself should call this.
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param arr the array to sort
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param indices elements-indices before the sort
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param tmp_arr scratch space
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \param tmp_idx scratch space
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \par History
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!>      12.2012 created [ole]
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
!> \author Ole Schuett
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
! **************************************************************************************************
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
      RECURSIVE SUBROUTINE cp_1d_i8_sort_low(arr, indices, tmp_arr, tmp_idx)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER(kind=int_8), DIMENSION(:), INTENT(INOUT)                 :: arr
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER, DIMENSION(size(arr)), INTENT(INOUT)          :: indices
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER(kind=int_8), DIMENSION((size(arr) + 1)/2), INTENT(INOUT)   :: tmp_arr
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER, DIMENSION((size(arr) + 1)/2), INTENT(INOUT)    :: tmp_idx
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER(kind=int_8)                                              :: a
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         INTEGER                                               :: t, m, i, j, k
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         LOGICAL                                               :: swapped
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ! a,t:  used during swaping of elements in arr and indices
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         swapped = .TRUE.
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ! If only a few elements are left we switch to bubble-sort for efficiency.
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         IF (size(arr) <= 7) THEN ! 7 seems to be a good choice for the moment
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            DO j = size(arr) - 1, 1, -1
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               swapped = .FALSE.
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               DO i = 1, j
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  IF (cp_1d_i8_less_than(arr(i + 1), arr(i))) THEN
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     ! swap arr(i) with arr(i+1)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     a = arr(i)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     arr(i) = arr(i + 1)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     arr(i + 1) = a
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     ! swap indices(i) with indices(i+1)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     t = indices(i)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     indices(i) = indices(i + 1)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     indices(i + 1) = t
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                     swapped = .TRUE.
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  END IF
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               END DO
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               IF (.NOT. swapped) EXIT
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            END DO
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            RETURN
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         END IF
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ! split list in half and recursively sort both sublists
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         m = (size(arr) + 1)/2 ! index where we're going to divide the list in two
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         CALL cp_1d_i8_sort_low(arr(1:m), indices(1:m), tmp_arr, tmp_idx)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         CALL cp_1d_i8_sort_low(arr(m + 1:), indices(m + 1:), tmp_arr, tmp_idx)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ! Check for a special case: Can we just concate the two sorted sublists?
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         ! This leads to O(n) scaling if the input is already sorted.
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         IF (cp_1d_i8_less_than(arr(m + 1), arr(m))) THEN
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! ...no - let's merge the two sorted sublists arr(:m) and arr(m+1:)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! Merge will be performed directly in arr. Need backup of first sublist.
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            tmp_arr(1:m) = arr(1:m)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            tmp_idx(1:m) = indices(1:m)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            i = 1; ! number of elemens consumed from 1st sublist
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            j = 1; ! number of elemens consumed from 2nd sublist
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            k = 1; ! number of elemens already merged
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            DO WHILE (i <= m .and. j <= size(arr) - m)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               IF (cp_1d_i8_less_than(arr(m + j), tmp_arr(i))) THEN
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  arr(k) = arr(m + j)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  indices(k) = indices(m + j)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  j = j + 1
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               ELSE
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  arr(k) = tmp_arr(i)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  indices(k) = tmp_idx(i)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
                  i = i + 1
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               END IF
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               k = k + 1
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            END DO
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! One of the two sublist is now empty.
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! Copy possibly remaining tail of 1st sublist
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            DO WHILE (i <= m)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               arr(k) = tmp_arr(i)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               indices(k) = tmp_idx(i)
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               i = i + 1
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
               k = k + 1
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            END DO
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
            ! The possibly remaining tail of 2nd sublist is already at the right spot.
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
         END IF
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"
      END SUBROUTINE cp_1d_i8_sort_low
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

# 64 "/builddir/build/BUILD/cp2k-2024.1/src/common/cp_array_sort.F"

   PURE FUNCTION cp_1d_i8_less_than(a, b) RESULT(res)
      INTEGER(kind=int_8), INTENT(IN) :: a, b
      LOGICAL                         :: res
      res = a < b
   END FUNCTION cp_1d_i8_less_than

END MODULE cp_array_sort
