# 1 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F" 1
!--------------------------------------------------------------------------------------------------!
!   CP2K: A general program to perform molecular dynamics simulations                              !
!   Copyright 2000-2024 CP2K developers group <https://cp2k.org>                                   !
!                                                                                                  !
!   SPDX-License-Identifier: GPL-2.0-or-later                                                      !
!--------------------------------------------------------------------------------------------------!

! **************************************************************************************************
!> \brief An array-based list which grows on demand.
!>        When the internal array is full, a new array of twice the size will be
!>        allocated and the items are copied over.
!>
!>        This list can also be used as a stack.
!>        Have look at list_push(), list_pop() and list_peek().
!> \par History
!>      12.2012 first version [ole]
!> \author Ole Schuett
! **************************************************************************************************

# 1 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.fypp" 1
# 7 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.fypp"
# 393 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.fypp"
# 21 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F" 2
# 22 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
MODULE list_routinestat
   USE timings_base_type, ONLY: callstack_entry_type, routine_stat_type, routine_report_type


# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
#include "../base/base_uses.f90"
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      IMPLICIT NONE
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      PRIVATE
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      PUBLIC :: list_routinestat_type, list_routinestat_init, list_routinestat_push, list_routinestat_pop, list_routinestat_peek,&
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
          & list_routinestat_insert, list_routinestat_set, list_routinestat_get, list_routinestat_del, list_routinestat_clear,&
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
          & list_routinestat_size, list_routinestat_destroy, list_routinestat_isready
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!this is an internal type
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      TYPE private_item_type_routinestat
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         PRIVATE
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         TYPE(routine_stat_type), POINTER                   :: value => NULL()
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      END TYPE private_item_type_routinestat
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!this is an internal type
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      TYPE private_item_p_type_routinestat
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         PRIVATE
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         TYPE(private_item_type_routinestat), POINTER :: p => Null()
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      END TYPE private_item_p_type_routinestat
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! this is the public type, which holds a list-instance
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      TYPE list_routinestat_type
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         PRIVATE
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         TYPE(private_item_p_type_routinestat), DIMENSION(:), POINTER   :: arr => Null()
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         INTEGER                                       :: size = -1
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      END TYPE list_routinestat_type
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      CONTAINS
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \brief Test if the given list has been initialized.
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \param list ...
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \return ...
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \par History
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>      12.2012 created [ole]
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \author Ole Schuett
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      FUNCTION list_routinestat_isready(list) RESULT(res)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         TYPE(list_routinestat_type), intent(in)  :: list
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         LOGICAL                                     :: res
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         res = ASSOCIATED(list%arr)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      END FUNCTION list_routinestat_isready
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \brief Allocates the internal data-structures of the given list.
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>        This has to be called before any of the other routines.
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>        For deallocation call list_[valuetype]_destroy.
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \param list ...
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \param initial_capacity The initial size of the internal array (default=11).
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \par History
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>      12.2012 created [ole]
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \author Ole Schuett
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      SUBROUTINE list_routinestat_init(list, initial_capacity)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         TYPE(list_routinestat_type), intent(inout)  :: list
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         INTEGER, INTENT(in), OPTIONAL               :: initial_capacity
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         INTEGER                                     :: stat
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         INTEGER                                     :: initial_capacity_
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         initial_capacity_ = 11
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         If (PRESENT(initial_capacity)) initial_capacity_ = initial_capacity
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (initial_capacity_ < 0) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_create: initial_capacity < 0")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (ASSOCIATED(list%arr)) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_create: list is already initialized.")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         ALLOCATE (list%arr(initial_capacity_), stat=stat)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (stat /= 0) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_init: allocation failed")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         list%size = 0
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      END SUBROUTINE list_routinestat_init
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \brief Deallocated the internal data-structures of the given list.
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>        Caution: If the stored values are pointers, their targets will
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>                 not get deallocated by this routine.
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \param list ...
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \par History
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>      12.2012 created [ole]
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \author Ole Schuett
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      SUBROUTINE list_routinestat_destroy(list)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         TYPE(list_routinestat_type), intent(inout)  :: list
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         INTEGER :: i
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (.not. ASSOCIATED(list%arr)) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_destroy: list is not initialized.")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         do i = 1, list%size
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            deallocate (list%arr(i)%p)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         end do
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         deallocate (list%arr)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         list%size = -1
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      END SUBROUTINE list_routinestat_destroy
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \brief Assings the given value to the given position in the list.
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>        Thereby, the former value at that position gets overwritten.
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>        If the position is out of bounds, the program stops.
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \param list ...
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \param value ...
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \param pos Position in the list - musst fulfill 0 < pos < list_size+1.
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \par History
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>      12.2012 created [ole]
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \author Ole Schuett
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      SUBROUTINE list_routinestat_set(list, value, pos)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         TYPE(list_routinestat_type), intent(inout)  :: list
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         TYPE(routine_stat_type), POINTER, intent(in) :: value
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         INTEGER, intent(in) :: pos
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (.not. ASSOCIATED(list%arr)) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_set: list is not initialized.")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (pos < 1) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_set: pos < 1")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (pos > list%size) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_set: pos > size")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         list%arr(pos)%p%value =>value
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      END SUBROUTINE list_routinestat_set
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \brief Appends the given value at the end of the list.
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \param list ...
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \param value ...
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \par History
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>      12.2012 created [ole]
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \author Ole Schuett
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      SUBROUTINE list_routinestat_push(list, value)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         TYPE(list_routinestat_type), intent(inout)  :: list
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         TYPE(routine_stat_type), POINTER, intent(in)                  :: value
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         INTEGER                                     :: stat
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (.not. ASSOCIATED(list%arr)) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_push: list is not initialized.")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         if (list%size == size(list%arr)) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            call change_capacity_routinestat (list, 2*size(list%arr) + 1)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         list%size = list%size + 1
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         ALLOCATE (list%arr(list%size)%p, stat=stat)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (stat /= 0) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_push: allocation failed")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         list%arr(list%size)%p%value =>value
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      END SUBROUTINE list_routinestat_push
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \brief Inserts the given value at the givenn position within the list.
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>        Values which lay behind the insertion-position move one position up.
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \param list ...
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \param value ...
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \param pos Position in the list - musst fulfill 0 < pos < list_size+2 .
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \par History
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>      12.2012 created [ole]
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \author Ole Schuett
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      SUBROUTINE list_routinestat_insert(list, value, pos)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         TYPE(list_routinestat_type), intent(inout)  :: list
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         TYPE(routine_stat_type), POINTER, intent(in) :: value
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         INTEGER, intent(in) :: pos
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         INTEGER :: i, stat
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (.not. ASSOCIATED(list%arr)) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_insert: list is not initialized.")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (pos < 1) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_insert: pos < 1")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (pos > list%size + 1) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_insert: pos > size+1")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         if (list%size == size(list%arr)) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            call change_capacity_routinestat (list, 2*size(list%arr) + 1)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         list%size = list%size + 1
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         do i = list%size, pos + 1, -1
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            list%arr(i)%p => list%arr(i - 1)%p
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         end do
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         ALLOCATE (list%arr(pos)%p, stat=stat)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (stat /= 0) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_insert: allocation failed.")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         list%arr(pos)%p%value =>value
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      END SUBROUTINE list_routinestat_insert
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \brief Returns the last element in the list.
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>    Is equivalent to: list_routinestat_get(list, list_routinestat_size(list))
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \param list ...
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \return ...
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \par History
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>      12.2012 created [ole]
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \author Ole Schuett
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      FUNCTION list_routinestat_peek(list) RESULT(value)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         TYPE(list_routinestat_type), intent(inout)  :: list
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         TYPE(routine_stat_type), POINTER  :: value
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (.not. ASSOCIATED(list%arr)) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_peek: list is not initialized.")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (list%size < 1) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_peek: list is empty.")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         value =>list%arr(list%size)%p%value
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      END FUNCTION list_routinestat_peek
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \brief Returns the last element in the list and removes it.
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>        Is equivialent to:
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>        value = list_routinestat_get(list, list_routinestat_size(list))
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>            call list_routinestat_del(list, list_routinestat_size(list))
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \param list ...
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \return ...
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \par History
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>      12.2012 created [ole]
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \author Ole Schuett
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      FUNCTION list_routinestat_pop(list) RESULT(value)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         TYPE(list_routinestat_type), intent(inout)  :: list
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         TYPE(routine_stat_type), POINTER  :: value
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (.not. ASSOCIATED(list%arr)) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_pop: list is not initialized.")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (list%size < 1) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_pop: list is empty.")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         value =>list%arr(list%size)%p%value
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         deallocate (list%arr(list%size)%p)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         list%size = list%size - 1
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      END FUNCTION list_routinestat_pop
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \brief Removes all values from the list. The list itself is not deallocated.
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \param list ...
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \par History
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>      12.2012 created [ole]
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \author Ole Schuett
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      SUBROUTINE list_routinestat_clear(list)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         TYPE(list_routinestat_type), intent(inout)  :: list
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         INTEGER :: i
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (.not. ASSOCIATED(list%arr)) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_clear: list is not initialized.")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         do i = 1, list%size
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            deallocate (list%arr(i)%p)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         end do
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         list%size = 0
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      END SUBROUTINE list_routinestat_clear
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \brief Returns the value at the given position from the list.
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \param list ...
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \param pos Position in the list - musst fulfill 0 < pos < list_size+1 .
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \return ...
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \par History
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>      12.2012 created [ole]
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \author Ole Schuett
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      FUNCTION list_routinestat_get(list, pos) RESULT(value)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         TYPE(list_routinestat_type), intent(in)  :: list
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         INTEGER, intent(in) :: pos
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         TYPE(routine_stat_type), POINTER :: value
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (.not. ASSOCIATED(list%arr)) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_get: list is not initialized.")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (pos < 1) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_get: pos < 1")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (pos > list%size) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_get: pos > size")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         value =>list%arr(pos)%p%value
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      END FUNCTION list_routinestat_get
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \brief Removes the value at the given position from the list.
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \param list ...
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \param pos Position in the list - musst fulfill 0 < pos < list_size+1 .
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \par History
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>      12.2012 created [ole]
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \author Ole Schuett
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      SUBROUTINE list_routinestat_del(list, pos)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         TYPE(list_routinestat_type), intent(inout)  :: list
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         INTEGER, intent(in) :: pos
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         INTEGER :: i
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (.not. ASSOCIATED(list%arr)) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_del: list is not initialized.")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (pos < 1) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_det: pos < 1")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (pos > list%size) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_det: pos > size")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         deallocate (list%arr(pos)%p)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         do i = pos, list%size - 1
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            list%arr(i)%p => list%arr(i + 1)%p
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         end do
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         list%size = list%size - 1
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      END SUBROUTINE list_routinestat_del
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \brief Returns the current size of the list.
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \param list ...
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \return ...
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \par History
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>      12.2012 created [ole]
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \author Ole Schuett
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      FUNCTION list_routinestat_size(list) RESULT(size)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         TYPE(list_routinestat_type), intent(in)  :: list
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         INTEGER :: size
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (.not. ASSOCIATED(list%arr)) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_size: list is not initialized.")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         size = list%size
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      END FUNCTION list_routinestat_size
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \brief Internal routine for changing the size of the internal array.
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \param list ...
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \param new_capacity ...
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \par History
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!>      12.2012 created [ole]
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
!> \author Ole Schuett
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      SUBROUTINE change_capacity_routinestat (list, new_capacity)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         TYPE(list_routinestat_type), intent(inout)  :: list
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         INTEGER, intent(in) :: new_capacity
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         INTEGER :: i, new_cap, stat
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         TYPE(private_item_p_type_routinestat), DIMENSION(:), POINTER :: old_arr
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         new_cap = new_capacity
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (new_cap < 0) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_change_capacity: new_capacity < 0")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (new_cap < list%size) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_change_capacity: new_capacity < size")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (new_cap > HUGE(i)) THEN
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            IF (size(list%arr) == HUGE(i)) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
               CPABORT("list_routinestat_change_capacity: list has reached integer limit.")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            new_cap = HUGE(i) ! grow as far as possible
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         END IF
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         old_arr => list%arr
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         allocate (list%arr(new_cap), stat=stat)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         IF (stat /= 0) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            CPABORT("list_routinestat_change_capacity: allocation failed")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         do i = 1, list%size
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            allocate (list%arr(i)%p, stat=stat)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            IF (stat /= 0) &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
               CPABORT("list_routinestat_change_capacity: allocation failed")
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            list%arr(i)%p%value =>old_arr(i)%p%value
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
            deallocate (old_arr(i)%p)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         end do
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
         deallocate (old_arr)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/common/list_routinestat.F"
      END SUBROUTINE change_capacity_routinestat

END MODULE list_routinestat
