# 1 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F" 1
!--------------------------------------------------------------------------------------------------!
!   CP2K: A general program to perform molecular dynamics simulations                              !
!   Copyright 2000-2024 CP2K developers group <https://cp2k.org>                                   !
!                                                                                                  !
!   SPDX-License-Identifier: GPL-2.0-or-later                                                      !
!--------------------------------------------------------------------------------------------------!

! **************************************************************************************************
!> \brief Utility routines for the memory handling.
!> \par History
!>      (12.2017) remove stop_memory
!> \author Matthias Krack (25.06.1999)
! **************************************************************************************************
MODULE memory_utilities

   USE kinds, ONLY: dp, int_8
#include "../base/base_uses.f90"

   IMPLICIT NONE

   PRIVATE

   CHARACTER(len=*), PARAMETER, PRIVATE :: moduleN = 'memory_utilities'

   PUBLIC :: reallocate

   INTERFACE reallocate
      MODULE PROCEDURE reallocate_c1, reallocate_c2, reallocate_c3, reallocate_c4, &
         reallocate_i1, reallocate_i2, reallocate_i3, reallocate_i4, &
         reallocate_r1, reallocate_r2, reallocate_r3, reallocate_r4, &
         reallocate_r5, reallocate_s1, reallocate_l1, reallocate_8i1, &
         reallocate_8i2
   END INTERFACE

CONTAINS

# 42 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
# 97 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

! **************************************************************************************************
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \brief (Re)Allocate a 1D vector of type COMPLEX(KIND=dp) with new dimensions (but same shape)
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param p pointer to the existing data, if NULL() calling this is equivalent to an ALLOCATE(...)
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb1_new new lower bound for dimension 1
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub1_new new upper bound for dimension 1
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
! **************************************************************************************************
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      SUBROUTINE reallocate_c1 (p, lb1_new,ub1_new)
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         COMPLEX(KIND=dp), &
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:), &
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER, INTENT(INOUT) :: p
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         INTEGER, INTENT(IN) :: &
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            lb1_new,ub1_new
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb1, lb1_old, ub1, ub1_old
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            COMPLEX(KIND=dp), &
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:), &
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER :: work
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         NULLIFY (work)
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(p)) THEN
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1_old = LBOUND(p, 1)
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1_old = UBOUND(p, 1)
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1 = MAX(lb1_new, lb1_old)
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1 = MIN(ub1_new, ub1_old)
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            work => p
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         ALLOCATE (p(lb1_new:ub1_new))
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         p = (0.0_dp, 0.0_dp)
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(work)) THEN
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            p(lb1:ub1) = work(lb1:ub1)
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DEALLOCATE (work)
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      END SUBROUTINE reallocate_c1
! **************************************************************************************************
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \brief (Re)Allocate a 2D vector of type COMPLEX(KIND=dp) with new dimensions (but same shape)
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param p pointer to the existing data, if NULL() calling this is equivalent to an ALLOCATE(...)
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb1_new new lower bound for dimension 1
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub1_new new upper bound for dimension 1
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb2_new new lower bound for dimension 2
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub2_new new upper bound for dimension 2
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
! **************************************************************************************************
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      SUBROUTINE reallocate_c2 (p, lb1_new,ub1_new,lb2_new,ub2_new)
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         COMPLEX(KIND=dp), &
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:,:), &
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER, INTENT(INOUT) :: p
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         INTEGER, INTENT(IN) :: &
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            lb1_new,ub1_new,lb2_new,ub2_new
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb1, lb1_old, ub1, ub1_old
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb2, lb2_old, ub2, ub2_old
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            COMPLEX(KIND=dp), &
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:,:), &
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER :: work
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         NULLIFY (work)
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(p)) THEN
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1_old = LBOUND(p, 1)
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1_old = UBOUND(p, 1)
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1 = MAX(lb1_new, lb1_old)
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1 = MIN(ub1_new, ub1_old)
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb2_old = LBOUND(p, 2)
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub2_old = UBOUND(p, 2)
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb2 = MAX(lb2_new, lb2_old)
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub2 = MIN(ub2_new, ub2_old)
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            work => p
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new))
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         p = (0.0_dp, 0.0_dp)
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(work)) THEN
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            p(lb1:ub1,lb2:ub2) = work(lb1:ub1,lb2:ub2)
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DEALLOCATE (work)
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 99 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      END SUBROUTINE reallocate_c2
! **************************************************************************************************
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \brief (Re)Allocate a 3D vector of type COMPLEX(KIND=dp) with new dimensions (but same shape)
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param p pointer to the existing data, if NULL() calling this is equivalent to an ALLOCATE(...)
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb1_new new lower bound for dimension 1
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub1_new new upper bound for dimension 1
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb2_new new lower bound for dimension 2
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub2_new new upper bound for dimension 2
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb3_new new lower bound for dimension 3
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub3_new new upper bound for dimension 3
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
! **************************************************************************************************
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      SUBROUTINE reallocate_c3 (p, lb1_new,ub1_new,lb2_new,ub2_new,lb3_new,ub3_new)
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         COMPLEX(KIND=dp), &
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:,:,:), &
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER, INTENT(INOUT) :: p
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         INTEGER, INTENT(IN) :: &
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            lb1_new,ub1_new,lb2_new,ub2_new,lb3_new,ub3_new
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb1, lb1_old, ub1, ub1_old
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb2, lb2_old, ub2, ub2_old
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb3, lb3_old, ub3, ub3_old
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            COMPLEX(KIND=dp), &
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:,:,:), &
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER :: work
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         NULLIFY (work)
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(p)) THEN
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1_old = LBOUND(p, 1)
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1_old = UBOUND(p, 1)
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1 = MAX(lb1_new, lb1_old)
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1 = MIN(ub1_new, ub1_old)
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb2_old = LBOUND(p, 2)
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub2_old = UBOUND(p, 2)
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb2 = MAX(lb2_new, lb2_old)
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub2 = MIN(ub2_new, ub2_old)
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb3_old = LBOUND(p, 3)
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub3_old = UBOUND(p, 3)
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb3 = MAX(lb3_new, lb3_old)
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub3 = MIN(ub3_new, ub3_old)
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            work => p
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new,lb3_new:ub3_new))
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         p = (0.0_dp, 0.0_dp)
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(work)) THEN
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            p(lb1:ub1,lb2:ub2,lb3:ub3) = work(lb1:ub1,lb2:ub2,lb3:ub3)
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DEALLOCATE (work)
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 100 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      END SUBROUTINE reallocate_c3
! **************************************************************************************************
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \brief (Re)Allocate a 4D vector of type COMPLEX(KIND=dp) with new dimensions (but same shape)
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param p pointer to the existing data, if NULL() calling this is equivalent to an ALLOCATE(...)
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb1_new new lower bound for dimension 1
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub1_new new upper bound for dimension 1
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb2_new new lower bound for dimension 2
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub2_new new upper bound for dimension 2
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb3_new new lower bound for dimension 3
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub3_new new upper bound for dimension 3
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb4_new new lower bound for dimension 4
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub4_new new upper bound for dimension 4
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
! **************************************************************************************************
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      SUBROUTINE reallocate_c4 (p, lb1_new,ub1_new,lb2_new,ub2_new,lb3_new,ub3_new,lb4_new,ub4_new)
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         COMPLEX(KIND=dp), &
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:,:,:,:), &
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER, INTENT(INOUT) :: p
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         INTEGER, INTENT(IN) :: &
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            lb1_new,ub1_new,lb2_new,ub2_new,lb3_new,ub3_new,lb4_new,ub4_new
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb1, lb1_old, ub1, ub1_old
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb2, lb2_old, ub2, ub2_old
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb3, lb3_old, ub3, ub3_old
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb4, lb4_old, ub4, ub4_old
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            COMPLEX(KIND=dp), &
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:,:,:,:), &
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER :: work
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         NULLIFY (work)
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(p)) THEN
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1_old = LBOUND(p, 1)
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1_old = UBOUND(p, 1)
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1 = MAX(lb1_new, lb1_old)
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1 = MIN(ub1_new, ub1_old)
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb2_old = LBOUND(p, 2)
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub2_old = UBOUND(p, 2)
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb2 = MAX(lb2_new, lb2_old)
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub2 = MIN(ub2_new, ub2_old)
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb3_old = LBOUND(p, 3)
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub3_old = UBOUND(p, 3)
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb3 = MAX(lb3_new, lb3_old)
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub3 = MIN(ub3_new, ub3_old)
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb4_old = LBOUND(p, 4)
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub4_old = UBOUND(p, 4)
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb4 = MAX(lb4_new, lb4_old)
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub4 = MIN(ub4_new, ub4_old)
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            work => p
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new,lb3_new:ub3_new,lb4_new:ub4_new))
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         p = (0.0_dp, 0.0_dp)
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(work)) THEN
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            p(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4) = work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4)
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DEALLOCATE (work)
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 101 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      END SUBROUTINE reallocate_c4
! **************************************************************************************************
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \brief (Re)Allocate a 1D vector of type INTEGER with new dimensions (but same shape)
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param p pointer to the existing data, if NULL() calling this is equivalent to an ALLOCATE(...)
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb1_new new lower bound for dimension 1
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub1_new new upper bound for dimension 1
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
! **************************************************************************************************
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      SUBROUTINE reallocate_i1 (p, lb1_new,ub1_new)
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         INTEGER, &
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:), &
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER, INTENT(INOUT) :: p
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         INTEGER, INTENT(IN) :: &
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            lb1_new,ub1_new
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb1, lb1_old, ub1, ub1_old
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER, &
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:), &
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER :: work
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         NULLIFY (work)
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(p)) THEN
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1_old = LBOUND(p, 1)
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1_old = UBOUND(p, 1)
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1 = MAX(lb1_new, lb1_old)
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1 = MIN(ub1_new, ub1_old)
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            work => p
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         ALLOCATE (p(lb1_new:ub1_new))
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         p = 0
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(work)) THEN
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            p(lb1:ub1) = work(lb1:ub1)
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DEALLOCATE (work)
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      END SUBROUTINE reallocate_i1
! **************************************************************************************************
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \brief (Re)Allocate a 2D vector of type INTEGER with new dimensions (but same shape)
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param p pointer to the existing data, if NULL() calling this is equivalent to an ALLOCATE(...)
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb1_new new lower bound for dimension 1
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub1_new new upper bound for dimension 1
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb2_new new lower bound for dimension 2
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub2_new new upper bound for dimension 2
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
! **************************************************************************************************
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      SUBROUTINE reallocate_i2 (p, lb1_new,ub1_new,lb2_new,ub2_new)
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         INTEGER, &
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:,:), &
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER, INTENT(INOUT) :: p
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         INTEGER, INTENT(IN) :: &
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            lb1_new,ub1_new,lb2_new,ub2_new
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb1, lb1_old, ub1, ub1_old
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb2, lb2_old, ub2, ub2_old
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER, &
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:,:), &
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER :: work
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         NULLIFY (work)
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(p)) THEN
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1_old = LBOUND(p, 1)
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1_old = UBOUND(p, 1)
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1 = MAX(lb1_new, lb1_old)
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1 = MIN(ub1_new, ub1_old)
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb2_old = LBOUND(p, 2)
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub2_old = UBOUND(p, 2)
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb2 = MAX(lb2_new, lb2_old)
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub2 = MIN(ub2_new, ub2_old)
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            work => p
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new))
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         p = 0
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(work)) THEN
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            p(lb1:ub1,lb2:ub2) = work(lb1:ub1,lb2:ub2)
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DEALLOCATE (work)
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 103 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      END SUBROUTINE reallocate_i2
! **************************************************************************************************
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \brief (Re)Allocate a 3D vector of type INTEGER with new dimensions (but same shape)
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param p pointer to the existing data, if NULL() calling this is equivalent to an ALLOCATE(...)
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb1_new new lower bound for dimension 1
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub1_new new upper bound for dimension 1
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb2_new new lower bound for dimension 2
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub2_new new upper bound for dimension 2
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb3_new new lower bound for dimension 3
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub3_new new upper bound for dimension 3
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
! **************************************************************************************************
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      SUBROUTINE reallocate_i3 (p, lb1_new,ub1_new,lb2_new,ub2_new,lb3_new,ub3_new)
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         INTEGER, &
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:,:,:), &
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER, INTENT(INOUT) :: p
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         INTEGER, INTENT(IN) :: &
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            lb1_new,ub1_new,lb2_new,ub2_new,lb3_new,ub3_new
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb1, lb1_old, ub1, ub1_old
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb2, lb2_old, ub2, ub2_old
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb3, lb3_old, ub3, ub3_old
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER, &
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:,:,:), &
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER :: work
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         NULLIFY (work)
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(p)) THEN
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1_old = LBOUND(p, 1)
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1_old = UBOUND(p, 1)
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1 = MAX(lb1_new, lb1_old)
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1 = MIN(ub1_new, ub1_old)
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb2_old = LBOUND(p, 2)
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub2_old = UBOUND(p, 2)
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb2 = MAX(lb2_new, lb2_old)
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub2 = MIN(ub2_new, ub2_old)
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb3_old = LBOUND(p, 3)
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub3_old = UBOUND(p, 3)
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb3 = MAX(lb3_new, lb3_old)
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub3 = MIN(ub3_new, ub3_old)
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            work => p
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new,lb3_new:ub3_new))
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         p = 0
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(work)) THEN
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            p(lb1:ub1,lb2:ub2,lb3:ub3) = work(lb1:ub1,lb2:ub2,lb3:ub3)
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DEALLOCATE (work)
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      END SUBROUTINE reallocate_i3
! **************************************************************************************************
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \brief (Re)Allocate a 4D vector of type INTEGER with new dimensions (but same shape)
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param p pointer to the existing data, if NULL() calling this is equivalent to an ALLOCATE(...)
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb1_new new lower bound for dimension 1
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub1_new new upper bound for dimension 1
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb2_new new lower bound for dimension 2
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub2_new new upper bound for dimension 2
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb3_new new lower bound for dimension 3
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub3_new new upper bound for dimension 3
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb4_new new lower bound for dimension 4
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub4_new new upper bound for dimension 4
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
! **************************************************************************************************
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      SUBROUTINE reallocate_i4 (p, lb1_new,ub1_new,lb2_new,ub2_new,lb3_new,ub3_new,lb4_new,ub4_new)
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         INTEGER, &
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:,:,:,:), &
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER, INTENT(INOUT) :: p
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         INTEGER, INTENT(IN) :: &
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            lb1_new,ub1_new,lb2_new,ub2_new,lb3_new,ub3_new,lb4_new,ub4_new
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb1, lb1_old, ub1, ub1_old
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb2, lb2_old, ub2, ub2_old
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb3, lb3_old, ub3, ub3_old
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb4, lb4_old, ub4, ub4_old
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER, &
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:,:,:,:), &
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER :: work
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         NULLIFY (work)
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(p)) THEN
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1_old = LBOUND(p, 1)
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1_old = UBOUND(p, 1)
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1 = MAX(lb1_new, lb1_old)
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1 = MIN(ub1_new, ub1_old)
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb2_old = LBOUND(p, 2)
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub2_old = UBOUND(p, 2)
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb2 = MAX(lb2_new, lb2_old)
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub2 = MIN(ub2_new, ub2_old)
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb3_old = LBOUND(p, 3)
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub3_old = UBOUND(p, 3)
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb3 = MAX(lb3_new, lb3_old)
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub3 = MIN(ub3_new, ub3_old)
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb4_old = LBOUND(p, 4)
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub4_old = UBOUND(p, 4)
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb4 = MAX(lb4_new, lb4_old)
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub4 = MIN(ub4_new, ub4_old)
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            work => p
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new,lb3_new:ub3_new,lb4_new:ub4_new))
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         p = 0
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(work)) THEN
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            p(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4) = work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4)
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DEALLOCATE (work)
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 105 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      END SUBROUTINE reallocate_i4
! **************************************************************************************************
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \brief (Re)Allocate a 1D vector of type INTEGER(KIND=int_8) with new dimensions (but same shape)
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param p pointer to the existing data, if NULL() calling this is equivalent to an ALLOCATE(...)
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb1_new new lower bound for dimension 1
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub1_new new upper bound for dimension 1
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
! **************************************************************************************************
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      SUBROUTINE reallocate_8i1 (p, lb1_new,ub1_new)
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         INTEGER(KIND=int_8), &
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:), &
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER, INTENT(INOUT) :: p
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         INTEGER, INTENT(IN) :: &
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            lb1_new,ub1_new
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb1, lb1_old, ub1, ub1_old
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER(KIND=int_8), &
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:), &
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER :: work
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         NULLIFY (work)
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(p)) THEN
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1_old = LBOUND(p, 1)
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1_old = UBOUND(p, 1)
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1 = MAX(lb1_new, lb1_old)
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1 = MIN(ub1_new, ub1_old)
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            work => p
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         ALLOCATE (p(lb1_new:ub1_new))
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         p = 0
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(work)) THEN
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            p(lb1:ub1) = work(lb1:ub1)
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DEALLOCATE (work)
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 106 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      END SUBROUTINE reallocate_8i1
! **************************************************************************************************
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \brief (Re)Allocate a 2D vector of type INTEGER(KIND=int_8) with new dimensions (but same shape)
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param p pointer to the existing data, if NULL() calling this is equivalent to an ALLOCATE(...)
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb1_new new lower bound for dimension 1
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub1_new new upper bound for dimension 1
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb2_new new lower bound for dimension 2
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub2_new new upper bound for dimension 2
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
! **************************************************************************************************
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      SUBROUTINE reallocate_8i2 (p, lb1_new,ub1_new,lb2_new,ub2_new)
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         INTEGER(KIND=int_8), &
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:,:), &
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER, INTENT(INOUT) :: p
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         INTEGER, INTENT(IN) :: &
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            lb1_new,ub1_new,lb2_new,ub2_new
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb1, lb1_old, ub1, ub1_old
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb2, lb2_old, ub2, ub2_old
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER(KIND=int_8), &
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:,:), &
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER :: work
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         NULLIFY (work)
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(p)) THEN
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1_old = LBOUND(p, 1)
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1_old = UBOUND(p, 1)
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1 = MAX(lb1_new, lb1_old)
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1 = MIN(ub1_new, ub1_old)
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb2_old = LBOUND(p, 2)
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub2_old = UBOUND(p, 2)
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb2 = MAX(lb2_new, lb2_old)
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub2 = MIN(ub2_new, ub2_old)
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            work => p
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new))
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         p = 0
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(work)) THEN
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            p(lb1:ub1,lb2:ub2) = work(lb1:ub1,lb2:ub2)
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DEALLOCATE (work)
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 107 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      END SUBROUTINE reallocate_8i2
! **************************************************************************************************
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \brief (Re)Allocate a 1D vector of type REAL(KIND=dp) with new dimensions (but same shape)
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param p pointer to the existing data, if NULL() calling this is equivalent to an ALLOCATE(...)
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb1_new new lower bound for dimension 1
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub1_new new upper bound for dimension 1
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
! **************************************************************************************************
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      SUBROUTINE reallocate_r1 (p, lb1_new,ub1_new)
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         REAL(KIND=dp), &
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:), &
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER, INTENT(INOUT) :: p
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         INTEGER, INTENT(IN) :: &
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            lb1_new,ub1_new
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb1, lb1_old, ub1, ub1_old
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            REAL(KIND=dp), &
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:), &
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER :: work
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         NULLIFY (work)
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(p)) THEN
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1_old = LBOUND(p, 1)
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1_old = UBOUND(p, 1)
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1 = MAX(lb1_new, lb1_old)
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1 = MIN(ub1_new, ub1_old)
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            work => p
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         ALLOCATE (p(lb1_new:ub1_new))
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         p = 0.0_dp
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(work)) THEN
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            p(lb1:ub1) = work(lb1:ub1)
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DEALLOCATE (work)
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      END SUBROUTINE reallocate_r1
! **************************************************************************************************
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \brief (Re)Allocate a 2D vector of type REAL(KIND=dp) with new dimensions (but same shape)
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param p pointer to the existing data, if NULL() calling this is equivalent to an ALLOCATE(...)
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb1_new new lower bound for dimension 1
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub1_new new upper bound for dimension 1
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb2_new new lower bound for dimension 2
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub2_new new upper bound for dimension 2
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
! **************************************************************************************************
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      SUBROUTINE reallocate_r2 (p, lb1_new,ub1_new,lb2_new,ub2_new)
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         REAL(KIND=dp), &
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:,:), &
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER, INTENT(INOUT) :: p
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         INTEGER, INTENT(IN) :: &
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            lb1_new,ub1_new,lb2_new,ub2_new
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb1, lb1_old, ub1, ub1_old
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb2, lb2_old, ub2, ub2_old
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            REAL(KIND=dp), &
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:,:), &
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER :: work
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         NULLIFY (work)
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(p)) THEN
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1_old = LBOUND(p, 1)
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1_old = UBOUND(p, 1)
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1 = MAX(lb1_new, lb1_old)
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1 = MIN(ub1_new, ub1_old)
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb2_old = LBOUND(p, 2)
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub2_old = UBOUND(p, 2)
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb2 = MAX(lb2_new, lb2_old)
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub2 = MIN(ub2_new, ub2_old)
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            work => p
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new))
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         p = 0.0_dp
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(work)) THEN
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            p(lb1:ub1,lb2:ub2) = work(lb1:ub1,lb2:ub2)
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DEALLOCATE (work)
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 109 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      END SUBROUTINE reallocate_r2
! **************************************************************************************************
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \brief (Re)Allocate a 3D vector of type REAL(KIND=dp) with new dimensions (but same shape)
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param p pointer to the existing data, if NULL() calling this is equivalent to an ALLOCATE(...)
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb1_new new lower bound for dimension 1
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub1_new new upper bound for dimension 1
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb2_new new lower bound for dimension 2
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub2_new new upper bound for dimension 2
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb3_new new lower bound for dimension 3
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub3_new new upper bound for dimension 3
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
! **************************************************************************************************
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      SUBROUTINE reallocate_r3 (p, lb1_new,ub1_new,lb2_new,ub2_new,lb3_new,ub3_new)
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         REAL(KIND=dp), &
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:,:,:), &
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER, INTENT(INOUT) :: p
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         INTEGER, INTENT(IN) :: &
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            lb1_new,ub1_new,lb2_new,ub2_new,lb3_new,ub3_new
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb1, lb1_old, ub1, ub1_old
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb2, lb2_old, ub2, ub2_old
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb3, lb3_old, ub3, ub3_old
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            REAL(KIND=dp), &
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:,:,:), &
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER :: work
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         NULLIFY (work)
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(p)) THEN
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1_old = LBOUND(p, 1)
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1_old = UBOUND(p, 1)
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1 = MAX(lb1_new, lb1_old)
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1 = MIN(ub1_new, ub1_old)
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb2_old = LBOUND(p, 2)
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub2_old = UBOUND(p, 2)
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb2 = MAX(lb2_new, lb2_old)
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub2 = MIN(ub2_new, ub2_old)
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb3_old = LBOUND(p, 3)
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub3_old = UBOUND(p, 3)
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb3 = MAX(lb3_new, lb3_old)
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub3 = MIN(ub3_new, ub3_old)
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            work => p
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new,lb3_new:ub3_new))
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         p = 0.0_dp
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(work)) THEN
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            p(lb1:ub1,lb2:ub2,lb3:ub3) = work(lb1:ub1,lb2:ub2,lb3:ub3)
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DEALLOCATE (work)
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      END SUBROUTINE reallocate_r3
! **************************************************************************************************
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \brief (Re)Allocate a 4D vector of type REAL(KIND=dp) with new dimensions (but same shape)
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param p pointer to the existing data, if NULL() calling this is equivalent to an ALLOCATE(...)
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb1_new new lower bound for dimension 1
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub1_new new upper bound for dimension 1
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb2_new new lower bound for dimension 2
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub2_new new upper bound for dimension 2
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb3_new new lower bound for dimension 3
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub3_new new upper bound for dimension 3
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb4_new new lower bound for dimension 4
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub4_new new upper bound for dimension 4
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
! **************************************************************************************************
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      SUBROUTINE reallocate_r4 (p, lb1_new,ub1_new,lb2_new,ub2_new,lb3_new,ub3_new,lb4_new,ub4_new)
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         REAL(KIND=dp), &
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:,:,:,:), &
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER, INTENT(INOUT) :: p
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         INTEGER, INTENT(IN) :: &
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            lb1_new,ub1_new,lb2_new,ub2_new,lb3_new,ub3_new,lb4_new,ub4_new
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb1, lb1_old, ub1, ub1_old
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb2, lb2_old, ub2, ub2_old
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb3, lb3_old, ub3, ub3_old
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb4, lb4_old, ub4, ub4_old
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            REAL(KIND=dp), &
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:,:,:,:), &
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER :: work
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         NULLIFY (work)
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(p)) THEN
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1_old = LBOUND(p, 1)
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1_old = UBOUND(p, 1)
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1 = MAX(lb1_new, lb1_old)
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1 = MIN(ub1_new, ub1_old)
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb2_old = LBOUND(p, 2)
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub2_old = UBOUND(p, 2)
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb2 = MAX(lb2_new, lb2_old)
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub2 = MIN(ub2_new, ub2_old)
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb3_old = LBOUND(p, 3)
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub3_old = UBOUND(p, 3)
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb3 = MAX(lb3_new, lb3_old)
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub3 = MIN(ub3_new, ub3_old)
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb4_old = LBOUND(p, 4)
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub4_old = UBOUND(p, 4)
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb4 = MAX(lb4_new, lb4_old)
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub4 = MIN(ub4_new, ub4_old)
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            work => p
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new,lb3_new:ub3_new,lb4_new:ub4_new))
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         p = 0.0_dp
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(work)) THEN
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            p(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4) = work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4)
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DEALLOCATE (work)
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 111 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      END SUBROUTINE reallocate_r4
! **************************************************************************************************
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \brief (Re)Allocate a 5D vector of type REAL(KIND=dp) with new dimensions (but same shape)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param p pointer to the existing data, if NULL() calling this is equivalent to an ALLOCATE(...)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb1_new new lower bound for dimension 1
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub1_new new upper bound for dimension 1
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb2_new new lower bound for dimension 2
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub2_new new upper bound for dimension 2
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb3_new new lower bound for dimension 3
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub3_new new upper bound for dimension 3
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb4_new new lower bound for dimension 4
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub4_new new upper bound for dimension 4
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb5_new new lower bound for dimension 5
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub5_new new upper bound for dimension 5
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
! **************************************************************************************************
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      SUBROUTINE reallocate_r5 (p, lb1_new,ub1_new,lb2_new,ub2_new,lb3_new,ub3_new,lb4_new,ub4_new,lb5_new,ub5_new)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         REAL(KIND=dp), &
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:,:,:,:,:), &
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER, INTENT(INOUT) :: p
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         INTEGER, INTENT(IN) :: &
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            lb1_new,ub1_new,lb2_new,ub2_new,lb3_new,ub3_new,lb4_new,ub4_new,lb5_new,ub5_new
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb1, lb1_old, ub1, ub1_old
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb2, lb2_old, ub2, ub2_old
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb3, lb3_old, ub3, ub3_old
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb4, lb4_old, ub4, ub4_old
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb5, lb5_old, ub5, ub5_old
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            REAL(KIND=dp), &
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:,:,:,:,:), &
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER :: work
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         NULLIFY (work)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(p)) THEN
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1_old = LBOUND(p, 1)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1_old = UBOUND(p, 1)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1 = MAX(lb1_new, lb1_old)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1 = MIN(ub1_new, ub1_old)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb2_old = LBOUND(p, 2)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub2_old = UBOUND(p, 2)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb2 = MAX(lb2_new, lb2_old)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub2 = MIN(ub2_new, ub2_old)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb3_old = LBOUND(p, 3)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub3_old = UBOUND(p, 3)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb3 = MAX(lb3_new, lb3_old)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub3 = MIN(ub3_new, ub3_old)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb4_old = LBOUND(p, 4)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub4_old = UBOUND(p, 4)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb4 = MAX(lb4_new, lb4_old)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub4 = MIN(ub4_new, ub4_old)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb5_old = LBOUND(p, 5)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub5_old = UBOUND(p, 5)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb5 = MAX(lb5_new, lb5_old)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub5 = MIN(ub5_new, ub5_old)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            work => p
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new,lb3_new:ub3_new,lb4_new:ub4_new,lb5_new:ub5_new))
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         p = 0.0_dp
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(work)) THEN
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            p(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4,lb5:ub5) = work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4,lb5:ub5)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DEALLOCATE (work)
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 112 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      END SUBROUTINE reallocate_r5
! **************************************************************************************************
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \brief (Re)Allocate a 1D vector of type LOGICAL with new dimensions (but same shape)
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param p pointer to the existing data, if NULL() calling this is equivalent to an ALLOCATE(...)
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb1_new new lower bound for dimension 1
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub1_new new upper bound for dimension 1
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
! **************************************************************************************************
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      SUBROUTINE reallocate_l1 (p, lb1_new,ub1_new)
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         LOGICAL, &
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:), &
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER, INTENT(INOUT) :: p
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         INTEGER, INTENT(IN) :: &
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            lb1_new,ub1_new
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb1, lb1_old, ub1, ub1_old
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            LOGICAL, &
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:), &
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER :: work
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         NULLIFY (work)
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(p)) THEN
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1_old = LBOUND(p, 1)
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1_old = UBOUND(p, 1)
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1 = MAX(lb1_new, lb1_old)
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1 = MIN(ub1_new, ub1_old)
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            work => p
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         ALLOCATE (p(lb1_new:ub1_new))
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         p = .FALSE.
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(work)) THEN
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            p(lb1:ub1) = work(lb1:ub1)
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DEALLOCATE (work)
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 113 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      END SUBROUTINE reallocate_l1
! **************************************************************************************************
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \brief (Re)Allocate a 1D vector of type CHARACTER(LEN=*) with new dimensions (but same shape)
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param p pointer to the existing data, if NULL() calling this is equivalent to an ALLOCATE(...)
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param lb1_new new lower bound for dimension 1
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
!> \param ub1_new new upper bound for dimension 1
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
! **************************************************************************************************
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      SUBROUTINE reallocate_s1 (p, lb1_new,ub1_new)
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         CHARACTER(LEN=*), &
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:), &
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER, INTENT(INOUT) :: p
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         INTEGER, INTENT(IN) :: &
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            lb1_new,ub1_new
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            INTEGER :: lb1, lb1_old, ub1, ub1_old
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         CHARACTER(LEN=LEN(p)), &
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DIMENSION(:), &
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            POINTER :: work
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         NULLIFY (work)
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(p)) THEN
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1_old = LBOUND(p, 1)
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1_old = UBOUND(p, 1)
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               lb1 = MAX(lb1_new, lb1_old)
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
               ub1 = MIN(ub1_new, ub1_old)
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            work => p
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         ALLOCATE (p(lb1_new:ub1_new))
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         p = ""
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         IF (ASSOCIATED(work)) THEN
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            p(lb1:ub1) = work(lb1:ub1)
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
            DEALLOCATE (work)
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
         END IF
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"

# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/memory_utilities.F"
      END SUBROUTINE reallocate_s1

END MODULE memory_utilities
