# 1 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F" 1
!--------------------------------------------------------------------------------------------------!
!   CP2K: A general program to perform molecular dynamics simulations                              !
!   Copyright 2000-2024 CP2K developers group <https://cp2k.org>                                   !
!                                                                                                  !
!   SPDX-License-Identifier: GPL-2.0-or-later                                                      !
!--------------------------------------------------------------------------------------------------!
MODULE routine_map
   USE kinds, ONLY: default_string_length, int_4, int_8
#include "../base/base_uses.f90"

   IMPLICIT NONE
   PRIVATE

# 1 "/builddir/build/BUILD/cp2k-2024.1/src/common/hash_map.fypp" 1
# 7 "/builddir/build/BUILD/cp2k-2024.1/src/common/hash_map.fypp"
# 428 "/builddir/build/BUILD/cp2k-2024.1/src/common/hash_map.fypp"
# 15 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F" 2

! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \brief A hash map (also known as hashtable or dictionary).
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!>        Internally the hash map uses an array to holds its data.
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!>        If this array reaches a load-factor of 75%, a new array with twice the
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!>        size will be allocated and the items are then copied over.
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!>        This ensures that the dictionary will perform operations in O(1).
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \par History
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!>      12.2012 first version [Ole Schuett]
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!>      08.2019 refactored for Fypp [Ole Schuett]
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \author Ole Schuett
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! ***************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      PUBLIC :: routine_map_init
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      PUBLIC :: routine_map_items
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      PUBLIC :: routine_map_haskey
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      PUBLIC :: routine_map_set
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      PUBLIC :: routine_map_get
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      PUBLIC :: routine_map_size
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      PUBLIC :: routine_map_destroy
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      PUBLIC :: routine_map_type
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      PUBLIC :: routine_map_item_type
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!this is an internal type
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!Calculating hashes might be expensive, therefore they are stored
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!for use during change_capacity().
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      TYPE private_item_type
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         PRIVATE
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         CHARACTER(LEN=default_string_length)                                :: key  = ""
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         INTEGER(kind=int_4)                              :: value  = 0_int_4
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         INTEGER(KIND=int_8)                         :: hash = 0_int_8
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(private_item_type), POINTER            :: next => Null()
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      END TYPE private_item_type
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!this is an internal type
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      TYPE private_item_p_type
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         PRIVATE
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(private_item_type), POINTER           :: p => Null()
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      END TYPE private_item_p_type
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! this is the public type, which holds a hash map instance
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      TYPE routine_map_type
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         PRIVATE
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(private_item_p_type), DIMENSION(:), POINTER      :: buckets => Null()
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         INTEGER                                               :: size = -1
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      END TYPE routine_map_type
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! this is a public type, its returned by  routine_map_items()
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      TYPE routine_map_item_type
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         CHARACTER(LEN=default_string_length)        :: key  = ""
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         INTEGER(kind=int_4)      :: value  = 0_int_4
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      END TYPE routine_map_item_type
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      CONTAINS
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \brief Allocates the internal data-structures of the given hash map.
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param hash_map ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param initial_capacity The initial size of the internal array (default=11).
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \author Ole Schuett
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      SUBROUTINE routine_map_init(hash_map, initial_capacity)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(routine_map_type), INTENT(inout)                 :: hash_map
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         INTEGER, INTENT(in), OPTIONAL                             :: initial_capacity
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         INTEGER :: initial_capacity_
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         IF (PRESENT(initial_capacity)) THEN
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            initial_capacity_ = initial_capacity
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         ELSE
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            initial_capacity_ = 11
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         END IF
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         IF (initial_capacity_ < 1) &
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            CPABORT("initial_capacity < 1")
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         IF (ASSOCIATED(hash_map%buckets)) &
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            CPABORT("hash map is already initialized.")
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         ALLOCATE (hash_map%buckets(initial_capacity_))
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         hash_map%size = 0
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      END SUBROUTINE routine_map_init
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \brief Test if the given hash map has been initialized.
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param hash_map ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \return ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \author Ole Schuett
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      FUNCTION routine_map_isready(hash_map) RESULT(res)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(routine_map_type), INTENT(inout)             :: hash_map
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         LOGICAL                                               :: res
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         res = ASSOCIATED(hash_map%buckets)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      END FUNCTION routine_map_isready
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \brief Deallocated the internal data-structures if the given hash map.
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!>        Caution: If the stored keys or values are pointers, their targets will
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!>                 not get deallocated by this routine.
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param hash_map ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \author Ole Schuett
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      SUBROUTINE routine_map_destroy(hash_map)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(routine_map_type), INTENT(inout)  :: hash_map
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(private_item_type), POINTER           :: item, prev_item
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         INTEGER :: i
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         CPASSERT(ASSOCIATED(hash_map%buckets))
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         DO i = 1, size(hash_map%buckets)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            item => hash_map%buckets(i)%p
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            DO WHILE (ASSOCIATED(item))
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
               prev_item => item
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
               item => item%next
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
               DEALLOCATE (prev_item)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            END DO
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         END DO
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         DEALLOCATE (hash_map%buckets)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         hash_map%size = -1
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      END SUBROUTINE routine_map_destroy
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \brief Stores, and possibly overwrites, a given value under a given key.
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param hash_map ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param key ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param value ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \author Ole Schuett
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      SUBROUTINE routine_map_set(hash_map, key, value)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(routine_map_type), INTENT(inout)             :: hash_map
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         CHARACTER(LEN=default_string_length), INTENT(in)                            :: key
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         INTEGER(kind=int_4), INTENT(in)                            :: value
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         INTEGER(KIND=int_8)                                   :: hash
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         CPASSERT(ASSOCIATED(hash_map%buckets))
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         hash = routine_map_hash_function(key)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         CALL routine_map_set_hashed(hash_map, key, value, hash)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      END SUBROUTINE routine_map_set
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \brief Common code used internally by routine_map_set() and routine_map_change_capacity().
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param hash_map ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param key ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param value ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param hash ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \author Ole Schuett
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      RECURSIVE SUBROUTINE routine_map_set_hashed(hash_map, key, value, hash)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(routine_map_type), INTENT(inout)               :: hash_map
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         CHARACTER(LEN=default_string_length), intent(in)                              :: key
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         INTEGER(kind=int_4), intent(in)                              :: value
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         INTEGER(KIND=int_8), intent(in)                         :: hash
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(private_item_type), POINTER                        :: item, new_item
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         INTEGER(KIND=int_8)                                     :: idx
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         idx = MOD(hash, INT(size(hash_map%buckets), KIND=int_8)) + 1
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         ! if already in hash map just update its value
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         item => hash_map%buckets(idx)%p
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         DO WHILE (ASSOCIATED(item))
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            IF (item%hash == hash) THEN
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
               IF (routine_map_keys_equal(item%key, key)) THEN
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
                  item%value =value
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
                  RETURN
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
               END IF
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            END IF
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            item => item%next
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         END DO
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         ! check load-factor
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         IF (4*hash_map%size > 3*size(hash_map%buckets)) THEN ! load-factor > 75%
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            call routine_map_change_capacity(hash_map, 2*size(hash_map%buckets)) !double capacity
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            idx = MOD(hash, INT(size(hash_map%buckets), KIND=int_8)) + 1
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         END IF
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         ! create a new item
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         allocate (new_item)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         new_item%hash = hash
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         new_item%key =key
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         new_item%value =value
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         new_item%next => hash_map%buckets(idx)%p
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         hash_map%buckets(idx)%p => new_item
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         hash_map%size = hash_map%size + 1
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      END SUBROUTINE routine_map_set_hashed
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \brief Internal routine for changing the hash map's capacity.
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param hash_map ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param new_capacity ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \author Ole Schuett
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      RECURSIVE SUBROUTINE routine_map_change_capacity(hash_map, new_capacity)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(routine_map_type), INTENT(inout)         :: hash_map
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         INTEGER, INTENT(in)                               :: new_capacity
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         INTEGER                                           :: i, old_size, new_cap
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(private_item_type), POINTER                  :: item, prev_item
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(private_item_p_type), DIMENSION(:), POINTER  :: old_buckets
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         new_cap = new_capacity
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         ! pre checks
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         IF (new_cap > HUGE(i)) THEN
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            IF (size(hash_map%buckets) == HUGE(i)) RETURN ! reached maximum - stay there.
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            new_cap = HUGE(i) ! grow as far as possible
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         END IF
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         CPASSERT(new_cap >= 1)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         CPASSERT(4*hash_map%size < 3*new_cap)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         old_size = hash_map%size
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         old_buckets => hash_map%buckets
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         ALLOCATE (hash_map%buckets(new_capacity))
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         hash_map%size = 0
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         DO i = 1, size(old_buckets)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            item => old_buckets(i)%p
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            DO WHILE (ASSOCIATED(item))
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
               CALL routine_map_set_hashed(hash_map, item%key, item%value, item%hash)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
               prev_item => item
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
               item => item%next
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
               DEALLOCATE (prev_item)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            END DO
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         END DO
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         DEALLOCATE (old_buckets)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         CPASSERT(old_size == hash_map%size)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      END SUBROUTINE routine_map_change_capacity
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \brief Gets a value for a given key from the hash map.
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!>        If the key is not found the default_value will be returned.
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!>        If the key is not found and default_value was not provided the program stops.
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param hash_map ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param key ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param default_value ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \return ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \author Ole Schuett
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      FUNCTION routine_map_get(hash_map, key, default_value) RESULT(value)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(routine_map_type), INTENT(in)              :: hash_map
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         CHARACTER(LEN=default_string_length), INTENT(in)                            :: key
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         INTEGER(kind=int_4), INTENT(in), OPTIONAL                :: default_value
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         INTEGER(kind=int_4)                                      :: value
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(private_item_type), POINTER                    :: item
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         INTEGER(KIND=int_8)                                 :: hash, idx
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         CPASSERT(ASSOCIATED(hash_map%buckets))
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         hash = routine_map_hash_function(key)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         idx = MOD(hash, INT(size(hash_map%buckets), KIND=int_8)) + 1
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         item => hash_map%buckets(idx)%p
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         DO WHILE (ASSOCIATED(item))
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            IF (item%hash == hash) THEN
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
               IF (routine_map_keys_equal(item%key, key)) THEN
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
                  value =item%value
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
                  RETURN
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
               END IF
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            END IF
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            item => item%next
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         END DO
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         IF (PRESENT(default_value)) THEN
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            value =default_value
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            RETURN
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         END IF
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         CPABORT("Key not found.")
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      END FUNCTION routine_map_get
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \brief Remove the value for a given key from the hash map.
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!>        If the key is not found the program stops.
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param hash_map ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param key ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \author Ole Schuett
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      SUBROUTINE routine_map_del(hash_map, key)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(routine_map_type), INTENT(inout)               :: hash_map
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         CHARACTER(LEN=default_string_length), INTENT(in)                                :: key
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(private_item_type), POINTER                        :: item, prev_item
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         INTEGER(KIND=int_8)                                     :: hash, idx
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         CPASSERT(ASSOCIATED(hash_map%buckets))
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         hash = routine_map_hash_function(key)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         idx = MOD(hash, INT(size(hash_map%buckets), KIND=int_8)) + 1
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         item => hash_map%buckets(idx)%p
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         prev_item => Null()
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         DO WHILE (ASSOCIATED(item))
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            IF (item%hash == hash) THEN
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
               IF (routine_map_keys_equal(item%key, key)) THEN
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
                  IF (ASSOCIATED(prev_item)) THEN
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
                     prev_item%next => item%next
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
                  ELSE
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
                     hash_map%buckets(idx)%p => item%next
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
                  END IF
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
                  DEALLOCATE (item)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
                  hash_map%size = hash_map%size - 1
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
                  RETURN
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
               END IF
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            END IF
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            prev_item => item
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            item => item%next
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         END DO
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         CPABORT("Key not found.")
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      END SUBROUTINE routine_map_del
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \brief Returns the number of key/value-items currently stored in the hash map.
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param hash_map ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \return ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \author Ole Schuett
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      FUNCTION routine_map_size(hash_map) RESULT(size)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(routine_map_type), INTENT(IN)  :: hash_map
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         INTEGER                                 :: size
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         CPASSERT(ASSOCIATED(hash_map%buckets))
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         size = hash_map%size
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      END FUNCTION routine_map_size
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \brief Checks whether a given key is currently stored in the hash_map.
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param hash_map ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param key ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \return ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \author Ole Schuett
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      FUNCTION routine_map_haskey(hash_map, key) RESULT(res)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(routine_map_type), INTENT(IN)                :: hash_map
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         CHARACTER(LEN=default_string_length), INTENT(IN)                              :: key
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         LOGICAL                                               :: res
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(private_item_type), POINTER                      :: item
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         INTEGER(KIND=int_8)                                   :: hash, idx
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         CPASSERT(ASSOCIATED(hash_map%buckets))
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         res = .FALSE.
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         IF (hash_map%size == 0) RETURN
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         hash = routine_map_hash_function(key)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         idx = MOD(hash, INT(size(hash_map%buckets), KIND=int_8)) + 1
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         item => hash_map%buckets(idx)%p
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         DO WHILE (ASSOCIATED(item))
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            IF (item%hash == hash) THEN
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
               IF (routine_map_keys_equal(item%key, key)) THEN
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
                  res = .TRUE.
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
                  return
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
               END IF
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            END IF
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            item => item%next
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         END DO
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      END FUNCTION routine_map_haskey
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \brief Returns a pointer to an array of all key/value-items stored in the hash map.
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!>        Caution: The caller is responsible for deallocating targeted array after usage.
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param hash_map ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \return ...
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \author Ole Schuett
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      FUNCTION routine_map_items(hash_map) RESULT(items)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(routine_map_type), INTENT(IN)                 :: hash_map
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(routine_map_item_type), DIMENSION(:), POINTER :: items
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(private_item_type), POINTER  :: item
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         INTEGER :: i, j
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         CPASSERT(ASSOCIATED(hash_map%buckets))
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         ALLOCATE (items(hash_map%size))
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         j = 1
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         DO i = 1, size(hash_map%buckets)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            item => hash_map%buckets(i)%p
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            DO WHILE (ASSOCIATED(item))
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
               items(j)%key =item%key
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
               items(j)%value =item%value
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
               j = j + 1
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
               item => item%next
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            END DO
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         END DO
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         CPASSERT(j == hash_map%size + 1)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      END FUNCTION routine_map_items
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \brief Copies all key/values-items from one hash map to another.
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!>        Afterwards hash_map will contain all items from the from_hash_map and
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!>        additionally all its previous items, which were not overwritten.
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!>        The two hash maps have to be of the same type.
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param hash_map destination of items
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \param from_hash_map source of items - will not be change
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
!> \author Ole Schuett
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
! **************************************************************************************************
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      SUBROUTINE routine_map_update(hash_map, from_hash_map)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(routine_map_type), INTENT(inout)              :: hash_map
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(routine_map_type), INTENT(in)                 :: from_hash_map
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         TYPE(routine_map_item_type), DIMENSION(:), POINTER :: from_items
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         INTEGER :: i
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         CPASSERT(ASSOCIATED(hash_map%buckets))
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         CPASSERT(ASSOCIATED(from_hash_map%buckets))
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         from_items => routine_map_items(from_hash_map)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         DO i = 1, size(from_items)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
            CALL routine_map_set(hash_map, from_items(i)%key, from_items(i)%value)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         END DO
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
         DEALLOCATE (from_items)
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"
      END SUBROUTINE routine_map_update
# 16 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

# 22 "/builddir/build/BUILD/cp2k-2024.1/src/common/routine_map.F"

! **************************************************************************************************
! This is joaat_hash from string_table.F
!
!> \brief generates the hash of a given string
!> \param key a string of any length
!> \return ...
!> \par History
!>       09.2006 created [Joost VandeVondele]
!>       12.2012 copied and adopted [ole]
!> \note
!>       http://en.wikipedia.org/wiki/Hash_table
!>       http://www.burtleburtle.net/bob/hash/doobs.html
! **************************************************************************************************
   PURE FUNCTION routine_map_hash_function(key) RESULT(hash)
      CHARACTER(LEN=*), INTENT(IN)                       :: key
      INTEGER(KIND=int_8)                                :: hash

      INTEGER(KIND=int_8), PARAMETER                     :: b32 = 2_int_8**32 - 1_int_8

      INTEGER                                            :: i

      hash = 0_int_8
      DO i = 1, LEN(key)
         hash = IAND(hash + ICHAR(key(i:i)), b32)
         hash = IAND(hash + IAND(ISHFT(hash, 10), b32), b32)
         hash = IAND(IEOR(hash, IAND(ISHFT(hash, -6), b32)), b32)
      END DO
      hash = IAND(hash + IAND(ISHFT(hash, 3), b32), b32)
      hash = IAND(IEOR(hash, IAND(ISHFT(hash, -11), b32)), b32)
      hash = IAND(hash + IAND(ISHFT(hash, 15), b32), b32)
   END FUNCTION routine_map_hash_function

! **************************************************************************************************
!> \brief ...
!> \param key ...
!> \return ...
! **************************************************************************************************
   PURE FUNCTION routine_map_keys_equal(key1, key2) RESULT(res)
      CHARACTER(LEN=*), INTENT(IN)                      :: key1, key2
      LOGICAL                                            :: res

      res = (key1 == key2)
   END FUNCTION routine_map_keys_equal

END MODULE routine_map
