# 1 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F" 1
!--------------------------------------------------------------------------------------------------!
!   CP2K: A general program to perform molecular dynamics simulations                              !
!   Copyright 2000-2024 CP2K developers group <https://cp2k.org>                                   !
!                                                                                                  !
!   SPDX-License-Identifier: GPL-2.0-or-later                                                      !
!--------------------------------------------------------------------------------------------------!

! **************************************************************************************************
!> \brief represent a simple array based list of the given type
!> \par History
!>      08.2003 created [fawzi]
!>      01.2017 ported to Fypp [Ole Schuett]
!> \author Fawzi Mohamed
! **************************************************************************************************
MODULE molecule_kind_list_types
   use molecule_kind_types, only: molecule_kind_type, deallocate_molecule_kind_set

#include "../base/base_uses.f90"

   IMPLICIT NONE
   PRIVATE

# 1 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/subsys_list.fypp" 1
# 7 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/subsys_list.fypp"
# 139 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/subsys_list.fypp"
# 24 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F" 2


# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
      LOGICAL, PRIVATE, PARAMETER :: debug_this_module = .TRUE.
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
      !API
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
      PUBLIC :: molecule_kind_list_type, molecule_kind_list_p_type, &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
                molecule_kind_list_create, molecule_kind_list_retain, &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
                molecule_kind_list_release
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \brief represent a list of objects
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \param ref_count reference count (see doc/ReferenceCounting.html)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \param n_el the number of elements in the list
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \param owns_list if the els are owned by this structure, and
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!>        should be deallocated by it
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \param list the array of object, might be oversized,
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!>        only the fist n_el have some meaning
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \par History
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!>      08.2003 created [fawzi]
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \author Fawzi Mohamed
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
      TYPE molecule_kind_list_type
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         INTEGER :: ref_count = 0, n_els = 0
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         LOGICAL :: owns_els = .FALSE.
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         TYPE(molecule_kind_type), DIMENSION(:), POINTER :: els => NULL()
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
      END TYPE molecule_kind_list_type
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \brief represents a pointer to a list
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \param list the pointer to the list
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \par History
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!>      08.2003 created [fawzi]
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \author Fawzi Mohamed
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
      TYPE molecule_kind_list_p_type
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         TYPE(molecule_kind_list_type), POINTER :: list => NULL()
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
      END TYPE molecule_kind_list_p_type
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
      CONTAINS
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \brief creates a list
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \param list the list to allocate and initialize
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \param els_ptr the elements to store in the list (the array is only,
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!>        referenced, not copied!)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \param owns_els if the list takes the ownership of els_ptr and
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!>        will deallocate it (defaults to true)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \param n_els number of elements in the list (at least one els_ptr or
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!>        n_els should be given)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \par History
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!>      08.2003 created [fawzi]
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \author Fawzi Mohamed
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
      SUBROUTINE molecule_kind_list_create(list, els_ptr, &
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
                                      owns_els, n_els)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         TYPE(molecule_kind_list_type), POINTER, OPTIONAL :: list
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         TYPE(molecule_kind_type), DIMENSION(:), POINTER, OPTIONAL :: els_ptr
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         LOGICAL, INTENT(in), OPTIONAL :: owns_els
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         INTEGER, INTENT(in), OPTIONAL :: n_els
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         CPASSERT(PRESENT(els_ptr) .OR. PRESENT(n_els))
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         ALLOCATE (list)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         list%ref_count = 1
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         list%owns_els = .TRUE.
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         list%n_els = 0
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         IF (PRESENT(owns_els)) list%owns_els = owns_els
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         NULLIFY (list%els)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         IF (PRESENT(els_ptr)) THEN
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
            list%els => els_ptr
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
            IF (ASSOCIATED(els_ptr)) THEN
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
               list%n_els = SIZE(els_ptr)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
            END IF
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         END IF
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         IF (PRESENT(n_els)) list%n_els = n_els
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         IF (.NOT. ASSOCIATED(list%els)) THEN
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
            ALLOCATE (list%els(list%n_els))
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
            CPASSERT(list%owns_els)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         END IF
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
      END SUBROUTINE molecule_kind_list_create
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \brief retains a list (see doc/ReferenceCounting.html)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \param list the list to retain
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \par History
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!>      08.2003 created [fawzi]
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \author Fawzi Mohamed
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
      SUBROUTINE molecule_kind_list_retain(list)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         TYPE(molecule_kind_list_type), POINTER :: list
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         CPASSERT(ASSOCIATED(list))
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         CPASSERT(list%ref_count > 0)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         list%ref_count = list%ref_count + 1
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
      END SUBROUTINE molecule_kind_list_retain
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \brief releases a  list (see doc/ReferenceCounting.html)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \param list the list to release
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \par History
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!>      08.2003 created [fawzi]
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
!> \author Fawzi Mohamed
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
! **************************************************************************************************
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
      SUBROUTINE molecule_kind_list_release(list)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         TYPE(molecule_kind_list_type), POINTER :: list
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"

# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         IF (ASSOCIATED(list)) THEN
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
            CPASSERT(list%ref_count > 0)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
            list%ref_count = list%ref_count - 1
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
            IF (list%ref_count == 0) THEN
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
               IF (list%owns_els) THEN
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
                  IF (ASSOCIATED(list%els)) THEN
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
                     CALL deallocate_molecule_kind_set(list%els)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
                  END IF
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
               END IF
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
               NULLIFY (list%els)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
               DEALLOCATE (list)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
            END IF
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         END IF
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
         NULLIFY (list)
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"
      END SUBROUTINE molecule_kind_list_release
# 25 "/builddir/build/BUILD/cp2k-2024.1/src/subsys/molecule_kind_list_types.F"


END MODULE molecule_kind_list_types
