# 1 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F" 1
!--------------------------------------------------------------------------------------------------!
!   CP2K: A general program to perform molecular dynamics simulations                              !
!   Copyright 2000-2024 CP2K developers group <https://cp2k.org>                                   !
!                                                                                                  !
!   SPDX-License-Identifier: GPL-2.0-or-later                                                      !
!--------------------------------------------------------------------------------------------------!
MODULE torch_api
   USE ISO_C_BINDING, ONLY: C_ASSOCIATED, &
                            C_BOOL, &
                            C_CHAR, &
                            C_FLOAT, &
                            C_DOUBLE, &
                            C_F_POINTER, &
                            C_INT, &
                            C_NULL_CHAR, &
                            C_NULL_PTR, &
                            C_PTR, &
                            C_INT64_T

   USE kinds, ONLY: sp, int_8, dp

#include "./base/base_uses.f90"

   IMPLICIT NONE

   PRIVATE

   TYPE torch_dict_type
      PRIVATE
      TYPE(C_PTR)                          :: c_ptr = C_NULL_PTR
   END TYPE torch_dict_type

   TYPE torch_model_type
      PRIVATE
      TYPE(C_PTR)                          :: c_ptr = C_NULL_PTR
   END TYPE torch_model_type

# 39 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
   INTERFACE torch_dict_insert
# 41 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
         MODULE PROCEDURE torch_dict_insert_float_1d
         MODULE PROCEDURE torch_dict_insert_int64_1d
         MODULE PROCEDURE torch_dict_insert_double_1d
# 41 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
         MODULE PROCEDURE torch_dict_insert_float_2d
         MODULE PROCEDURE torch_dict_insert_int64_2d
         MODULE PROCEDURE torch_dict_insert_double_2d
# 41 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
         MODULE PROCEDURE torch_dict_insert_float_3d
         MODULE PROCEDURE torch_dict_insert_int64_3d
         MODULE PROCEDURE torch_dict_insert_double_3d
# 45 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
   END INTERFACE torch_dict_insert

   INTERFACE torch_dict_get
# 49 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
         MODULE PROCEDURE torch_dict_get_float_1d
         MODULE PROCEDURE torch_dict_get_int64_1d
         MODULE PROCEDURE torch_dict_get_double_1d
# 49 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
         MODULE PROCEDURE torch_dict_get_float_2d
         MODULE PROCEDURE torch_dict_get_int64_2d
         MODULE PROCEDURE torch_dict_get_double_2d
# 49 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
         MODULE PROCEDURE torch_dict_get_float_3d
         MODULE PROCEDURE torch_dict_get_int64_3d
         MODULE PROCEDURE torch_dict_get_double_3d
# 53 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
   END INTERFACE torch_dict_get

   PUBLIC :: torch_dict_type, torch_dict_create, torch_dict_release
   PUBLIC :: torch_dict_insert, torch_dict_get
   PUBLIC :: torch_model_type, torch_model_load, torch_model_eval, torch_model_release
   PUBLIC :: torch_model_read_metadata
   PUBLIC :: torch_cuda_is_available, torch_allow_tf32, torch_model_freeze

CONTAINS

# 64 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
# 65 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
# 66 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"

# 68 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
# 69 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"

! **************************************************************************************************
!> \brief Inserts array into Torch dictionary. The passed array has to outlive the dictionary!
!> \author Ole Schuett
! **************************************************************************************************
         SUBROUTINE torch_dict_insert_float_1d(dict, key, source)
            TYPE(torch_dict_type), INTENT(INOUT)               :: dict
            CHARACTER(len=*), INTENT(IN)                       :: key
# 78 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            REAL(sp), CONTIGUOUS, DIMENSION(:), INTENT(IN)  :: source

#if defined(__LIBTORCH)
            INTEGER(kind=int_8), DIMENSION(1)          :: sizes_c

            INTERFACE
               SUBROUTINE torch_c_dict_insert_float (dict, key, ndims, sizes, source) &
                  BIND(C, name="torch_c_dict_insert_float")
                  IMPORT :: C_CHAR, C_PTR, C_INT, C_INT64_T, C_FLOAT, C_DOUBLE
                  TYPE(C_PTR), VALUE                           :: dict
                  CHARACTER(kind=C_CHAR), DIMENSION(*)         :: key
                  INTEGER(kind=C_INT), VALUE                   :: ndims
                  INTEGER(kind=C_INT64_T), DIMENSION(*)        :: sizes
                  REAL(kind=C_FLOAT), DIMENSION(*)                     :: source
               END SUBROUTINE torch_c_dict_insert_float
            END INTERFACE

# 96 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_c(1) = SIZE(source, 1) ! C arrays are stored row-major.
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"

            CPASSERT(C_ASSOCIATED(dict%c_ptr))
            CALL torch_c_dict_insert_float (dict=dict%c_ptr, &
                                                   key=TRIM(key)//C_NULL_CHAR, &
                                                   ndims=1, &
                                                   sizes=sizes_c, &
                                                   source=source)
#else
            CPABORT("CP2K compiled without the Torch library.")
            MARK_USED(dict)
            MARK_USED(key)
            MARK_USED(source)
#endif
         END SUBROUTINE torch_dict_insert_float_1d

! **************************************************************************************************
!> \brief Retrieves array from Torch dictionary. The returned array has to deallocated by caller!
!> \author Ole Schuett
! **************************************************************************************************
         SUBROUTINE torch_dict_get_float_1d(dict, key, dest)
            TYPE(torch_dict_type), INTENT(IN)                  :: dict
            CHARACTER(len=*), INTENT(IN)                       :: key
# 121 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            REAL(sp), DIMENSION(:), POINTER      :: dest

#if defined(__LIBTORCH)
            INTEGER(kind=int_8), DIMENSION(1)          :: sizes_f, sizes_c
            TYPE(C_PTR)                                        :: dest_c

            INTERFACE
               SUBROUTINE torch_c_dict_get_float (dict, key, ndims, sizes, dest) &
                  BIND(C, name="torch_c_dict_get_float")
                  IMPORT :: C_CHAR, C_PTR, C_INT, C_INT64_T
                  TYPE(C_PTR), VALUE                           :: dict
                  CHARACTER(kind=C_CHAR), DIMENSION(*)         :: key
                  INTEGER(kind=C_INT), VALUE                   :: ndims
                  INTEGER(kind=C_INT64_T), DIMENSION(*)        :: sizes
                  TYPE(C_PTR)                                  :: dest
               END SUBROUTINE torch_c_dict_get_float
            END INTERFACE

            sizes_c(:) = -1
            dest_c = C_NULL_PTR
            CPASSERT(C_ASSOCIATED(dict%c_ptr))
            CPASSERT(.NOT. ASSOCIATED(dest))
            CALL torch_c_dict_get_float (dict=dict%c_ptr, &
                                                key=TRIM(key)//C_NULL_CHAR, &
                                                ndims=1, &
                                                sizes=sizes_c, &
                                                dest=dest_c)

            CPASSERT(ALL(sizes_c >= 0))
            CPASSERT(C_ASSOCIATED(dest_c))

# 153 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_f(1) = sizes_c(1) ! C arrays are stored row-major.
# 155 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            CALL C_F_POINTER(dest_c, dest, shape=sizes_f)
#else
            CPABORT("CP2K compiled without the Torch library.")
            MARK_USED(dict)
            MARK_USED(key)
            MARK_USED(dest)
#endif
         END SUBROUTINE torch_dict_get_float_1d

# 69 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"

! **************************************************************************************************
!> \brief Inserts array into Torch dictionary. The passed array has to outlive the dictionary!
!> \author Ole Schuett
! **************************************************************************************************
         SUBROUTINE torch_dict_insert_int64_1d(dict, key, source)
            TYPE(torch_dict_type), INTENT(INOUT)               :: dict
            CHARACTER(len=*), INTENT(IN)                       :: key
# 78 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            INTEGER(kind=int_8), CONTIGUOUS, DIMENSION(:), INTENT(IN)  :: source

#if defined(__LIBTORCH)
            INTEGER(kind=int_8), DIMENSION(1)          :: sizes_c

            INTERFACE
               SUBROUTINE torch_c_dict_insert_int64 (dict, key, ndims, sizes, source) &
                  BIND(C, name="torch_c_dict_insert_int64")
                  IMPORT :: C_CHAR, C_PTR, C_INT, C_INT64_T, C_FLOAT, C_DOUBLE
                  TYPE(C_PTR), VALUE                           :: dict
                  CHARACTER(kind=C_CHAR), DIMENSION(*)         :: key
                  INTEGER(kind=C_INT), VALUE                   :: ndims
                  INTEGER(kind=C_INT64_T), DIMENSION(*)        :: sizes
                  INTEGER(kind=C_INT64_T), DIMENSION(*)                     :: source
               END SUBROUTINE torch_c_dict_insert_int64
            END INTERFACE

# 96 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_c(1) = SIZE(source, 1) ! C arrays are stored row-major.
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"

            CPASSERT(C_ASSOCIATED(dict%c_ptr))
            CALL torch_c_dict_insert_int64 (dict=dict%c_ptr, &
                                                   key=TRIM(key)//C_NULL_CHAR, &
                                                   ndims=1, &
                                                   sizes=sizes_c, &
                                                   source=source)
#else
            CPABORT("CP2K compiled without the Torch library.")
            MARK_USED(dict)
            MARK_USED(key)
            MARK_USED(source)
#endif
         END SUBROUTINE torch_dict_insert_int64_1d

! **************************************************************************************************
!> \brief Retrieves array from Torch dictionary. The returned array has to deallocated by caller!
!> \author Ole Schuett
! **************************************************************************************************
         SUBROUTINE torch_dict_get_int64_1d(dict, key, dest)
            TYPE(torch_dict_type), INTENT(IN)                  :: dict
            CHARACTER(len=*), INTENT(IN)                       :: key
# 121 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            INTEGER(kind=int_8), DIMENSION(:), POINTER      :: dest

#if defined(__LIBTORCH)
            INTEGER(kind=int_8), DIMENSION(1)          :: sizes_f, sizes_c
            TYPE(C_PTR)                                        :: dest_c

            INTERFACE
               SUBROUTINE torch_c_dict_get_int64 (dict, key, ndims, sizes, dest) &
                  BIND(C, name="torch_c_dict_get_int64")
                  IMPORT :: C_CHAR, C_PTR, C_INT, C_INT64_T
                  TYPE(C_PTR), VALUE                           :: dict
                  CHARACTER(kind=C_CHAR), DIMENSION(*)         :: key
                  INTEGER(kind=C_INT), VALUE                   :: ndims
                  INTEGER(kind=C_INT64_T), DIMENSION(*)        :: sizes
                  TYPE(C_PTR)                                  :: dest
               END SUBROUTINE torch_c_dict_get_int64
            END INTERFACE

            sizes_c(:) = -1
            dest_c = C_NULL_PTR
            CPASSERT(C_ASSOCIATED(dict%c_ptr))
            CPASSERT(.NOT. ASSOCIATED(dest))
            CALL torch_c_dict_get_int64 (dict=dict%c_ptr, &
                                                key=TRIM(key)//C_NULL_CHAR, &
                                                ndims=1, &
                                                sizes=sizes_c, &
                                                dest=dest_c)

            CPASSERT(ALL(sizes_c >= 0))
            CPASSERT(C_ASSOCIATED(dest_c))

# 153 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_f(1) = sizes_c(1) ! C arrays are stored row-major.
# 155 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            CALL C_F_POINTER(dest_c, dest, shape=sizes_f)
#else
            CPABORT("CP2K compiled without the Torch library.")
            MARK_USED(dict)
            MARK_USED(key)
            MARK_USED(dest)
#endif
         END SUBROUTINE torch_dict_get_int64_1d

# 69 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"

! **************************************************************************************************
!> \brief Inserts array into Torch dictionary. The passed array has to outlive the dictionary!
!> \author Ole Schuett
! **************************************************************************************************
         SUBROUTINE torch_dict_insert_double_1d(dict, key, source)
            TYPE(torch_dict_type), INTENT(INOUT)               :: dict
            CHARACTER(len=*), INTENT(IN)                       :: key
# 78 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            REAL(dp), CONTIGUOUS, DIMENSION(:), INTENT(IN)  :: source

#if defined(__LIBTORCH)
            INTEGER(kind=int_8), DIMENSION(1)          :: sizes_c

            INTERFACE
               SUBROUTINE torch_c_dict_insert_double (dict, key, ndims, sizes, source) &
                  BIND(C, name="torch_c_dict_insert_double")
                  IMPORT :: C_CHAR, C_PTR, C_INT, C_INT64_T, C_FLOAT, C_DOUBLE
                  TYPE(C_PTR), VALUE                           :: dict
                  CHARACTER(kind=C_CHAR), DIMENSION(*)         :: key
                  INTEGER(kind=C_INT), VALUE                   :: ndims
                  INTEGER(kind=C_INT64_T), DIMENSION(*)        :: sizes
                  REAL(kind=C_DOUBLE), DIMENSION(*)                     :: source
               END SUBROUTINE torch_c_dict_insert_double
            END INTERFACE

# 96 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_c(1) = SIZE(source, 1) ! C arrays are stored row-major.
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"

            CPASSERT(C_ASSOCIATED(dict%c_ptr))
            CALL torch_c_dict_insert_double (dict=dict%c_ptr, &
                                                   key=TRIM(key)//C_NULL_CHAR, &
                                                   ndims=1, &
                                                   sizes=sizes_c, &
                                                   source=source)
#else
            CPABORT("CP2K compiled without the Torch library.")
            MARK_USED(dict)
            MARK_USED(key)
            MARK_USED(source)
#endif
         END SUBROUTINE torch_dict_insert_double_1d

! **************************************************************************************************
!> \brief Retrieves array from Torch dictionary. The returned array has to deallocated by caller!
!> \author Ole Schuett
! **************************************************************************************************
         SUBROUTINE torch_dict_get_double_1d(dict, key, dest)
            TYPE(torch_dict_type), INTENT(IN)                  :: dict
            CHARACTER(len=*), INTENT(IN)                       :: key
# 121 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            REAL(dp), DIMENSION(:), POINTER      :: dest

#if defined(__LIBTORCH)
            INTEGER(kind=int_8), DIMENSION(1)          :: sizes_f, sizes_c
            TYPE(C_PTR)                                        :: dest_c

            INTERFACE
               SUBROUTINE torch_c_dict_get_double (dict, key, ndims, sizes, dest) &
                  BIND(C, name="torch_c_dict_get_double")
                  IMPORT :: C_CHAR, C_PTR, C_INT, C_INT64_T
                  TYPE(C_PTR), VALUE                           :: dict
                  CHARACTER(kind=C_CHAR), DIMENSION(*)         :: key
                  INTEGER(kind=C_INT), VALUE                   :: ndims
                  INTEGER(kind=C_INT64_T), DIMENSION(*)        :: sizes
                  TYPE(C_PTR)                                  :: dest
               END SUBROUTINE torch_c_dict_get_double
            END INTERFACE

            sizes_c(:) = -1
            dest_c = C_NULL_PTR
            CPASSERT(C_ASSOCIATED(dict%c_ptr))
            CPASSERT(.NOT. ASSOCIATED(dest))
            CALL torch_c_dict_get_double (dict=dict%c_ptr, &
                                                key=TRIM(key)//C_NULL_CHAR, &
                                                ndims=1, &
                                                sizes=sizes_c, &
                                                dest=dest_c)

            CPASSERT(ALL(sizes_c >= 0))
            CPASSERT(C_ASSOCIATED(dest_c))

# 153 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_f(1) = sizes_c(1) ! C arrays are stored row-major.
# 155 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            CALL C_F_POINTER(dest_c, dest, shape=sizes_f)
#else
            CPABORT("CP2K compiled without the Torch library.")
            MARK_USED(dict)
            MARK_USED(key)
            MARK_USED(dest)
#endif
         END SUBROUTINE torch_dict_get_double_1d

# 165 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
# 68 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
# 69 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"

! **************************************************************************************************
!> \brief Inserts array into Torch dictionary. The passed array has to outlive the dictionary!
!> \author Ole Schuett
! **************************************************************************************************
         SUBROUTINE torch_dict_insert_float_2d(dict, key, source)
            TYPE(torch_dict_type), INTENT(INOUT)               :: dict
            CHARACTER(len=*), INTENT(IN)                       :: key
# 78 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            REAL(sp), CONTIGUOUS, DIMENSION(:, :), INTENT(IN)  :: source

#if defined(__LIBTORCH)
            INTEGER(kind=int_8), DIMENSION(2)          :: sizes_c

            INTERFACE
               SUBROUTINE torch_c_dict_insert_float (dict, key, ndims, sizes, source) &
                  BIND(C, name="torch_c_dict_insert_float")
                  IMPORT :: C_CHAR, C_PTR, C_INT, C_INT64_T, C_FLOAT, C_DOUBLE
                  TYPE(C_PTR), VALUE                           :: dict
                  CHARACTER(kind=C_CHAR), DIMENSION(*)         :: key
                  INTEGER(kind=C_INT), VALUE                   :: ndims
                  INTEGER(kind=C_INT64_T), DIMENSION(*)        :: sizes
                  REAL(kind=C_FLOAT), DIMENSION(*)                     :: source
               END SUBROUTINE torch_c_dict_insert_float
            END INTERFACE

# 96 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_c(1) = SIZE(source, 2) ! C arrays are stored row-major.
# 96 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_c(2) = SIZE(source, 1) ! C arrays are stored row-major.
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"

            CPASSERT(C_ASSOCIATED(dict%c_ptr))
            CALL torch_c_dict_insert_float (dict=dict%c_ptr, &
                                                   key=TRIM(key)//C_NULL_CHAR, &
                                                   ndims=2, &
                                                   sizes=sizes_c, &
                                                   source=source)
#else
            CPABORT("CP2K compiled without the Torch library.")
            MARK_USED(dict)
            MARK_USED(key)
            MARK_USED(source)
#endif
         END SUBROUTINE torch_dict_insert_float_2d

! **************************************************************************************************
!> \brief Retrieves array from Torch dictionary. The returned array has to deallocated by caller!
!> \author Ole Schuett
! **************************************************************************************************
         SUBROUTINE torch_dict_get_float_2d(dict, key, dest)
            TYPE(torch_dict_type), INTENT(IN)                  :: dict
            CHARACTER(len=*), INTENT(IN)                       :: key
# 121 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            REAL(sp), DIMENSION(:, :), POINTER      :: dest

#if defined(__LIBTORCH)
            INTEGER(kind=int_8), DIMENSION(2)          :: sizes_f, sizes_c
            TYPE(C_PTR)                                        :: dest_c

            INTERFACE
               SUBROUTINE torch_c_dict_get_float (dict, key, ndims, sizes, dest) &
                  BIND(C, name="torch_c_dict_get_float")
                  IMPORT :: C_CHAR, C_PTR, C_INT, C_INT64_T
                  TYPE(C_PTR), VALUE                           :: dict
                  CHARACTER(kind=C_CHAR), DIMENSION(*)         :: key
                  INTEGER(kind=C_INT), VALUE                   :: ndims
                  INTEGER(kind=C_INT64_T), DIMENSION(*)        :: sizes
                  TYPE(C_PTR)                                  :: dest
               END SUBROUTINE torch_c_dict_get_float
            END INTERFACE

            sizes_c(:) = -1
            dest_c = C_NULL_PTR
            CPASSERT(C_ASSOCIATED(dict%c_ptr))
            CPASSERT(.NOT. ASSOCIATED(dest))
            CALL torch_c_dict_get_float (dict=dict%c_ptr, &
                                                key=TRIM(key)//C_NULL_CHAR, &
                                                ndims=2, &
                                                sizes=sizes_c, &
                                                dest=dest_c)

            CPASSERT(ALL(sizes_c >= 0))
            CPASSERT(C_ASSOCIATED(dest_c))

# 153 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_f(1) = sizes_c(2) ! C arrays are stored row-major.
# 153 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_f(2) = sizes_c(1) ! C arrays are stored row-major.
# 155 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            CALL C_F_POINTER(dest_c, dest, shape=sizes_f)
#else
            CPABORT("CP2K compiled without the Torch library.")
            MARK_USED(dict)
            MARK_USED(key)
            MARK_USED(dest)
#endif
         END SUBROUTINE torch_dict_get_float_2d

# 69 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"

! **************************************************************************************************
!> \brief Inserts array into Torch dictionary. The passed array has to outlive the dictionary!
!> \author Ole Schuett
! **************************************************************************************************
         SUBROUTINE torch_dict_insert_int64_2d(dict, key, source)
            TYPE(torch_dict_type), INTENT(INOUT)               :: dict
            CHARACTER(len=*), INTENT(IN)                       :: key
# 78 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            INTEGER(kind=int_8), CONTIGUOUS, DIMENSION(:, :), INTENT(IN)  :: source

#if defined(__LIBTORCH)
            INTEGER(kind=int_8), DIMENSION(2)          :: sizes_c

            INTERFACE
               SUBROUTINE torch_c_dict_insert_int64 (dict, key, ndims, sizes, source) &
                  BIND(C, name="torch_c_dict_insert_int64")
                  IMPORT :: C_CHAR, C_PTR, C_INT, C_INT64_T, C_FLOAT, C_DOUBLE
                  TYPE(C_PTR), VALUE                           :: dict
                  CHARACTER(kind=C_CHAR), DIMENSION(*)         :: key
                  INTEGER(kind=C_INT), VALUE                   :: ndims
                  INTEGER(kind=C_INT64_T), DIMENSION(*)        :: sizes
                  INTEGER(kind=C_INT64_T), DIMENSION(*)                     :: source
               END SUBROUTINE torch_c_dict_insert_int64
            END INTERFACE

# 96 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_c(1) = SIZE(source, 2) ! C arrays are stored row-major.
# 96 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_c(2) = SIZE(source, 1) ! C arrays are stored row-major.
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"

            CPASSERT(C_ASSOCIATED(dict%c_ptr))
            CALL torch_c_dict_insert_int64 (dict=dict%c_ptr, &
                                                   key=TRIM(key)//C_NULL_CHAR, &
                                                   ndims=2, &
                                                   sizes=sizes_c, &
                                                   source=source)
#else
            CPABORT("CP2K compiled without the Torch library.")
            MARK_USED(dict)
            MARK_USED(key)
            MARK_USED(source)
#endif
         END SUBROUTINE torch_dict_insert_int64_2d

! **************************************************************************************************
!> \brief Retrieves array from Torch dictionary. The returned array has to deallocated by caller!
!> \author Ole Schuett
! **************************************************************************************************
         SUBROUTINE torch_dict_get_int64_2d(dict, key, dest)
            TYPE(torch_dict_type), INTENT(IN)                  :: dict
            CHARACTER(len=*), INTENT(IN)                       :: key
# 121 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            INTEGER(kind=int_8), DIMENSION(:, :), POINTER      :: dest

#if defined(__LIBTORCH)
            INTEGER(kind=int_8), DIMENSION(2)          :: sizes_f, sizes_c
            TYPE(C_PTR)                                        :: dest_c

            INTERFACE
               SUBROUTINE torch_c_dict_get_int64 (dict, key, ndims, sizes, dest) &
                  BIND(C, name="torch_c_dict_get_int64")
                  IMPORT :: C_CHAR, C_PTR, C_INT, C_INT64_T
                  TYPE(C_PTR), VALUE                           :: dict
                  CHARACTER(kind=C_CHAR), DIMENSION(*)         :: key
                  INTEGER(kind=C_INT), VALUE                   :: ndims
                  INTEGER(kind=C_INT64_T), DIMENSION(*)        :: sizes
                  TYPE(C_PTR)                                  :: dest
               END SUBROUTINE torch_c_dict_get_int64
            END INTERFACE

            sizes_c(:) = -1
            dest_c = C_NULL_PTR
            CPASSERT(C_ASSOCIATED(dict%c_ptr))
            CPASSERT(.NOT. ASSOCIATED(dest))
            CALL torch_c_dict_get_int64 (dict=dict%c_ptr, &
                                                key=TRIM(key)//C_NULL_CHAR, &
                                                ndims=2, &
                                                sizes=sizes_c, &
                                                dest=dest_c)

            CPASSERT(ALL(sizes_c >= 0))
            CPASSERT(C_ASSOCIATED(dest_c))

# 153 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_f(1) = sizes_c(2) ! C arrays are stored row-major.
# 153 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_f(2) = sizes_c(1) ! C arrays are stored row-major.
# 155 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            CALL C_F_POINTER(dest_c, dest, shape=sizes_f)
#else
            CPABORT("CP2K compiled without the Torch library.")
            MARK_USED(dict)
            MARK_USED(key)
            MARK_USED(dest)
#endif
         END SUBROUTINE torch_dict_get_int64_2d

# 69 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"

! **************************************************************************************************
!> \brief Inserts array into Torch dictionary. The passed array has to outlive the dictionary!
!> \author Ole Schuett
! **************************************************************************************************
         SUBROUTINE torch_dict_insert_double_2d(dict, key, source)
            TYPE(torch_dict_type), INTENT(INOUT)               :: dict
            CHARACTER(len=*), INTENT(IN)                       :: key
# 78 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            REAL(dp), CONTIGUOUS, DIMENSION(:, :), INTENT(IN)  :: source

#if defined(__LIBTORCH)
            INTEGER(kind=int_8), DIMENSION(2)          :: sizes_c

            INTERFACE
               SUBROUTINE torch_c_dict_insert_double (dict, key, ndims, sizes, source) &
                  BIND(C, name="torch_c_dict_insert_double")
                  IMPORT :: C_CHAR, C_PTR, C_INT, C_INT64_T, C_FLOAT, C_DOUBLE
                  TYPE(C_PTR), VALUE                           :: dict
                  CHARACTER(kind=C_CHAR), DIMENSION(*)         :: key
                  INTEGER(kind=C_INT), VALUE                   :: ndims
                  INTEGER(kind=C_INT64_T), DIMENSION(*)        :: sizes
                  REAL(kind=C_DOUBLE), DIMENSION(*)                     :: source
               END SUBROUTINE torch_c_dict_insert_double
            END INTERFACE

# 96 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_c(1) = SIZE(source, 2) ! C arrays are stored row-major.
# 96 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_c(2) = SIZE(source, 1) ! C arrays are stored row-major.
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"

            CPASSERT(C_ASSOCIATED(dict%c_ptr))
            CALL torch_c_dict_insert_double (dict=dict%c_ptr, &
                                                   key=TRIM(key)//C_NULL_CHAR, &
                                                   ndims=2, &
                                                   sizes=sizes_c, &
                                                   source=source)
#else
            CPABORT("CP2K compiled without the Torch library.")
            MARK_USED(dict)
            MARK_USED(key)
            MARK_USED(source)
#endif
         END SUBROUTINE torch_dict_insert_double_2d

! **************************************************************************************************
!> \brief Retrieves array from Torch dictionary. The returned array has to deallocated by caller!
!> \author Ole Schuett
! **************************************************************************************************
         SUBROUTINE torch_dict_get_double_2d(dict, key, dest)
            TYPE(torch_dict_type), INTENT(IN)                  :: dict
            CHARACTER(len=*), INTENT(IN)                       :: key
# 121 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            REAL(dp), DIMENSION(:, :), POINTER      :: dest

#if defined(__LIBTORCH)
            INTEGER(kind=int_8), DIMENSION(2)          :: sizes_f, sizes_c
            TYPE(C_PTR)                                        :: dest_c

            INTERFACE
               SUBROUTINE torch_c_dict_get_double (dict, key, ndims, sizes, dest) &
                  BIND(C, name="torch_c_dict_get_double")
                  IMPORT :: C_CHAR, C_PTR, C_INT, C_INT64_T
                  TYPE(C_PTR), VALUE                           :: dict
                  CHARACTER(kind=C_CHAR), DIMENSION(*)         :: key
                  INTEGER(kind=C_INT), VALUE                   :: ndims
                  INTEGER(kind=C_INT64_T), DIMENSION(*)        :: sizes
                  TYPE(C_PTR)                                  :: dest
               END SUBROUTINE torch_c_dict_get_double
            END INTERFACE

            sizes_c(:) = -1
            dest_c = C_NULL_PTR
            CPASSERT(C_ASSOCIATED(dict%c_ptr))
            CPASSERT(.NOT. ASSOCIATED(dest))
            CALL torch_c_dict_get_double (dict=dict%c_ptr, &
                                                key=TRIM(key)//C_NULL_CHAR, &
                                                ndims=2, &
                                                sizes=sizes_c, &
                                                dest=dest_c)

            CPASSERT(ALL(sizes_c >= 0))
            CPASSERT(C_ASSOCIATED(dest_c))

# 153 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_f(1) = sizes_c(2) ! C arrays are stored row-major.
# 153 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_f(2) = sizes_c(1) ! C arrays are stored row-major.
# 155 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            CALL C_F_POINTER(dest_c, dest, shape=sizes_f)
#else
            CPABORT("CP2K compiled without the Torch library.")
            MARK_USED(dict)
            MARK_USED(key)
            MARK_USED(dest)
#endif
         END SUBROUTINE torch_dict_get_double_2d

# 165 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
# 68 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
# 69 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"

! **************************************************************************************************
!> \brief Inserts array into Torch dictionary. The passed array has to outlive the dictionary!
!> \author Ole Schuett
! **************************************************************************************************
         SUBROUTINE torch_dict_insert_float_3d(dict, key, source)
            TYPE(torch_dict_type), INTENT(INOUT)               :: dict
            CHARACTER(len=*), INTENT(IN)                       :: key
# 78 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            REAL(sp), CONTIGUOUS, DIMENSION(:, :, :), INTENT(IN)  :: source

#if defined(__LIBTORCH)
            INTEGER(kind=int_8), DIMENSION(3)          :: sizes_c

            INTERFACE
               SUBROUTINE torch_c_dict_insert_float (dict, key, ndims, sizes, source) &
                  BIND(C, name="torch_c_dict_insert_float")
                  IMPORT :: C_CHAR, C_PTR, C_INT, C_INT64_T, C_FLOAT, C_DOUBLE
                  TYPE(C_PTR), VALUE                           :: dict
                  CHARACTER(kind=C_CHAR), DIMENSION(*)         :: key
                  INTEGER(kind=C_INT), VALUE                   :: ndims
                  INTEGER(kind=C_INT64_T), DIMENSION(*)        :: sizes
                  REAL(kind=C_FLOAT), DIMENSION(*)                     :: source
               END SUBROUTINE torch_c_dict_insert_float
            END INTERFACE

# 96 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_c(1) = SIZE(source, 3) ! C arrays are stored row-major.
# 96 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_c(2) = SIZE(source, 2) ! C arrays are stored row-major.
# 96 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_c(3) = SIZE(source, 1) ! C arrays are stored row-major.
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"

            CPASSERT(C_ASSOCIATED(dict%c_ptr))
            CALL torch_c_dict_insert_float (dict=dict%c_ptr, &
                                                   key=TRIM(key)//C_NULL_CHAR, &
                                                   ndims=3, &
                                                   sizes=sizes_c, &
                                                   source=source)
#else
            CPABORT("CP2K compiled without the Torch library.")
            MARK_USED(dict)
            MARK_USED(key)
            MARK_USED(source)
#endif
         END SUBROUTINE torch_dict_insert_float_3d

! **************************************************************************************************
!> \brief Retrieves array from Torch dictionary. The returned array has to deallocated by caller!
!> \author Ole Schuett
! **************************************************************************************************
         SUBROUTINE torch_dict_get_float_3d(dict, key, dest)
            TYPE(torch_dict_type), INTENT(IN)                  :: dict
            CHARACTER(len=*), INTENT(IN)                       :: key
# 121 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            REAL(sp), DIMENSION(:, :, :), POINTER      :: dest

#if defined(__LIBTORCH)
            INTEGER(kind=int_8), DIMENSION(3)          :: sizes_f, sizes_c
            TYPE(C_PTR)                                        :: dest_c

            INTERFACE
               SUBROUTINE torch_c_dict_get_float (dict, key, ndims, sizes, dest) &
                  BIND(C, name="torch_c_dict_get_float")
                  IMPORT :: C_CHAR, C_PTR, C_INT, C_INT64_T
                  TYPE(C_PTR), VALUE                           :: dict
                  CHARACTER(kind=C_CHAR), DIMENSION(*)         :: key
                  INTEGER(kind=C_INT), VALUE                   :: ndims
                  INTEGER(kind=C_INT64_T), DIMENSION(*)        :: sizes
                  TYPE(C_PTR)                                  :: dest
               END SUBROUTINE torch_c_dict_get_float
            END INTERFACE

            sizes_c(:) = -1
            dest_c = C_NULL_PTR
            CPASSERT(C_ASSOCIATED(dict%c_ptr))
            CPASSERT(.NOT. ASSOCIATED(dest))
            CALL torch_c_dict_get_float (dict=dict%c_ptr, &
                                                key=TRIM(key)//C_NULL_CHAR, &
                                                ndims=3, &
                                                sizes=sizes_c, &
                                                dest=dest_c)

            CPASSERT(ALL(sizes_c >= 0))
            CPASSERT(C_ASSOCIATED(dest_c))

# 153 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_f(1) = sizes_c(3) ! C arrays are stored row-major.
# 153 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_f(2) = sizes_c(2) ! C arrays are stored row-major.
# 153 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_f(3) = sizes_c(1) ! C arrays are stored row-major.
# 155 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            CALL C_F_POINTER(dest_c, dest, shape=sizes_f)
#else
            CPABORT("CP2K compiled without the Torch library.")
            MARK_USED(dict)
            MARK_USED(key)
            MARK_USED(dest)
#endif
         END SUBROUTINE torch_dict_get_float_3d

# 69 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"

! **************************************************************************************************
!> \brief Inserts array into Torch dictionary. The passed array has to outlive the dictionary!
!> \author Ole Schuett
! **************************************************************************************************
         SUBROUTINE torch_dict_insert_int64_3d(dict, key, source)
            TYPE(torch_dict_type), INTENT(INOUT)               :: dict
            CHARACTER(len=*), INTENT(IN)                       :: key
# 78 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            INTEGER(kind=int_8), CONTIGUOUS, DIMENSION(:, :, :), INTENT(IN)  :: source

#if defined(__LIBTORCH)
            INTEGER(kind=int_8), DIMENSION(3)          :: sizes_c

            INTERFACE
               SUBROUTINE torch_c_dict_insert_int64 (dict, key, ndims, sizes, source) &
                  BIND(C, name="torch_c_dict_insert_int64")
                  IMPORT :: C_CHAR, C_PTR, C_INT, C_INT64_T, C_FLOAT, C_DOUBLE
                  TYPE(C_PTR), VALUE                           :: dict
                  CHARACTER(kind=C_CHAR), DIMENSION(*)         :: key
                  INTEGER(kind=C_INT), VALUE                   :: ndims
                  INTEGER(kind=C_INT64_T), DIMENSION(*)        :: sizes
                  INTEGER(kind=C_INT64_T), DIMENSION(*)                     :: source
               END SUBROUTINE torch_c_dict_insert_int64
            END INTERFACE

# 96 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_c(1) = SIZE(source, 3) ! C arrays are stored row-major.
# 96 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_c(2) = SIZE(source, 2) ! C arrays are stored row-major.
# 96 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_c(3) = SIZE(source, 1) ! C arrays are stored row-major.
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"

            CPASSERT(C_ASSOCIATED(dict%c_ptr))
            CALL torch_c_dict_insert_int64 (dict=dict%c_ptr, &
                                                   key=TRIM(key)//C_NULL_CHAR, &
                                                   ndims=3, &
                                                   sizes=sizes_c, &
                                                   source=source)
#else
            CPABORT("CP2K compiled without the Torch library.")
            MARK_USED(dict)
            MARK_USED(key)
            MARK_USED(source)
#endif
         END SUBROUTINE torch_dict_insert_int64_3d

! **************************************************************************************************
!> \brief Retrieves array from Torch dictionary. The returned array has to deallocated by caller!
!> \author Ole Schuett
! **************************************************************************************************
         SUBROUTINE torch_dict_get_int64_3d(dict, key, dest)
            TYPE(torch_dict_type), INTENT(IN)                  :: dict
            CHARACTER(len=*), INTENT(IN)                       :: key
# 121 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            INTEGER(kind=int_8), DIMENSION(:, :, :), POINTER      :: dest

#if defined(__LIBTORCH)
            INTEGER(kind=int_8), DIMENSION(3)          :: sizes_f, sizes_c
            TYPE(C_PTR)                                        :: dest_c

            INTERFACE
               SUBROUTINE torch_c_dict_get_int64 (dict, key, ndims, sizes, dest) &
                  BIND(C, name="torch_c_dict_get_int64")
                  IMPORT :: C_CHAR, C_PTR, C_INT, C_INT64_T
                  TYPE(C_PTR), VALUE                           :: dict
                  CHARACTER(kind=C_CHAR), DIMENSION(*)         :: key
                  INTEGER(kind=C_INT), VALUE                   :: ndims
                  INTEGER(kind=C_INT64_T), DIMENSION(*)        :: sizes
                  TYPE(C_PTR)                                  :: dest
               END SUBROUTINE torch_c_dict_get_int64
            END INTERFACE

            sizes_c(:) = -1
            dest_c = C_NULL_PTR
            CPASSERT(C_ASSOCIATED(dict%c_ptr))
            CPASSERT(.NOT. ASSOCIATED(dest))
            CALL torch_c_dict_get_int64 (dict=dict%c_ptr, &
                                                key=TRIM(key)//C_NULL_CHAR, &
                                                ndims=3, &
                                                sizes=sizes_c, &
                                                dest=dest_c)

            CPASSERT(ALL(sizes_c >= 0))
            CPASSERT(C_ASSOCIATED(dest_c))

# 153 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_f(1) = sizes_c(3) ! C arrays are stored row-major.
# 153 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_f(2) = sizes_c(2) ! C arrays are stored row-major.
# 153 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_f(3) = sizes_c(1) ! C arrays are stored row-major.
# 155 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            CALL C_F_POINTER(dest_c, dest, shape=sizes_f)
#else
            CPABORT("CP2K compiled without the Torch library.")
            MARK_USED(dict)
            MARK_USED(key)
            MARK_USED(dest)
#endif
         END SUBROUTINE torch_dict_get_int64_3d

# 69 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"

! **************************************************************************************************
!> \brief Inserts array into Torch dictionary. The passed array has to outlive the dictionary!
!> \author Ole Schuett
! **************************************************************************************************
         SUBROUTINE torch_dict_insert_double_3d(dict, key, source)
            TYPE(torch_dict_type), INTENT(INOUT)               :: dict
            CHARACTER(len=*), INTENT(IN)                       :: key
# 78 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            REAL(dp), CONTIGUOUS, DIMENSION(:, :, :), INTENT(IN)  :: source

#if defined(__LIBTORCH)
            INTEGER(kind=int_8), DIMENSION(3)          :: sizes_c

            INTERFACE
               SUBROUTINE torch_c_dict_insert_double (dict, key, ndims, sizes, source) &
                  BIND(C, name="torch_c_dict_insert_double")
                  IMPORT :: C_CHAR, C_PTR, C_INT, C_INT64_T, C_FLOAT, C_DOUBLE
                  TYPE(C_PTR), VALUE                           :: dict
                  CHARACTER(kind=C_CHAR), DIMENSION(*)         :: key
                  INTEGER(kind=C_INT), VALUE                   :: ndims
                  INTEGER(kind=C_INT64_T), DIMENSION(*)        :: sizes
                  REAL(kind=C_DOUBLE), DIMENSION(*)                     :: source
               END SUBROUTINE torch_c_dict_insert_double
            END INTERFACE

# 96 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_c(1) = SIZE(source, 3) ! C arrays are stored row-major.
# 96 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_c(2) = SIZE(source, 2) ! C arrays are stored row-major.
# 96 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_c(3) = SIZE(source, 1) ! C arrays are stored row-major.
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"

            CPASSERT(C_ASSOCIATED(dict%c_ptr))
            CALL torch_c_dict_insert_double (dict=dict%c_ptr, &
                                                   key=TRIM(key)//C_NULL_CHAR, &
                                                   ndims=3, &
                                                   sizes=sizes_c, &
                                                   source=source)
#else
            CPABORT("CP2K compiled without the Torch library.")
            MARK_USED(dict)
            MARK_USED(key)
            MARK_USED(source)
#endif
         END SUBROUTINE torch_dict_insert_double_3d

! **************************************************************************************************
!> \brief Retrieves array from Torch dictionary. The returned array has to deallocated by caller!
!> \author Ole Schuett
! **************************************************************************************************
         SUBROUTINE torch_dict_get_double_3d(dict, key, dest)
            TYPE(torch_dict_type), INTENT(IN)                  :: dict
            CHARACTER(len=*), INTENT(IN)                       :: key
# 121 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            REAL(dp), DIMENSION(:, :, :), POINTER      :: dest

#if defined(__LIBTORCH)
            INTEGER(kind=int_8), DIMENSION(3)          :: sizes_f, sizes_c
            TYPE(C_PTR)                                        :: dest_c

            INTERFACE
               SUBROUTINE torch_c_dict_get_double (dict, key, ndims, sizes, dest) &
                  BIND(C, name="torch_c_dict_get_double")
                  IMPORT :: C_CHAR, C_PTR, C_INT, C_INT64_T
                  TYPE(C_PTR), VALUE                           :: dict
                  CHARACTER(kind=C_CHAR), DIMENSION(*)         :: key
                  INTEGER(kind=C_INT), VALUE                   :: ndims
                  INTEGER(kind=C_INT64_T), DIMENSION(*)        :: sizes
                  TYPE(C_PTR)                                  :: dest
               END SUBROUTINE torch_c_dict_get_double
            END INTERFACE

            sizes_c(:) = -1
            dest_c = C_NULL_PTR
            CPASSERT(C_ASSOCIATED(dict%c_ptr))
            CPASSERT(.NOT. ASSOCIATED(dest))
            CALL torch_c_dict_get_double (dict=dict%c_ptr, &
                                                key=TRIM(key)//C_NULL_CHAR, &
                                                ndims=3, &
                                                sizes=sizes_c, &
                                                dest=dest_c)

            CPASSERT(ALL(sizes_c >= 0))
            CPASSERT(C_ASSOCIATED(dest_c))

# 153 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_f(1) = sizes_c(3) ! C arrays are stored row-major.
# 153 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_f(2) = sizes_c(2) ! C arrays are stored row-major.
# 153 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
               sizes_f(3) = sizes_c(1) ! C arrays are stored row-major.
# 155 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
            CALL C_F_POINTER(dest_c, dest, shape=sizes_f)
#else
            CPABORT("CP2K compiled without the Torch library.")
            MARK_USED(dict)
            MARK_USED(key)
            MARK_USED(dest)
#endif
         END SUBROUTINE torch_dict_get_double_3d

# 165 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"
# 166 "/builddir/build/BUILD/cp2k-2024.1/src/torch_api.F"

! **************************************************************************************************
!> \brief Creates an empty Torch dictionary.
!> \author Ole Schuett
! **************************************************************************************************
   SUBROUTINE torch_dict_create(dict)
      TYPE(torch_dict_type), INTENT(INOUT)               :: dict

#if defined(__LIBTORCH)
      INTERFACE
         SUBROUTINE torch_c_dict_create(dict) BIND(C, name="torch_c_dict_create")
            IMPORT :: C_PTR
            TYPE(C_PTR)                               :: dict
         END SUBROUTINE torch_c_dict_create
      END INTERFACE

      CPASSERT(.NOT. C_ASSOCIATED(dict%c_ptr))
      CALL torch_c_dict_create(dict=dict%c_ptr)
      CPASSERT(C_ASSOCIATED(dict%c_ptr))
#else
      CPABORT("CP2K was compiled without Torch library.")
      MARK_USED(dict)
#endif
   END SUBROUTINE torch_dict_create

! **************************************************************************************************
!> \brief Releases a Torch dictionary and all its ressources.
!> \author Ole Schuett
! **************************************************************************************************
   SUBROUTINE torch_dict_release(dict)
      TYPE(torch_dict_type), INTENT(INOUT)               :: dict

#if defined(__LIBTORCH)
      INTERFACE
         SUBROUTINE torch_c_dict_release(dict) BIND(C, name="torch_c_dict_release")
            IMPORT :: C_PTR
            TYPE(C_PTR), VALUE                        :: dict
         END SUBROUTINE torch_c_dict_release
      END INTERFACE

      CPASSERT(C_ASSOCIATED(dict%c_ptr))
      CALL torch_c_dict_release(dict=dict%c_ptr)
      dict%c_ptr = C_NULL_PTR
#else
      CPABORT("CP2K was compiled without Torch library.")
      MARK_USED(dict)
#endif
   END SUBROUTINE torch_dict_release

! **************************************************************************************************
!> \brief Loads a Torch model from given "*.pth" file. (In Torch lingo models are called modules)
!> \author Ole Schuett
! **************************************************************************************************
   SUBROUTINE torch_model_load(model, filename)
      TYPE(torch_model_type), INTENT(INOUT)              :: model
      CHARACTER(len=*), INTENT(IN)                       :: filename

#if defined(__LIBTORCH)
      INTERFACE
         SUBROUTINE torch_c_model_load(model, filename) BIND(C, name="torch_c_model_load")
            IMPORT :: C_PTR, C_CHAR
            TYPE(C_PTR)                               :: model
            CHARACTER(kind=C_CHAR), DIMENSION(*)      :: filename
         END SUBROUTINE torch_c_model_load
      END INTERFACE

      CPASSERT(.NOT. C_ASSOCIATED(model%c_ptr))
      CALL torch_c_model_load(model=model%c_ptr, filename=TRIM(filename)//C_NULL_CHAR)
      CPASSERT(C_ASSOCIATED(model%c_ptr))
#else
      CPABORT("CP2K was compiled without Torch library.")
      MARK_USED(model)
      MARK_USED(filename)
#endif
   END SUBROUTINE torch_model_load

! **************************************************************************************************
!> \brief Evaluates the given Torch model. (In Torch lingo this operation is called forward())
!> \author Ole Schuett
! **************************************************************************************************
   SUBROUTINE torch_model_eval(model, inputs, outputs)
      TYPE(torch_model_type), INTENT(INOUT)              :: model
      TYPE(torch_dict_type), INTENT(IN)                  :: inputs
      TYPE(torch_dict_type), INTENT(INOUT)               :: outputs

#if defined(__LIBTORCH)
      INTERFACE
         SUBROUTINE torch_c_model_eval(model, inputs, outputs) BIND(C, name="torch_c_model_eval")
            IMPORT :: C_PTR
            TYPE(C_PTR), VALUE                        :: model
            TYPE(C_PTR), VALUE                        :: inputs
            TYPE(C_PTR), VALUE                        :: outputs
         END SUBROUTINE torch_c_model_eval
      END INTERFACE

      CPASSERT(C_ASSOCIATED(model%c_ptr))
      CPASSERT(C_ASSOCIATED(inputs%c_ptr))
      CPASSERT(C_ASSOCIATED(outputs%c_ptr))
      CALL torch_c_model_eval(model=model%c_ptr, &
                              inputs=inputs%c_ptr, &
                              outputs=outputs%c_ptr)
#else
      CPABORT("CP2K was compiled without Torch library.")
      MARK_USED(model)
      MARK_USED(inputs)
      MARK_USED(outputs)
#endif
   END SUBROUTINE torch_model_eval

! **************************************************************************************************
!> \brief Releases a Torch model and all its ressources.
!> \author Ole Schuett
! **************************************************************************************************
   SUBROUTINE torch_model_release(model)
      TYPE(torch_model_type), INTENT(INOUT)              :: model

#if defined(__LIBTORCH)
      INTERFACE
         SUBROUTINE torch_c_model_release(model) BIND(C, name="torch_c_model_release")
            IMPORT :: C_PTR
            TYPE(C_PTR), VALUE                        :: model
         END SUBROUTINE torch_c_model_release
      END INTERFACE

      CPASSERT(C_ASSOCIATED(model%c_ptr))
      CALL torch_c_model_release(model=model%c_ptr)
      model%c_ptr = C_NULL_PTR
#else
      CPABORT("CP2K was compiled without Torch library.")
      MARK_USED(model)
#endif
   END SUBROUTINE torch_model_release

! **************************************************************************************************
!> \brief Reads metadata entry from given "*.pth" file. (In Torch lingo they are called extra files)
!> \author Ole Schuett
! **************************************************************************************************
   FUNCTION torch_model_read_metadata(filename, key) RESULT(res)
      CHARACTER(len=*), INTENT(IN)                       :: filename, key
      CHARACTER(:), ALLOCATABLE                           :: res

#if defined(__LIBTORCH)
      CHARACTER(LEN=1, KIND=C_CHAR), DIMENSION(:), &
         POINTER                                         :: content_f
      INTEGER                                            :: i
      INTEGER                                            :: length
      TYPE(C_PTR)                                        :: content_c

      INTERFACE
         SUBROUTINE torch_c_model_read_metadata(filename, key, content, length) &
            BIND(C, name="torch_c_model_read_metadata")
            IMPORT :: C_CHAR, C_PTR, C_INT
            CHARACTER(kind=C_CHAR), DIMENSION(*)      :: filename, key
            TYPE(C_PTR)                               :: content
            INTEGER(kind=C_INT)                       :: length
         END SUBROUTINE torch_c_model_read_metadata
      END INTERFACE

      content_c = C_NULL_PTR
      length = -1
      CALL torch_c_model_read_metadata(filename=TRIM(filename)//C_NULL_CHAR, &
                                       key=TRIM(key)//C_NULL_CHAR, &
                                       content=content_c, &
                                       length=length)
      CPASSERT(C_ASSOCIATED(content_c))
      CPASSERT(length >= 0)

      CALL C_F_POINTER(content_c, content_f, shape=(/length + 1/))
      CPASSERT(content_f(length + 1) == C_NULL_CHAR)

      ALLOCATE (CHARACTER(LEN=length) :: res)
      DO i = 1, length
         CPASSERT(content_f(i) /= C_NULL_CHAR)
         res(i:i) = content_f(i)
      END DO

      DEALLOCATE (content_f) ! Was allocated on the C side.
#else
      CPABORT("CP2K was compiled without Torch library.")
      MARK_USED(filename)
      MARK_USED(key)
      MARK_USED(res)
#endif
   END FUNCTION torch_model_read_metadata

! **************************************************************************************************
!> \brief Returns true iff the Torch CUDA backend is available.
!> \author Ole Schuett
! **************************************************************************************************
   FUNCTION torch_cuda_is_available() RESULT(res)
      LOGICAL                                            :: res

#if defined(__LIBTORCH)
      INTERFACE
         FUNCTION torch_c_cuda_is_available() BIND(C, name="torch_c_cuda_is_available")
            IMPORT :: C_BOOL
            LOGICAL(C_BOOL)                           :: torch_c_cuda_is_available
         END FUNCTION torch_c_cuda_is_available
      END INTERFACE

      res = torch_c_cuda_is_available()
#else
      CPABORT("CP2K was compiled without Torch library.")
      MARK_USED(res)
#endif
   END FUNCTION torch_cuda_is_available

! **************************************************************************************************
!> \brief Set whether to allow the use of TF32.
!>        Needed due to changes in defaults from pytorch 1.7 to 1.11 to >=1.12
!>        See https://pytorch.org/docs/stable/notes/cuda.html
!> \author Gabriele Tocci
! **************************************************************************************************
   SUBROUTINE torch_allow_tf32(allow_tf32)
      LOGICAL, INTENT(IN)                                  :: allow_tf32

#if defined(__LIBTORCH)
      INTERFACE
         SUBROUTINE torch_c_allow_tf32(allow_tf32) BIND(C, name="torch_c_allow_tf32")
            IMPORT :: C_BOOL
            LOGICAL(C_BOOL), VALUE                  :: allow_tf32
         END SUBROUTINE torch_c_allow_tf32
      END INTERFACE

      CALL torch_c_allow_tf32(allow_tf32=LOGICAL(allow_tf32, C_BOOL))
#else
      CPABORT("CP2K was compiled without Torch library.")
      MARK_USED(allow_tf32)
#endif
   END SUBROUTINE torch_allow_tf32

! **************************************************************************************************
!> \brief Freeze the given Torch model: applies generic optimization that speed up model.
!>        See https://pytorch.org/docs/stable/generated/torch.jit.freeze.html
!> \author Gabriele Tocci
! **************************************************************************************************
   SUBROUTINE torch_model_freeze(model)
      TYPE(torch_model_type), INTENT(INOUT)              :: model

#if defined(__LIBTORCH)
      INTERFACE
         SUBROUTINE torch_c_model_freeze(model) BIND(C, name="torch_c_model_freeze")
            IMPORT :: C_PTR
            TYPE(C_PTR), VALUE                        :: model
         END SUBROUTINE torch_c_model_freeze
      END INTERFACE

      CPASSERT(C_ASSOCIATED(model%c_ptr))
      CALL torch_c_model_freeze(model=model%c_ptr)
#else
      CPABORT("CP2K was compiled without Torch library.")
      MARK_USED(model)
#endif
   END SUBROUTINE torch_model_freeze

END MODULE torch_api
