# 1 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F" 1
!--------------------------------------------------------------------------------------------------!
!   CP2K: A general program to perform molecular dynamics simulations                              !
!   Copyright 2000-2024 CP2K developers group <https://cp2k.org>                                   !
!                                                                                                  !
!   SPDX-License-Identifier: GPL-2.0-or-later                                                      !
!--------------------------------------------------------------------------------------------------!

! **************************************************************************************************
!> \brief An array-based list which grows on demand.
!>        When the internal array is full, a new array of twice the size will be
!>        allocated and the items are copied over.
!>
!>        This list can also be used as a stack.
!>        Have look at list_push(), list_pop() and list_peek().
!> \note it's not possible to put all templates in a single module because this would lead to circular
!>       dependencies (timer_env_type contains list_routinestat_type and list_callstackentry_type, and
!>       list_timerenv_type contains timer_env_type)
!> \par History
!>      12.2012 first version [ole]
!> \author Ole Schuett
! **************************************************************************************************

# 1 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.fypp" 1
# 7 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.fypp"
# 393 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.fypp"
# 24 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F" 2

MODULE list

   USE timings_base_type, ONLY: routine_report_type, callstack_entry_type, routine_stat_type
   USE list_callstackentry, ONLY: list_callstackentry_type, list_callstackentry_init, list_callstackentry_push,&
# 28 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
       & list_callstackentry_pop, list_callstackentry_peek, list_callstackentry_insert, list_callstackentry_set,&
# 28 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
       & list_callstackentry_get, list_callstackentry_del, list_callstackentry_clear, list_callstackentry_size,&
# 28 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
       & list_callstackentry_destroy, list_callstackentry_isready
   USE list_routinereport, ONLY: list_routinereport_type, list_routinereport_init, list_routinereport_push,&
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
       & list_routinereport_pop, list_routinereport_peek, list_routinereport_insert, list_routinereport_set,&
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
       & list_routinereport_get, list_routinereport_del, list_routinereport_clear, list_routinereport_size,&
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
       & list_routinereport_destroy, list_routinereport_isready
   USE list_routinestat, ONLY: list_routinestat_type, list_routinestat_init, list_routinestat_push, list_routinestat_pop,&
# 30 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
       & list_routinestat_peek, list_routinestat_insert, list_routinestat_set, list_routinestat_get, list_routinestat_del,&
# 30 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
       & list_routinestat_clear, list_routinestat_size, list_routinestat_destroy, list_routinestat_isready
   USE list_timerenv, ONLY: list_timerenv_type, list_timerenv_init, list_timerenv_push, list_timerenv_pop, list_timerenv_peek,&
# 31 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
       & list_timerenv_insert, list_timerenv_set, list_timerenv_get, list_timerenv_del, list_timerenv_clear, list_timerenv_size,&
# 31 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
       & list_timerenv_destroy, list_timerenv_isready

#include "../base/base_uses.f90"
   IMPLICIT NONE

   PRIVATE
   CHARACTER(len=*), PARAMETER, PRIVATE :: moduleN = 'list'

   PUBLIC :: list_init, list_isready, list_push, list_pop, list_peek, &
             list_get, list_size, list_destroy

# 43 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
      PUBLIC :: list_timerenv_type
# 43 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
      PUBLIC :: list_routinestat_type
# 43 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
      PUBLIC :: list_routinereport_type
# 43 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
      PUBLIC :: list_callstackentry_type
# 45 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"

   INTERFACE list_init
# 48 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_timerenv_init
# 48 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinestat_init
# 48 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinereport_init
# 48 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_callstackentry_init
# 50 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
   END INTERFACE

   INTERFACE list_isready
# 54 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_timerenv_isready
# 54 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinestat_isready
# 54 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinereport_isready
# 54 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_callstackentry_isready
# 56 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
   END INTERFACE

   INTERFACE list_push
# 60 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_timerenv_push
# 60 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinestat_push
# 60 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinereport_push
# 60 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_callstackentry_push
# 62 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
   END INTERFACE

   INTERFACE list_pop
# 66 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_timerenv_pop
# 66 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinestat_pop
# 66 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinereport_pop
# 66 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_callstackentry_pop
# 68 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
   END INTERFACE

   INTERFACE list_peek
# 72 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_timerenv_peek
# 72 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinestat_peek
# 72 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinereport_peek
# 72 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_callstackentry_peek
# 74 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
   END INTERFACE

   INTERFACE list_insert
# 78 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_timerenv_insert
# 78 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinestat_insert
# 78 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinereport_insert
# 78 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_callstackentry_insert
# 80 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
   END INTERFACE

   INTERFACE list_set
# 84 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_timerenv_set
# 84 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinestat_set
# 84 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinereport_set
# 84 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_callstackentry_set
# 86 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
   END INTERFACE

   INTERFACE list_get
# 90 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_timerenv_get
# 90 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinestat_get
# 90 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinereport_get
# 90 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_callstackentry_get
# 92 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
   END INTERFACE

   INTERFACE list_del
# 96 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_timerenv_del
# 96 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinestat_del
# 96 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinereport_del
# 96 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_callstackentry_del
# 98 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
   END INTERFACE

   INTERFACE list_clear
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_timerenv_clear
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinestat_clear
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinereport_clear
# 102 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_callstackentry_clear
# 104 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
   END INTERFACE

   INTERFACE list_size
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_timerenv_size
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinestat_size
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinereport_size
# 108 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_callstackentry_size
# 110 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
   END INTERFACE

   INTERFACE list_destroy
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_timerenv_destroy
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinestat_destroy
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_routinereport_destroy
# 114 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
         MODULE PROCEDURE list_callstackentry_destroy
# 116 "/builddir/build/BUILD/cp2k-2024.1/src/common/list.F"
   END INTERFACE

END MODULE list
