# 1 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F" 1
!--------------------------------------------------------------------------------------------------!
!   CP2K: A general program to perform molecular dynamics simulations                              !
!   Copyright 2000-2024 CP2K developers group <https://cp2k.org>                                   !
!                                                                                                  !
!   SPDX-License-Identifier: GPL-2.0-or-later                                                      !
!--------------------------------------------------------------------------------------------------!

# 1 "/builddir/build/BUILD/cp2k-2024.1/src/input/../common/cp_linked_list.fypp" 1
# 7 "/builddir/build/BUILD/cp2k-2024.1/src/input/../common/cp_linked_list.fypp"
# 610 "/builddir/build/BUILD/cp2k-2024.1/src/input/../common/cp_linked_list.fypp"
# 9 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F" 2

MODULE cp_linked_list_input
   USE kinds, only: dp, default_string_length
   USE input_val_types, only: val_type, val_p_type
#include "../base/base_uses.f90"

# 28 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief describes a generic linked list template.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      Linked list are supposed to always use pointers to the nodes for
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      basically everything: a pointer to the node is a list, an element of
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      the list, an iterator between the elment of the list.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      An empty list is represented by an unassociated pointer.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      1.2002 created
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      4.2002 changed most of the methods, by making access to the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>             always through pointers (identifying pointer, list and iterators)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      6.2004 removed %initialized from list elements
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      IMPLICIT NONE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      PRIVATE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      LOGICAL, PRIVATE, PARAMETER :: debug_this_module = .TRUE.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!API type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         PUBLIC :: cp_sll_int_type, cp_sll_int_p_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         PUBLIC :: cp_sll_real_type, cp_sll_real_p_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         PUBLIC :: cp_sll_logical_type, cp_sll_logical_p_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         PUBLIC :: cp_sll_char_type, cp_sll_char_p_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         PUBLIC :: cp_sll_val_type, cp_sll_val_p_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!API common methods
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      PUBLIC :: cp_create, cp_dealloc, cp_next
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!API special get
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      PUBLIC :: cp_get_first_el, cp_get_rest, cp_get_length, cp_get_element_at, cp_to_array
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!API special set
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      PUBLIC :: cp_set_element_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!API structure manipulation
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      PUBLIC :: cp_insert, cp_remove_first_el, cp_remove_el, cp_remove_all
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      !API underlying routines
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         PUBLIC :: cp_sll_int_create, cp_sll_int_dealloc, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_int_next, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_int_get_length, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_int_get_el_at, cp_sll_int_set_el_at, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_int_get_first_el, cp_sll_int_get_rest, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_int_insert_el, cp_sll_int_insert_el_at, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_int_rm_first_el, cp_sll_int_rm_el_at, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_int_rm_all_el, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_int_to_array
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         PUBLIC :: cp_sll_real_create, cp_sll_real_dealloc, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_real_next, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_real_get_length, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_real_get_el_at, cp_sll_real_set_el_at, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_real_get_first_el, cp_sll_real_get_rest, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_real_insert_el, cp_sll_real_insert_el_at, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_real_rm_first_el, cp_sll_real_rm_el_at, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_real_rm_all_el, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_real_to_array
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         PUBLIC :: cp_sll_logical_create, cp_sll_logical_dealloc, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_logical_next, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_logical_get_length, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_logical_get_el_at, cp_sll_logical_set_el_at, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_logical_get_first_el, cp_sll_logical_get_rest, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_logical_insert_el, cp_sll_logical_insert_el_at, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_logical_rm_first_el, cp_sll_logical_rm_el_at, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_logical_rm_all_el, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_logical_to_array
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         PUBLIC :: cp_sll_char_create, cp_sll_char_dealloc, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_char_next, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_char_get_length, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_char_get_el_at, cp_sll_char_set_el_at, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_char_get_first_el, cp_sll_char_get_rest, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_char_insert_el, cp_sll_char_insert_el_at, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_char_rm_first_el, cp_sll_char_rm_el_at, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_char_rm_all_el, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_char_to_array
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         PUBLIC :: cp_sll_val_create, cp_sll_val_dealloc, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_val_next, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_val_get_length, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_val_get_el_at, cp_sll_val_set_el_at, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_val_get_first_el, cp_sll_val_get_rest, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_val_insert_el, cp_sll_val_insert_el_at, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_val_rm_first_el, cp_sll_val_rm_el_at, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_val_rm_all_el, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                   cp_sll_val_to_array
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! creation of an object (from a pointer)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      INTERFACE cp_create
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_int_create
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_real_create
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_logical_create
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_char_create
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_val_create
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      END INTERFACE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! destruction of an object (from a pointer)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      INTERFACE cp_dealloc
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_int_dealloc
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_real_dealloc
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_logical_dealloc
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_char_dealloc
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_val_dealloc
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      END INTERFACE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! iterates to the next element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      INTERFACE cp_next
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_int_next
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_real_next
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_logical_next
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_char_next
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_val_next
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      END INTERFACE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! returns the first element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      INTERFACE cp_get_first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_int_get_first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_real_get_first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_logical_get_first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_char_get_first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_val_get_first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      END INTERFACE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! returns the rest of the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      INTERFACE cp_get_rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_int_get_rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_real_get_rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_logical_get_rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_char_get_rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_val_get_rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      END INTERFACE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! returns the length of the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      INTERFACE cp_get_length
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_int_get_length
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_real_get_length
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_logical_get_length
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_char_get_length
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_val_get_length
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      END INTERFACE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! returns the element at the given position
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      INTERFACE cp_get_element_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_int_get_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_real_get_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_logical_get_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_char_get_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_val_get_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      END INTERFACE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! sets the element at the given position
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      INTERFACE cp_set_element_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_int_set_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_real_set_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_logical_set_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_char_set_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_val_set_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      END INTERFACE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! inserts one element call cp_insert(list,element,...)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      INTERFACE cp_insert
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_int_insert_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_real_insert_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_logical_insert_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_char_insert_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_val_insert_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      END INTERFACE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      INTERFACE cp_insert_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_int_insert_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_real_insert_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_logical_insert_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_char_insert_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_val_insert_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      END INTERFACE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! removes an element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      INTERFACE cp_remove_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_int_rm_first_el, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               cp_sll_int_rm_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_real_rm_first_el, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               cp_sll_real_rm_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_logical_rm_first_el, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               cp_sll_logical_rm_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_char_rm_first_el, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               cp_sll_char_rm_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_val_rm_first_el, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               cp_sll_val_rm_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      END INTERFACE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! removes the first el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      INTERFACE cp_remove_first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_int_rm_first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_real_rm_first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_logical_rm_first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_char_rm_first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_val_rm_first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      END INTERFACE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! remove all the elements
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      INTERFACE cp_remove_all
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_int_rm_all_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_real_rm_all_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_logical_rm_all_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_char_rm_all_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            MODULE PROCEDURE cp_sll_val_rm_all_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      END INTERFACE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! transorms the list in array
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      INTERFACE cp_to_array
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            module procedure cp_sll_int_to_array
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            module procedure cp_sll_real_to_array
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            module procedure cp_sll_logical_to_array
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            module procedure cp_sll_char_to_array
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            module procedure cp_sll_val_to_array
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      END INTERFACE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief represent a single linked list that stores pointers to the elements
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param first_el the element that is stored in this node.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param rest the rest of the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param empty true if the list pointer is not associated, if it points to
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>             to a not it is always false (as there is at least the
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>             first_el in the list)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param length the number of elements in the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      List are alway accessed through pointers, so every node of the
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      linked list can be seen as a list, its first element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      a pointer to the position before itself, in a very natural way:
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      all the insertions take place before the actual element, and
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      you still can insert an element at the end.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      This way I could identify nodes, lists and pointers between the
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      elements of the list.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      Indexing is 1 based.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         TYPE cp_sll_int_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            integer :: first_el  = 0
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: rest => NULL()
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END TYPE cp_sll_int_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief represent a single linked list that stores pointers to the elements
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param first_el the element that is stored in this node.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param rest the rest of the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param empty true if the list pointer is not associated, if it points to
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>             to a not it is always false (as there is at least the
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>             first_el in the list)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param length the number of elements in the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      List are alway accessed through pointers, so every node of the
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      linked list can be seen as a list, its first element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      a pointer to the position before itself, in a very natural way:
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      all the insertions take place before the actual element, and
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      you still can insert an element at the end.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      This way I could identify nodes, lists and pointers between the
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      elements of the list.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      Indexing is 1 based.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         TYPE cp_sll_real_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            REAL(kind=dp) :: first_el  = 0.0_dp
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: rest => NULL()
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END TYPE cp_sll_real_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief represent a single linked list that stores pointers to the elements
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param first_el the element that is stored in this node.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param rest the rest of the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param empty true if the list pointer is not associated, if it points to
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>             to a not it is always false (as there is at least the
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>             first_el in the list)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param length the number of elements in the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      List are alway accessed through pointers, so every node of the
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      linked list can be seen as a list, its first element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      a pointer to the position before itself, in a very natural way:
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      all the insertions take place before the actual element, and
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      you still can insert an element at the end.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      This way I could identify nodes, lists and pointers between the
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      elements of the list.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      Indexing is 1 based.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         TYPE cp_sll_logical_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            logical :: first_el  = .FALSE.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: rest => NULL()
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END TYPE cp_sll_logical_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief represent a single linked list that stores pointers to the elements
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param first_el the element that is stored in this node.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param rest the rest of the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param empty true if the list pointer is not associated, if it points to
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>             to a not it is always false (as there is at least the
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>             first_el in the list)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param length the number of elements in the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      List are alway accessed through pointers, so every node of the
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      linked list can be seen as a list, its first element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      a pointer to the position before itself, in a very natural way:
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      all the insertions take place before the actual element, and
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      you still can insert an element at the end.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      This way I could identify nodes, lists and pointers between the
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      elements of the list.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      Indexing is 1 based.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         TYPE cp_sll_char_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            character(len=default_string_length) :: first_el  = ""
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: rest => NULL()
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END TYPE cp_sll_char_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief represent a single linked list that stores pointers to the elements
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param first_el the element that is stored in this node.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param rest the rest of the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param empty true if the list pointer is not associated, if it points to
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>             to a not it is always false (as there is at least the
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>             first_el in the list)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param length the number of elements in the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      List are alway accessed through pointers, so every node of the
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      linked list can be seen as a list, its first element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      a pointer to the position before itself, in a very natural way:
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      all the insertions take place before the actual element, and
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      you still can insert an element at the end.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      This way I could identify nodes, lists and pointers between the
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      elements of the list.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      Indexing is 1 based.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         TYPE cp_sll_val_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            type(val_type),pointer :: first_el  => NULL()
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: rest => NULL()
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END TYPE cp_sll_val_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief pointer to a linked list (to make arrays of pointers)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param list the pointer to the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         TYPE cp_sll_int_p_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: list => NULL()
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END TYPE cp_sll_int_p_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief pointer to a linked list (to make arrays of pointers)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param list the pointer to the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         TYPE cp_sll_real_p_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: list => NULL()
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END TYPE cp_sll_real_p_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief pointer to a linked list (to make arrays of pointers)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param list the pointer to the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         TYPE cp_sll_logical_p_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: list => NULL()
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END TYPE cp_sll_logical_p_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief pointer to a linked list (to make arrays of pointers)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param list the pointer to the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         TYPE cp_sll_char_p_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: list => NULL()
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END TYPE cp_sll_char_p_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief pointer to a linked list (to make arrays of pointers)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param list the pointer to the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         TYPE cp_sll_val_p_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: list => NULL()
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END TYPE cp_sll_val_p_type
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      CONTAINS
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
      
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! =========== creation / destruction ========
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief allocates and initializes a single linked list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the single linked list to initialize
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param first_el the first element of this list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param rest the following elements (if not given: empty)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_int_create(sll, first_el, rest)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            integer, intent(in), OPTIONAL :: first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER, OPTIONAL :: rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (.NOT. PRESENT(first_el)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               NULLIFY (sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (PRESENT(rest)) sll => rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ALLOCATE (sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               sll%first_el =first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               NULLIFY (sll%rest)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (PRESENT(rest)) sll%rest => rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_int_create
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief allocates and initializes a single linked list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the single linked list to initialize
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param first_el the first element of this list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param rest the following elements (if not given: empty)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_real_create(sll, first_el, rest)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            REAL(kind=dp), intent(in), OPTIONAL :: first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER, OPTIONAL :: rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (.NOT. PRESENT(first_el)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               NULLIFY (sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (PRESENT(rest)) sll => rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ALLOCATE (sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               sll%first_el =first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               NULLIFY (sll%rest)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (PRESENT(rest)) sll%rest => rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_real_create
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief allocates and initializes a single linked list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the single linked list to initialize
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param first_el the first element of this list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param rest the following elements (if not given: empty)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_logical_create(sll, first_el, rest)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            logical, intent(in), OPTIONAL :: first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER, OPTIONAL :: rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (.NOT. PRESENT(first_el)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               NULLIFY (sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (PRESENT(rest)) sll => rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ALLOCATE (sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               sll%first_el =first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               NULLIFY (sll%rest)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (PRESENT(rest)) sll%rest => rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_logical_create
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief allocates and initializes a single linked list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the single linked list to initialize
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param first_el the first element of this list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param rest the following elements (if not given: empty)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_char_create(sll, first_el, rest)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            character(len=default_string_length), intent(in), OPTIONAL :: first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER, OPTIONAL :: rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (.NOT. PRESENT(first_el)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               NULLIFY (sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (PRESENT(rest)) sll => rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ALLOCATE (sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               sll%first_el =first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               NULLIFY (sll%rest)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (PRESENT(rest)) sll%rest => rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_char_create
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief allocates and initializes a single linked list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the single linked list to initialize
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param first_el the first element of this list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param rest the following elements (if not given: empty)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_val_create(sll, first_el, rest)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            type(val_type),pointer, intent(in), OPTIONAL :: first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER, OPTIONAL :: rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (.NOT. PRESENT(first_el)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               NULLIFY (sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (PRESENT(rest)) sll => rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ALLOCATE (sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               sll%first_el =>first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               NULLIFY (sll%rest)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (PRESENT(rest)) sll%rest => rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_val_create
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief deallocates the singly linked list starting at sll.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      Does not work if loops are present!
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list to be deallocated
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      does not deallocate the elements that are stored in the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      check more?
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_int_dealloc(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            CALL cp_sll_int_rm_all_el(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_int_dealloc
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief deallocates the singly linked list starting at sll.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      Does not work if loops are present!
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list to be deallocated
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      does not deallocate the elements that are stored in the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      check more?
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_real_dealloc(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            CALL cp_sll_real_rm_all_el(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_real_dealloc
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief deallocates the singly linked list starting at sll.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      Does not work if loops are present!
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list to be deallocated
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      does not deallocate the elements that are stored in the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      check more?
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_logical_dealloc(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            CALL cp_sll_logical_rm_all_el(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_logical_dealloc
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief deallocates the singly linked list starting at sll.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      Does not work if loops are present!
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list to be deallocated
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      does not deallocate the elements that are stored in the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      check more?
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_char_dealloc(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            CALL cp_sll_char_rm_all_el(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_char_dealloc
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief deallocates the singly linked list starting at sll.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      Does not work if loops are present!
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list to be deallocated
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      does not deallocate the elements that are stored in the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      check more?
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_val_dealloc(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            CALL cp_sll_val_rm_all_el(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_val_dealloc
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! * low-level *
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief deallocates a node of a singly linked list (low level)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the node to be deallocated
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_int_dealloc_node(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            DEALLOCATE (sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_int_dealloc_node
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief deallocates a node of a singly linked list (low level)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the node to be deallocated
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_real_dealloc_node(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            DEALLOCATE (sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_real_dealloc_node
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief deallocates a node of a singly linked list (low level)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the node to be deallocated
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_logical_dealloc_node(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            DEALLOCATE (sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_logical_dealloc_node
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief deallocates a node of a singly linked list (low level)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the node to be deallocated
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_char_dealloc_node(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            DEALLOCATE (sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_char_dealloc_node
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief deallocates a node of a singly linked list (low level)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the node to be deallocated
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_val_dealloc_node(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            DEALLOCATE (sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_val_dealloc_node
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! ============= get/set ============
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns the first element stored in the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the single linked list to get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_int_get_first_el(sll) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            integer :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            res =sll%first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_int_get_first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns the first element stored in the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the single linked list to get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_real_get_first_el(sll) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            REAL(kind=dp) :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            res =sll%first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_real_get_first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns the first element stored in the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the single linked list to get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_logical_get_first_el(sll) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            logical :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            res =sll%first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_logical_get_first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns the first element stored in the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the single linked list to get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_char_get_first_el(sll) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            character(len=default_string_length) :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            res =sll%first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_char_get_first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns the first element stored in the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the single linked list to get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_val_get_first_el(sll) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            type(val_type),pointer :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            res =>sll%first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_val_get_first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns the rest of the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the single linked list to get the rest from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param iter how many times the call to rest should be iterated,
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>            defaults to 1; -1 means till end of the list.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      split the case iter=1 to make it more optimized?
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_int_get_rest(sll, iter) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, OPTIONAL :: iter
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER :: i
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (.NOT. ASSOCIATED(sll)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               NULLIFY (res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (PRESENT(iter)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  res => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  DO i = 1, iter
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     IF (ASSOCIATED(res%rest)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                        res => res%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                        CPABORT("tried to go past end")
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  IF (iter == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                        IF (.NOT. ASSOCIATED(res%rest)) EXIT
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                        res => res%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  res => sll%rest ! make the common case fast...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_int_get_rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns the rest of the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the single linked list to get the rest from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param iter how many times the call to rest should be iterated,
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>            defaults to 1; -1 means till end of the list.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      split the case iter=1 to make it more optimized?
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_real_get_rest(sll, iter) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, OPTIONAL :: iter
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER :: i
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (.NOT. ASSOCIATED(sll)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               NULLIFY (res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (PRESENT(iter)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  res => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  DO i = 1, iter
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     IF (ASSOCIATED(res%rest)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                        res => res%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                        CPABORT("tried to go past end")
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  IF (iter == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                        IF (.NOT. ASSOCIATED(res%rest)) EXIT
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                        res => res%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  res => sll%rest ! make the common case fast...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_real_get_rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns the rest of the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the single linked list to get the rest from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param iter how many times the call to rest should be iterated,
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>            defaults to 1; -1 means till end of the list.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      split the case iter=1 to make it more optimized?
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_logical_get_rest(sll, iter) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, OPTIONAL :: iter
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER :: i
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (.NOT. ASSOCIATED(sll)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               NULLIFY (res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (PRESENT(iter)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  res => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  DO i = 1, iter
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     IF (ASSOCIATED(res%rest)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                        res => res%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                        CPABORT("tried to go past end")
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  IF (iter == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                        IF (.NOT. ASSOCIATED(res%rest)) EXIT
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                        res => res%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  res => sll%rest ! make the common case fast...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_logical_get_rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns the rest of the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the single linked list to get the rest from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param iter how many times the call to rest should be iterated,
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>            defaults to 1; -1 means till end of the list.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      split the case iter=1 to make it more optimized?
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_char_get_rest(sll, iter) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, OPTIONAL :: iter
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER :: i
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (.NOT. ASSOCIATED(sll)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               NULLIFY (res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (PRESENT(iter)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  res => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  DO i = 1, iter
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     IF (ASSOCIATED(res%rest)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                        res => res%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                        CPABORT("tried to go past end")
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  IF (iter == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                        IF (.NOT. ASSOCIATED(res%rest)) EXIT
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                        res => res%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  res => sll%rest ! make the common case fast...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_char_get_rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns the rest of the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the single linked list to get the rest from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param iter how many times the call to rest should be iterated,
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>            defaults to 1; -1 means till end of the list.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      split the case iter=1 to make it more optimized?
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_val_get_rest(sll, iter) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, OPTIONAL :: iter
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER :: i
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (.NOT. ASSOCIATED(sll)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               NULLIFY (res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (PRESENT(iter)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  res => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  DO i = 1, iter
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     IF (ASSOCIATED(res%rest)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                        res => res%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                        CPABORT("tried to go past end")
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  IF (iter == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                        IF (.NOT. ASSOCIATED(res%rest)) EXIT
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                        res => res%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                     END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  res => sll%rest ! make the common case fast...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_val_get_rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns the length of the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you want to know the length of
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(n))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_int_get_length(sll) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), pointer :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER ::res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: iterator
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            res = 0
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            iterator => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (ASSOCIATED(iterator)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  res = res + 1
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  iterator => iterator%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  EXIT
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_int_get_length
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns the length of the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you want to know the length of
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(n))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_real_get_length(sll) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), pointer :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER ::res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: iterator
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            res = 0
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            iterator => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (ASSOCIATED(iterator)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  res = res + 1
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  iterator => iterator%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  EXIT
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_real_get_length
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns the length of the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you want to know the length of
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(n))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_logical_get_length(sll) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), pointer :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER ::res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: iterator
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            res = 0
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            iterator => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (ASSOCIATED(iterator)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  res = res + 1
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  iterator => iterator%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  EXIT
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_logical_get_length
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns the length of the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you want to know the length of
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(n))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_char_get_length(sll) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), pointer :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER ::res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: iterator
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            res = 0
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            iterator => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (ASSOCIATED(iterator)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  res = res + 1
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  iterator => iterator%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  EXIT
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_char_get_length
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns the length of the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you want to know the length of
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(n))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_val_get_length(sll) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), pointer :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER ::res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: iterator
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            res = 0
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            iterator => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (ASSOCIATED(iterator)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  res = res + 1
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  iterator => iterator%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  EXIT
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_val_get_length
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns the element at the given index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param index the position of the element (stating at 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(index))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_int_get_el_at(sll, index) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            integer :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, INTENT(in) :: index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: pos
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (index == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               pos => cp_sll_int_get_rest(sll, iter=-1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               pos => cp_sll_int_get_rest(sll, iter=index - 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            CPASSERT(ASSOCIATED(pos))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            res =pos%first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_int_get_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns the element at the given index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param index the position of the element (stating at 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(index))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_real_get_el_at(sll, index) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            REAL(kind=dp) :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, INTENT(in) :: index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: pos
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (index == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               pos => cp_sll_real_get_rest(sll, iter=-1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               pos => cp_sll_real_get_rest(sll, iter=index - 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            CPASSERT(ASSOCIATED(pos))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            res =pos%first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_real_get_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns the element at the given index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param index the position of the element (stating at 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(index))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_logical_get_el_at(sll, index) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            logical :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, INTENT(in) :: index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: pos
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (index == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               pos => cp_sll_logical_get_rest(sll, iter=-1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               pos => cp_sll_logical_get_rest(sll, iter=index - 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            CPASSERT(ASSOCIATED(pos))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            res =pos%first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_logical_get_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns the element at the given index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param index the position of the element (stating at 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(index))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_char_get_el_at(sll, index) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            character(len=default_string_length) :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, INTENT(in) :: index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: pos
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (index == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               pos => cp_sll_char_get_rest(sll, iter=-1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               pos => cp_sll_char_get_rest(sll, iter=index - 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            CPASSERT(ASSOCIATED(pos))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            res =pos%first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_char_get_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns the element at the given index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param index the position of the element (stating at 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(index))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_val_get_el_at(sll, index) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            type(val_type),pointer :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, INTENT(in) :: index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: pos
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (index == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               pos => cp_sll_val_get_rest(sll, iter=-1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               pos => cp_sll_val_get_rest(sll, iter=index - 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            CPASSERT(ASSOCIATED(pos))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            res =>pos%first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_val_get_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief sets the element at the given index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param index the position of the element (stating at 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>             -1 means at the end
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param value the new element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(index))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_int_set_el_at(sll, index, value)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            integer, intent(in) :: value
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, INTENT(in) :: index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: pos
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (index == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               pos => cp_sll_int_get_rest(sll, iter=-1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               pos => cp_sll_int_get_rest(sll, iter=index - 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            CPASSERT(ASSOCIATED(pos))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            pos%first_el =value
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_int_set_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief sets the element at the given index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param index the position of the element (stating at 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>             -1 means at the end
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param value the new element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(index))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_real_set_el_at(sll, index, value)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            REAL(kind=dp), intent(in) :: value
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, INTENT(in) :: index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: pos
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (index == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               pos => cp_sll_real_get_rest(sll, iter=-1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               pos => cp_sll_real_get_rest(sll, iter=index - 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            CPASSERT(ASSOCIATED(pos))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            pos%first_el =value
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_real_set_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief sets the element at the given index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param index the position of the element (stating at 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>             -1 means at the end
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param value the new element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(index))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_logical_set_el_at(sll, index, value)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            logical, intent(in) :: value
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, INTENT(in) :: index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: pos
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (index == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               pos => cp_sll_logical_get_rest(sll, iter=-1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               pos => cp_sll_logical_get_rest(sll, iter=index - 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            CPASSERT(ASSOCIATED(pos))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            pos%first_el =value
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_logical_set_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief sets the element at the given index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param index the position of the element (stating at 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>             -1 means at the end
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param value the new element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(index))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_char_set_el_at(sll, index, value)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            character(len=default_string_length), intent(in) :: value
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, INTENT(in) :: index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: pos
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (index == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               pos => cp_sll_char_get_rest(sll, iter=-1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               pos => cp_sll_char_get_rest(sll, iter=index - 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            CPASSERT(ASSOCIATED(pos))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            pos%first_el =value
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_char_set_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief sets the element at the given index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param index the position of the element (stating at 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>             -1 means at the end
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param value the new element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(index))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_val_set_el_at(sll, index, value)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            type(val_type),pointer, intent(in) :: value
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, INTENT(in) :: index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: pos
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (index == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               pos => cp_sll_val_get_rest(sll, iter=-1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               pos => cp_sll_val_get_rest(sll, iter=index - 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            CPASSERT(ASSOCIATED(pos))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            pos%first_el =>value
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_val_set_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! * iteration *
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns true if the actual element is valid (i.e. iterator ont at end)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      moves the iterator to the next element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param iterator iterator that moves along the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param el_att the actual element (valid only if the function returns true)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_int_next(iterator, el_att) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: iterator
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            integer, intent(out), OPTIONAL :: el_att
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            LOGICAL :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (ASSOCIATED(iterator)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               res = .TRUE.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               if (present(el_att)) el_att =iterator%first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               iterator => iterator%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               res = .FALSE.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_int_next
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns true if the actual element is valid (i.e. iterator ont at end)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      moves the iterator to the next element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param iterator iterator that moves along the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param el_att the actual element (valid only if the function returns true)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_real_next(iterator, el_att) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: iterator
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            REAL(kind=dp), intent(out), OPTIONAL :: el_att
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            LOGICAL :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (ASSOCIATED(iterator)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               res = .TRUE.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               if (present(el_att)) el_att =iterator%first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               iterator => iterator%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               res = .FALSE.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_real_next
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns true if the actual element is valid (i.e. iterator ont at end)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      moves the iterator to the next element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param iterator iterator that moves along the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param el_att the actual element (valid only if the function returns true)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_logical_next(iterator, el_att) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: iterator
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            logical, intent(out), OPTIONAL :: el_att
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            LOGICAL :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (ASSOCIATED(iterator)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               res = .TRUE.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               if (present(el_att)) el_att =iterator%first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               iterator => iterator%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               res = .FALSE.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_logical_next
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns true if the actual element is valid (i.e. iterator ont at end)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      moves the iterator to the next element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param iterator iterator that moves along the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param el_att the actual element (valid only if the function returns true)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_char_next(iterator, el_att) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: iterator
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            character(len=default_string_length), intent(out), OPTIONAL :: el_att
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            LOGICAL :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (ASSOCIATED(iterator)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               res = .TRUE.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               if (present(el_att)) el_att =iterator%first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               iterator => iterator%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               res = .FALSE.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_char_next
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns true if the actual element is valid (i.e. iterator ont at end)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      moves the iterator to the next element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param iterator iterator that moves along the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param el_att the actual element (valid only if the function returns true)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_val_next(iterator, el_att) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: iterator
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            type(val_type),pointer, intent(out), OPTIONAL :: el_att
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            LOGICAL :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (ASSOCIATED(iterator)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               res = .TRUE.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               if (present(el_att)) el_att =>iterator%first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               iterator => iterator%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               res = .FALSE.
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_val_next
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! ============ structure modifications ============
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief insert an element at the beginning of the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the single linked list point at the beginning of which
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>           you want to add the element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param el the element to add
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      fast (O(1))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_int_insert_el(sll, el)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            integer, intent(in):: el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: newSlot
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            NULLIFY (newSlot)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            CALL cp_sll_int_create(newSlot, first_el=el, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                                                rest=sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            sll => newSlot
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_int_insert_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief insert an element at the beginning of the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the single linked list point at the beginning of which
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>           you want to add the element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param el the element to add
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      fast (O(1))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_real_insert_el(sll, el)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            REAL(kind=dp), intent(in):: el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: newSlot
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            NULLIFY (newSlot)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            CALL cp_sll_real_create(newSlot, first_el=el, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                                                rest=sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            sll => newSlot
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_real_insert_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief insert an element at the beginning of the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the single linked list point at the beginning of which
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>           you want to add the element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param el the element to add
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      fast (O(1))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_logical_insert_el(sll, el)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            logical, intent(in):: el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: newSlot
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            NULLIFY (newSlot)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            CALL cp_sll_logical_create(newSlot, first_el=el, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                                                rest=sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            sll => newSlot
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_logical_insert_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief insert an element at the beginning of the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the single linked list point at the beginning of which
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>           you want to add the element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param el the element to add
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      fast (O(1))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_char_insert_el(sll, el)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            character(len=default_string_length), intent(in):: el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: newSlot
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            NULLIFY (newSlot)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            CALL cp_sll_char_create(newSlot, first_el=el, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                                                rest=sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            sll => newSlot
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_char_insert_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief insert an element at the beginning of the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the single linked list point at the beginning of which
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>           you want to add the element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param el the element to add
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      fast (O(1))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_val_insert_el(sll, el)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            type(val_type),pointer, intent(in):: el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: newSlot
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            NULLIFY (newSlot)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            CALL cp_sll_val_create(newSlot, first_el=el, &
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                                                rest=sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            sll => newSlot
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_val_insert_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief remove the first element of the linked list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list whose first element has to be removed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      fast (O(1))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_int_rm_first_el(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: node_to_rm
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            node_to_rm => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (ASSOCIATED(sll)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               sll => sll%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_int_dealloc_node(node_to_rm)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CPABORT("tried to remove first el of an empty list")
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_int_rm_first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief remove the first element of the linked list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list whose first element has to be removed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      fast (O(1))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_real_rm_first_el(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: node_to_rm
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            node_to_rm => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (ASSOCIATED(sll)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               sll => sll%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_real_dealloc_node(node_to_rm)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CPABORT("tried to remove first el of an empty list")
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_real_rm_first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief remove the first element of the linked list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list whose first element has to be removed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      fast (O(1))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_logical_rm_first_el(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: node_to_rm
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            node_to_rm => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (ASSOCIATED(sll)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               sll => sll%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_logical_dealloc_node(node_to_rm)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CPABORT("tried to remove first el of an empty list")
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_logical_rm_first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief remove the first element of the linked list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list whose first element has to be removed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      fast (O(1))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_char_rm_first_el(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: node_to_rm
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            node_to_rm => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (ASSOCIATED(sll)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               sll => sll%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_char_dealloc_node(node_to_rm)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CPABORT("tried to remove first el of an empty list")
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_char_rm_first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief remove the first element of the linked list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list whose first element has to be removed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      fast (O(1))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_val_rm_first_el(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: node_to_rm
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            node_to_rm => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (ASSOCIATED(sll)) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               sll => sll%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_val_dealloc_node(node_to_rm)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CPABORT("tried to remove first el of an empty list")
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_val_rm_first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief inserts the element at the given index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param el the new element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param index the position of the element (stating at 1).
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>             If it is -1, it means at end
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(index))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_int_insert_el_at(sll, el, index)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            integer, intent(in) :: el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, INTENT(in) :: index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: pos
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (index == 1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_int_insert_el(sll, el)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (index == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  pos => cp_sll_int_get_rest(sll, iter=-1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  pos => cp_sll_int_get_rest(sll, iter=index - 2)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CPASSERT(ASSOCIATED(pos))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_int_insert_el(pos%rest, el)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_int_insert_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief inserts the element at the given index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param el the new element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param index the position of the element (stating at 1).
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>             If it is -1, it means at end
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(index))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_real_insert_el_at(sll, el, index)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            REAL(kind=dp), intent(in) :: el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, INTENT(in) :: index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: pos
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (index == 1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_real_insert_el(sll, el)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (index == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  pos => cp_sll_real_get_rest(sll, iter=-1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  pos => cp_sll_real_get_rest(sll, iter=index - 2)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CPASSERT(ASSOCIATED(pos))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_real_insert_el(pos%rest, el)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_real_insert_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief inserts the element at the given index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param el the new element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param index the position of the element (stating at 1).
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>             If it is -1, it means at end
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(index))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_logical_insert_el_at(sll, el, index)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            logical, intent(in) :: el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, INTENT(in) :: index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: pos
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (index == 1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_logical_insert_el(sll, el)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (index == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  pos => cp_sll_logical_get_rest(sll, iter=-1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  pos => cp_sll_logical_get_rest(sll, iter=index - 2)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CPASSERT(ASSOCIATED(pos))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_logical_insert_el(pos%rest, el)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_logical_insert_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief inserts the element at the given index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param el the new element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param index the position of the element (stating at 1).
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>             If it is -1, it means at end
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(index))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_char_insert_el_at(sll, el, index)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            character(len=default_string_length), intent(in) :: el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, INTENT(in) :: index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: pos
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (index == 1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_char_insert_el(sll, el)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (index == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  pos => cp_sll_char_get_rest(sll, iter=-1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  pos => cp_sll_char_get_rest(sll, iter=index - 2)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CPASSERT(ASSOCIATED(pos))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_char_insert_el(pos%rest, el)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_char_insert_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief inserts the element at the given index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param el the new element
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param index the position of the element (stating at 1).
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>             If it is -1, it means at end
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(index))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_val_insert_el_at(sll, el, index)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            type(val_type),pointer, intent(in) :: el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, INTENT(in) :: index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: pos
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (index == 1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_val_insert_el(sll, el)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (index == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  pos => cp_sll_val_get_rest(sll, iter=-1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  pos => cp_sll_val_get_rest(sll, iter=index - 2)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CPASSERT(ASSOCIATED(pos))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_val_insert_el(pos%rest, el)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_val_insert_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief removes the element at the given index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param index the position of the element (stating at 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(index))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_int_rm_el_at(sll, index)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, INTENT(in)::index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: pos
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (index == 1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_int_rm_first_el(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (index == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  pos => cp_sll_int_get_rest(sll, iter=-1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  pos => cp_sll_int_get_rest(sll, iter=index - 2)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CPASSERT(ASSOCIATED(pos))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_int_rm_first_el(pos%rest)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_int_rm_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief removes the element at the given index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param index the position of the element (stating at 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(index))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_real_rm_el_at(sll, index)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, INTENT(in)::index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: pos
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (index == 1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_real_rm_first_el(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (index == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  pos => cp_sll_real_get_rest(sll, iter=-1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  pos => cp_sll_real_get_rest(sll, iter=index - 2)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CPASSERT(ASSOCIATED(pos))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_real_rm_first_el(pos%rest)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_real_rm_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief removes the element at the given index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param index the position of the element (stating at 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(index))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_logical_rm_el_at(sll, index)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, INTENT(in)::index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: pos
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (index == 1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_logical_rm_first_el(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (index == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  pos => cp_sll_logical_get_rest(sll, iter=-1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  pos => cp_sll_logical_get_rest(sll, iter=index - 2)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CPASSERT(ASSOCIATED(pos))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_logical_rm_first_el(pos%rest)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_logical_rm_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief removes the element at the given index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param index the position of the element (stating at 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(index))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_char_rm_el_at(sll, index)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, INTENT(in)::index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: pos
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (index == 1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_char_rm_first_el(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (index == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  pos => cp_sll_char_get_rest(sll, iter=-1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  pos => cp_sll_char_get_rest(sll, iter=index - 2)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CPASSERT(ASSOCIATED(pos))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_char_rm_first_el(pos%rest)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_char_rm_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief removes the element at the given index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list you get the element from
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param index the position of the element (stating at 1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      slow (O(index))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_val_rm_el_at(sll, index)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER, INTENT(in)::index
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: pos
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            IF (index == 1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_val_rm_first_el(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (index == -1) THEN
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  pos => cp_sll_val_get_rest(sll, iter=-1)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ELSE
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
                  pos => cp_sll_val_get_rest(sll, iter=index - 2)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CPASSERT(ASSOCIATED(pos))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_val_rm_first_el(pos%rest)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END IF
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_val_rm_el_at
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief removes all the elements from the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list that should be removed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      check more?
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_int_rm_all_el(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: next_node, actual_node
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            actual_node => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (.NOT. ASSOCIATED(actual_node)) EXIT
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               next_node => actual_node%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_int_dealloc_node(actual_node)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               actual_node => next_node
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            NULLIFY (sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_int_rm_all_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief removes all the elements from the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list that should be removed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      check more?
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_real_rm_all_el(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: next_node, actual_node
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            actual_node => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (.NOT. ASSOCIATED(actual_node)) EXIT
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               next_node => actual_node%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_real_dealloc_node(actual_node)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               actual_node => next_node
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            NULLIFY (sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_real_rm_all_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief removes all the elements from the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list that should be removed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      check more?
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_logical_rm_all_el(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: next_node, actual_node
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            actual_node => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (.NOT. ASSOCIATED(actual_node)) EXIT
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               next_node => actual_node%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_logical_dealloc_node(actual_node)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               actual_node => next_node
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            NULLIFY (sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_logical_rm_all_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief removes all the elements from the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list that should be removed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      check more?
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_char_rm_all_el(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: next_node, actual_node
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            actual_node => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (.NOT. ASSOCIATED(actual_node)) EXIT
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               next_node => actual_node%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_char_dealloc_node(actual_node)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               actual_node => next_node
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            NULLIFY (sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_char_rm_all_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief removes all the elements from the list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list that should be removed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      none
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \note
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      check more?
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         SUBROUTINE cp_sll_val_rm_all_el(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: next_node, actual_node
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            actual_node => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               IF (.NOT. ASSOCIATED(actual_node)) EXIT
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               next_node => actual_node%rest
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CALL cp_sll_val_dealloc_node(actual_node)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               actual_node => next_node
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            NULLIFY (sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END SUBROUTINE cp_sll_val_rm_all_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns a newly allocated array with the same contents as
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      the linked list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list to transform in array
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      07.2002 created [fawzi]
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_int_to_array(sll) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            integer, DIMENSION(:), POINTER :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER :: len, i
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            LOGICAL :: ok
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_int_type), POINTER :: iter
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            len = cp_sll_int_get_length(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ALLOCATE (res(len))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            iter => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            DO i = 1, len
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               res(i) =iter%first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ok = cp_sll_int_next(iter)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CPASSERT(ok .OR. i == len)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_int_to_array
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns a newly allocated array with the same contents as
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      the linked list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list to transform in array
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      07.2002 created [fawzi]
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_real_to_array(sll) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            REAL(kind=dp), DIMENSION(:), POINTER :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER :: len, i
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            LOGICAL :: ok
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_real_type), POINTER :: iter
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            len = cp_sll_real_get_length(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ALLOCATE (res(len))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            iter => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            DO i = 1, len
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               res(i) =iter%first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ok = cp_sll_real_next(iter)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CPASSERT(ok .OR. i == len)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_real_to_array
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns a newly allocated array with the same contents as
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      the linked list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list to transform in array
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      07.2002 created [fawzi]
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_logical_to_array(sll) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            logical, DIMENSION(:), POINTER :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER :: len, i
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            LOGICAL :: ok
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_logical_type), POINTER :: iter
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            len = cp_sll_logical_get_length(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ALLOCATE (res(len))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            iter => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            DO i = 1, len
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               res(i) =iter%first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ok = cp_sll_logical_next(iter)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CPASSERT(ok .OR. i == len)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_logical_to_array
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns a newly allocated array with the same contents as
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      the linked list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list to transform in array
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      07.2002 created [fawzi]
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_char_to_array(sll) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            character(len=default_string_length), DIMENSION(:), POINTER :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER :: len, i
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            LOGICAL :: ok
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_char_type), POINTER :: iter
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            len = cp_sll_char_get_length(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ALLOCATE (res(len))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            iter => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            DO i = 1, len
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               res(i) =iter%first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ok = cp_sll_char_next(iter)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CPASSERT(ok .OR. i == len)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_char_to_array
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \brief returns a newly allocated array with the same contents as
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      the linked list
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \param sll the list to transform in array
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \return ...
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \par History
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!>      07.2002 created [fawzi]
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
!> \author Fawzi Mohamed
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
! **************************************************************************************************
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         FUNCTION cp_sll_val_to_array(sll) RESULT(res)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            type(val_p_type), DIMENSION(:), POINTER :: res
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            INTEGER :: len, i
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            LOGICAL :: ok
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            TYPE(cp_sll_val_type), POINTER :: iter
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"

# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            len = cp_sll_val_get_length(sll)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            ALLOCATE (res(len))
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            iter => sll
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            DO i = 1, len
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               res(i) %val=>iter%first_el
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               ok = cp_sll_val_next(iter)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
               CPASSERT(ok .OR. i == len)
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
            END DO
# 29 "/builddir/build/BUILD/cp2k-2024.1/src/input/cp_linked_list_input.F"
         END FUNCTION cp_sll_val_to_array
END MODULE
